@@ -28,6 +28,8 @@
 
 #include <gavl/gavl.h>
 #include <gavl/compression.h>
+#include <gavl/metadata.h>
+
 #include "bgavdefs.h" // This is ugly, but works
 
 #ifdef __cplusplus
@@ -113,7 +115,7 @@ bgav_t * bgav_create();
  * This structure is returned by \ref bgav_get_metadata
  */
 
-typedef struct bgav_metadata_s bgav_metadata_t;
+typedef gavl_metadata_t bgav_metadata_t;
 
 /** \ingroup metadata
  * \brief Get the author (or composer) of the track
@@ -169,6 +171,15 @@ const char * bgav_metadata_get_album(const bgav_metadata_t * metadata);
 BGAV_PUBLIC
 const char * bgav_metadata_get_artist(const bgav_metadata_t * metadata);
 
+/** \ingroup metadata
+ * \brief Get the album artist of this track
+ * \param metadata Metadata container
+ * \returns The album artist in UTF-8 or NULL
+ */
+
+BGAV_PUBLIC
+const char * bgav_metadata_get_albumartist(const bgav_metadata_t * metadata);
+
 /** \ingroup metadata
  * \brief Get the genre this track belongs to
  * \param metadata Metadata container
@@ -300,7 +311,7 @@ void bgav_options_set_read_timeout(bgav_options_t * opt, int timeout);
 
 
 BGAV_PUBLIC
-void bgav_options_set_rtp_port_base(bgav_options_t*b, int p);
+void bgav_options_set_rtp_port_base(bgav_options_t * opt, int p);
 
 /** \ingroup options
  *  \brief Try TCP before UDP
@@ -312,7 +323,7 @@ void bgav_options_set_rtp_port_base(bgav_options_t*b, int p);
  */
 
 BGAV_PUBLIC
-void bgav_options_set_rtp_try_tcp(bgav_options_t*b, int enable);
+void bgav_options_set_rtp_try_tcp(bgav_options_t * opt, int enable);
 
 /** \ingroup options
  *  \brief Set network bandwidth
@@ -616,7 +627,7 @@ void bgav_options_set_dv_datetime(bgav_options_t* opt,
 /** \ingroup options
  *  \brief Shrink factor
  *  \param opt Option container
- *  \param Exponent of the shrink factor
+ *  \param factor Exponent of the shrink factor
  *
  *  This enables downscaling of images while decoding.
  *  Currently only supported for JPEG-2000.
@@ -629,7 +640,7 @@ void bgav_options_set_shrink(bgav_options_t* opt,
 /** \ingroup options
  *  \brief VDPAU acceleration
  *  \param opt Option container
- *  \param 1 to enable vdpau, 0 else
+ *  \param vdpau 1 to enable vdpau, 0 else
  *
  *  Since 1.0.2
  */
@@ -638,6 +649,30 @@ BGAV_PUBLIC
 void bgav_options_set_vdpau(bgav_options_t* opt,
                             int vdpau);
 
+/** \ingroup options
+ *  \brief Dump file headers
+ *  \param opt Option container
+ *  \param enable 1 to dump file headers, 0 else
+ *
+ *  Since 1.1.1
+ */
+
+BGAV_PUBLIC
+void bgav_options_set_dump_headers(bgav_options_t* opt,
+                                   int enable);
+
+/** \ingroup options
+ *  \brief Dump file indices
+ *  \param opt Option container
+ *  \param enable 1 to dump file indices, 0 else
+ *
+ *  Since 1.1.1
+ */
+
+BGAV_PUBLIC
+void bgav_options_set_dump_indices(bgav_options_t* opt,
+                                   int enable);
+
   
 
 /** \ingroup options
@@ -1029,6 +1064,7 @@ int bgav_open_fd(bgav_t * bgav, int fd,
  *  \param priv Private argument for the callbacks
  *  \param filename The filename of the input or NULL if this info is not known.
  *  \param mimetype The mimetype of the input or NULL if this info is not known.
+ *  \param total_bytes File size in bytes or 0 if this info is not known.
  *  \returns 1 on success, 0 else.
  */
 
@@ -1110,6 +1146,9 @@ typedef struct
 
 typedef struct
   {
+  char * name;                       //!< Name (optional)
+  bgav_metadata_t * metadata;        //!< Metadata (optional)
+  
   int num_audio_streams;             //!< Number of logical audio streams
   bgav_edl_stream_t * audio_streams; //!< Logical audio streams
 
@@ -1378,7 +1417,28 @@ BGAV_PUBLIC
 const char * bgav_get_audio_language(bgav_t * bgav, int stream);
 
 /** \ingroup streams
- *  \brief Get the language of an audio stream
+ *  \brief Get the bitrate of an audio stream
+ *  \param bgav A decoder instance
+ *  \param stream Audio stream index (starting with 0)
+ *  \returns Bitrate in bits / sec. 0 means VBR or unknown
+ */
+
+BGAV_PUBLIC
+int bgav_get_audio_bitrate(bgav_t * bgav, int stream);
+
+/** \ingroup streams
+ *  \brief Get the metadata of an audio stream
+ *  \param bgav A decoder instance
+ *  \param stream Audio stream index (starting with 0)
+ *  \returns The metadata for the stream
+ */
+
+BGAV_PUBLIC
+const bgav_metadata_t *
+bgav_get_audio_metadata(bgav_t * bgav, int stream);
+  
+/** \ingroup streams
+ *  \brief Get the language of a subtitle stream
  *  \param bgav A decoder instance
  *  \param stream Subtitle stream index (starting with 0)
  *  \returns A language string.
@@ -1387,6 +1447,29 @@ const char * bgav_get_audio_language(bgav_t * bgav, int stream);
 BGAV_PUBLIC
 const char * bgav_get_subtitle_language(bgav_t * bgav, int stream);
 
+/** \ingroup streams
+ *  \brief Get the metadata of a subtitle stream
+ *  \param bgav A decoder instance
+ *  \param stream Subtitle stream index (starting with 0)
+ *  \returns The metadata for the stream
+ */
+
+BGAV_PUBLIC
+const bgav_metadata_t *
+bgav_get_subtitle_metadata(bgav_t * bgav, int stream);
+
+/** \ingroup streams
+ *  \brief Get the metadata of a video stream
+ *  \param bgav A decoder instance
+ *  \param stream Video stream index (starting with 0)
+ *  \returns The metadata for the stream
+ */
+
+BGAV_PUBLIC
+const bgav_metadata_t *
+bgav_get_video_metadata(bgav_t * bgav, int stream);
+
+  
 /** \ingroup streams
  *  \brief Stream action
  *
