@@ -28,6 +28,8 @@
 #include <libintl.h>
 
 #include <os.h>
+#include <gavl/metatags.h>
+
 
 #define PACKET_PADDING 32
 
@@ -78,11 +80,7 @@ typedef struct bgav_video_format_tracker_s bgav_video_format_tracker_t;
 #include <id3.h>
 #include <yml.h>
 #include <packettimer.h>
-
-#define BGAV_CODING_TYPE_I 'I'
-#define BGAV_CODING_TYPE_P 'P'
-#define BGAV_CODING_TYPE_B 'B'
-#define BGAV_CODING_TYPE_D 'D' /* Unsupported */
+#include <frametype.h>
 
 /* subsequent calls of read() or peek() will return the next packet */
 typedef bgav_packet_t * (*bgav_get_packet_callback)(void * data);
@@ -100,6 +98,7 @@ typedef struct
 void bgav_packet_source_copy(bgav_packet_source_t * dst,
                              const bgav_packet_source_t * src);
 
+#if 0
 /* Metadata */
 
 struct bgav_metadata_s
@@ -112,6 +111,7 @@ struct bgav_metadata_s
   char * artist;
   char * genre;
   char * date;
+  char * albumartist;
   int track;
   };
 
@@ -125,7 +125,10 @@ void bgav_metadata_merge2(bgav_metadata_t * dst,
                           bgav_metadata_t * src);
 
 void bgav_metadata_free(bgav_metadata_t*);
+void bgav_metadata_copy(bgav_metadata_t * dst,
+                        bgav_metadata_t * src);
 
+#endif
 /* Decoder structures */
 
 struct bgav_audio_decoder_s
@@ -210,8 +213,41 @@ struct bgav_subtitle_overlay_decoder_context_s
   const bgav_subtitle_overlay_decoder_t * decoder;
   };
 
+/* Palette support */
+
+typedef struct
+  {
+  uint16_t r;
+  uint16_t g;
+  uint16_t b;
+  uint16_t a;
+  } bgav_palette_entry_t;
+
+/* These map a palette entry to a gavl format frame */
+
+#define BGAV_PALETTE_2_RGB24(pal, dst) \
+dst[0] = pal.r >> 8;\
+dst[1] = pal.g >> 8;\
+dst[2] = pal.b >> 8;
+
+#define BGAV_PALETTE_2_RGBA32(pal, dst) \
+dst[0] = pal.r >> 8;\
+dst[1] = pal.g >> 8;\
+dst[2] = pal.b >> 8;\
+dst[3] = pal.a >> 8;
+
+#define BGAV_PALETTE_2_BGR24(pal, dst) \
+dst[2] = pal.r >> 8;\
+dst[1] = pal.g >> 8;\
+dst[0] = pal.b >> 8;
+
 /* Packet */
 
+#define BGAV_CODING_TYPE_I 'I'
+#define BGAV_CODING_TYPE_P 'P'
+#define BGAV_CODING_TYPE_B 'B'
+#define BGAV_CODING_TYPE_D 'D' /* Unsupported */
+
 #define PACKET_FLAG_KEY    (1<<8)
 #define PACKET_FLAG_SKIP   (1<<9)
 #define PACKET_FLAG_LAST   (1<<10)
@@ -263,6 +299,10 @@ struct bgav_packet_s
   struct bgav_packet_s * next;
 
   uint32_t flags;
+
+  /* Palette data */
+  int palette_size;
+  bgav_palette_entry_t * palette;
   };
 
 /* packet.c */
@@ -277,6 +317,12 @@ void bgav_packet_swap_data(bgav_packet_t * p1, bgav_packet_t * p2);
 void bgav_packet_pad(bgav_packet_t * p);
 void bgav_packet_reset(bgav_packet_t * p);
 
+void bgav_packet_alloc_palette(bgav_packet_t * p, int size);
+void bgav_packet_free_palette(bgav_packet_t * p);
+void bgav_packet_copy_metadata(bgav_packet_t * dst,
+                               const bgav_packet_t * src);
+
+
 // void bgav_packet_done_write(bgav_packet_t *);
 // void bgav_packet_done_read(bgav_packet_t *);
 
@@ -322,33 +368,6 @@ void bgav_packet_pool_put(bgav_packet_pool_t * pp,
 void bgav_packet_pool_destroy(bgav_packet_pool_t*);
 
 
-/* Palette support */
-
-typedef struct
-  {
-  uint16_t r;
-  uint16_t g;
-  uint16_t b;
-  uint16_t a;
-  } bgav_palette_entry_t;
-
-/* These map a palette entry to a gavl format frame */
-
-#define BGAV_PALETTE_2_RGB24(pal, dst) \
-dst[0] = pal.r >> 8;\
-dst[1] = pal.g >> 8;\
-dst[2] = pal.b >> 8;
-
-#define BGAV_PALETTE_2_RGBA32(pal, dst) \
-dst[0] = pal.r >> 8;\
-dst[1] = pal.g >> 8;\
-dst[2] = pal.b >> 8;\
-dst[3] = pal.a >> 8;
-
-#define BGAV_PALETTE_2_BGR24(pal, dst) \
-dst[2] = pal.r >> 8;\
-dst[1] = pal.g >> 8;\
-dst[0] = pal.b >> 8;
 
 /* Stream types
    (changing numeric values alters the index file format) */
@@ -376,34 +395,34 @@ typedef enum
 #define STREAM_PARSE_FRAME        (1<<2) /* Frame aligned but no keyframes */
 #define STREAM_B_FRAMES           (1<<3)
 #define STREAM_WRONG_B_TIMESTAMPS (1<<4)
-#define STREAM_HEADER_FROM_PARSER (1<<5)
-#define STREAM_STILL_MODE         (1<<6) /* Still image mode */
-#define STREAM_STILL_SHOWN        (1<<7) /* Still image already shown */
-#define STREAM_EOF_D              (1<<8) /* End of file at demuxer */
-#define STREAM_EOF_C              (1<<9) /* End of file at codec */
+#define STREAM_STILL_MODE         (1<<6)  /* Still image mode          */
+#define STREAM_STILL_SHOWN        (1<<7)  /* Still image already shown */
+#define STREAM_EOF_D              (1<<8)  /* End of file at demuxer    */
+#define STREAM_EOF_C              (1<<9)  /* End of file at codec      */
+#define STREAM_NEED_FRAMETYPES    (1<<10) /* Need frame types          */
 
 /* Stream can have a nonzero start time */
-#define STREAM_NEED_START_TIME   (1<<10)
+#define STREAM_NEED_START_TIME   (1<<11)
 
 /* Picture is available for immediate output */
-#define STREAM_HAVE_PICTURE       (1<<11)
+#define STREAM_HAVE_PICTURE       (1<<12)
 
 /* Already got the format from the parser */
-#define STREAM_PARSE_HAVE_FORMAT  (1<<12)
+#define STREAM_PARSE_HAVE_FORMAT  (1<<13)
 
-#define STREAM_RAW_PACKETS        (1<<13)
-#define STREAM_FIELD_PICTURES     (1<<14)
-#define STREAM_FILTER_PACKETS     (1<<15)
-#define STREAM_SBR                (1<<16)
-#define STREAM_NO_DURATIONS       (1<<17)
-#define STREAM_HAS_DTS            (1<<18)
-#define STREAM_B_PYRAMID          (1<<19)
+#define STREAM_RAW_PACKETS        (1<<14)
+#define STREAM_FIELD_PICTURES     (1<<15)
+#define STREAM_FILTER_PACKETS     (1<<16)
+#define STREAM_SBR                (1<<17)
+#define STREAM_NO_DURATIONS       (1<<18)
+#define STREAM_HAS_DTS            (1<<19)
+#define STREAM_B_PYRAMID          (1<<20)
 
 
 /* Stream could not get exact compression info from the
  * demuxer
  */
-#define STREAM_NEED_EXACT_COMPRESSION (1<<12)
+#define STREAM_NEED_EXACT_COMPRESSION (1<<21)
 
 #define STREAM_SET_SYNC(s, t)  (s)->sync_time = t
 #define STREAM_GET_SYNC(s)     (s)->sync_time
@@ -472,14 +491,14 @@ struct bgav_stream_s
 
   bgav_packet_t * packet;
   int             packet_seq;
- 
-  char * description;   /* Technical description of the stream */
-  char * info;          /* Info about the stream (e.g. Directors comments) */
   
   /* Language (ISO 639-3 B code) */
 
-  char language[4];
-    
+  //  char language[4];
+
+  gavl_metadata_t m;
+  
+  
   /*
    *  Sometimes, the bitrates important for codecs 
    *  and the bitrates set in the container format
@@ -528,7 +547,11 @@ struct bgav_stream_s
   bgav_packet_pool_t * pp;  /* Where to put consumed
                                packets for later use */
 
-  bgav_packet_timer_t * pt; /* Correct timestamps from broken containers */
+  /* Correct timestamps from broken containers */
+  bgav_packet_timer_t * pt;
+  
+  /* Detect frametypes from broken containers */
+  bgav_frametype_detector_t * fd; 
   
   union
     {
@@ -566,21 +589,10 @@ struct bgav_stream_s
       int depth;
       int planes;     /* For M$ formats only */
       int image_size; /* For M$ formats only */
-      int flip_y;
       
       bgav_video_decoder_context_t * decoder;
       gavl_video_format_t format;
-      int palette_size;
-      bgav_palette_entry_t * palette;
-      int palette_changed;
-      
-/* How to get frame times and durations */
-#define BGAV_FRAMETIME_CONSTANT  0 /* Strictly constant framerate */
-#define BGAV_FRAMETIME_PACKET    1 /* Packets contain complete frames and have correct duration fields */
-#define BGAV_FRAMETIME_PTS       2 /* Packets contain complete frames but have only correct PTS field  */
-#define BGAV_FRAMETIME_CODEC     3 /* PTS and duration can only be generated by the codec              */
-      
-      int frametime_mode;
+      //      int palette_changed;
       
       bgav_video_parser_t * parser;
       bgav_keyframe_table_t * kft;
@@ -588,6 +600,14 @@ struct bgav_stream_s
       int max_ref_frames; /* Needed for VDPAU */
       
       bgav_video_format_tracker_t * ft;
+
+      struct
+        {
+        bgav_palette_entry_t * entries;
+        int size;
+        int sent;
+        } pal;
+      
       } video;
     struct
       {
@@ -617,6 +637,8 @@ void bgav_stream_create_packet_buffer(bgav_stream_t * stream);
 void bgav_stream_init(bgav_stream_t * stream, const bgav_options_t * opt);
 void bgav_stream_free(bgav_stream_t * stream);
 void bgav_stream_dump(bgav_stream_t * s);
+void bgav_stream_set_extradata(bgav_stream_t * s, const uint8_t * data, int len);
+
 
 /* Top level packet functions */
 bgav_packet_t * bgav_stream_get_packet_write(bgav_stream_t * s);
@@ -852,6 +874,9 @@ struct bgav_options_s
   int threads;
 
   int log_level;
+
+  int dump_headers;
+  int dump_indices;
   
   /* Callbacks */
 
@@ -875,7 +900,7 @@ struct bgav_options_s
   
   bgav_index_callback index_callback;
   void * index_callback_data;
-
+  
   };
 
 BGAV_PUBLIC void bgav_options_set_defaults(bgav_options_t*opt);
@@ -900,7 +925,7 @@ struct bgav_input_s
 
   /* Some inputs support multiple tracks */
 
-  void    (*select_track)(bgav_input_context_t*, int);
+  int    (*select_track)(bgav_input_context_t*, int);
 
   /* Alternate API: Sector based read and seek access */
 
@@ -1344,7 +1369,6 @@ struct bgav_demuxer_context_s
   bgav_input_context_t * input;
   
   bgav_track_table_t * tt;
-  char * stream_description;
 
   int packet_size; /* Optional, if it's fixed */
   
@@ -1679,9 +1703,9 @@ int bgav_check_file_read(const char * filename);
 
 /* Read a single line from a filedescriptor */
 
-int bgav_read_line_fd(int fd, char ** ret, int * ret_alloc, int milliseconds);
+int bgav_read_line_fd(const bgav_options_t * opt, int fd, char ** ret, int * ret_alloc, int milliseconds);
 
-int bgav_read_data_fd(int fd, uint8_t * ret, int size, int milliseconds);
+int bgav_read_data_fd(const bgav_options_t * opt, int fd, uint8_t * ret, int size, int milliseconds);
 
 /* tcp.c */
 
@@ -1899,6 +1923,7 @@ typedef struct
 
 void bgav_bytebuffer_append(bgav_bytebuffer_t * b, bgav_packet_t * p, int padding);
 void bgav_bytebuffer_append_data(bgav_bytebuffer_t * b, uint8_t * data, int len, int padding);
+int bgav_bytebuffer_append_read(bgav_bytebuffer_t * b, bgav_input_context_t * input, int len, int padding);
 void bgav_bytebuffer_remove(bgav_bytebuffer_t * b, int bytes);
 void bgav_bytebuffer_free(bgav_bytebuffer_t * b);
 void bgav_bytebuffer_flush(bgav_bytebuffer_t * b);
