@@ -280,7 +280,7 @@ static void cleanup_stream_asf(bgav_stream_t * s)
   asf_audio_stream_t * as;
   if(s->type == BGAV_STREAM_AUDIO)
     {
-    as = (asf_audio_stream_t*)s->priv;
+    as = s->priv;
     
     if(as->scramble_buffer)
       free(as->scramble_buffer);
@@ -331,7 +331,7 @@ static int read_bitrate_properties(bgav_demuxer_context_t * ctx)
   uint16_t i_tmp;
   int i;
   asf_t * asf;
-  asf = (asf_t*)(ctx->priv);
+  asf = ctx->priv;
 
   if(!bgav_input_read_16_le(ctx->input, &i_tmp))
     return 0;
@@ -362,7 +362,7 @@ static void update_stream_bitrates(bgav_demuxer_context_t * ctx)
   bgav_stream_t * stream;
     
   asf_t * asf;
-  asf = (asf_t*)(ctx->priv);
+  asf = ctx->priv;
 
   for(i = 0; i < asf->num_stream_bitrates; i++)
     {
@@ -378,11 +378,10 @@ static void update_stream_bitrates(bgav_demuxer_context_t * ctx)
 
 static int read_metadata(bgav_demuxer_context_t * ctx)
   {
-  char * str = (char*)0;
+  char * str = NULL;
   uint16_t len1, len2, len3, len4, len5;
   int str_len;
-  bgav_charset_converter_t * cnv =
-    (bgav_charset_converter_t*)0;
+  bgav_charset_converter_t * cnv = NULL;
   
   if(!bgav_input_read_16_le(ctx->input, &len1) ||
      !bgav_input_read_16_le(ctx->input, &len2) ||
@@ -411,8 +410,9 @@ static int read_metadata(bgav_demuxer_context_t * ctx)
     {
     if(bgav_input_read_data(ctx->input, (uint8_t*)str, len1) < len1)
       goto fail;
-    ctx->tt->cur->metadata.title = bgav_convert_string(cnv, str, len1,
-                                              NULL);
+    gavl_metadata_set_nocpy(&ctx->tt->cur->metadata,
+                            GAVL_META_TITLE, bgav_convert_string(cnv, str, len1,
+                                                                 NULL));
     }
 
   /* Author */
@@ -421,8 +421,9 @@ static int read_metadata(bgav_demuxer_context_t * ctx)
     {
     if(bgav_input_read_data(ctx->input, (uint8_t*)str, len2) < len2)
       goto fail;
-    ctx->tt->cur->metadata.author =
-      bgav_convert_string(cnv, str, len2, NULL);
+    gavl_metadata_set_nocpy(&ctx->tt->cur->metadata,
+                            GAVL_META_AUTHOR, bgav_convert_string(cnv, str, len2,
+                                                                  NULL));
     }
 
   /* Copyright */
@@ -431,8 +432,9 @@ static int read_metadata(bgav_demuxer_context_t * ctx)
     {
     if(bgav_input_read_data(ctx->input, (uint8_t*)str, len3) < len3)
       goto fail;
-    ctx->tt->cur->metadata.copyright =
-      bgav_convert_string(cnv, str, len3, NULL);
+    gavl_metadata_set_nocpy(&ctx->tt->cur->metadata,
+                            GAVL_META_COPYRIGHT, bgav_convert_string(cnv, str, len3,
+                                                                     NULL));
     }
 
   /* Comment */
@@ -441,8 +443,9 @@ static int read_metadata(bgav_demuxer_context_t * ctx)
     {
     if(bgav_input_read_data(ctx->input, (uint8_t*)str, len4) < len4)
       goto fail;
-    ctx->tt->cur->metadata.comment =
-      bgav_convert_string(cnv, str, len4, NULL);
+    gavl_metadata_set_nocpy(&ctx->tt->cur->metadata,
+                            GAVL_META_COMMENT, bgav_convert_string(cnv, str, len4,
+                                                                   NULL));
     }
 
   /* Unknown stuff */
@@ -469,7 +472,7 @@ static int open_asf(bgav_demuxer_context_t * ctx)
   
   bgav_BITMAPINFOHEADER_t bh;
   bgav_WAVEFORMAT_t     wf;
-  uint8_t * buf = (uint8_t*)0;
+  uint8_t * buf = NULL;
   uint8_t * pos;
   uint16_t stream_number;
   int buf_size = 0;
@@ -628,7 +631,6 @@ static int open_asf(bgav_demuxer_context_t * ctx)
       else if(bgav_GUID_equal(&guid, &guid_video_media))
         {
         bgav_vs = bgav_track_add_video_stream(ctx->tt->cur, ctx->opt);
-        bgav_vs->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
         bgav_vs->cleanup = cleanup_stream_asf;
         bgav_vs->stream_id = stream_number;
 
@@ -665,12 +667,8 @@ static int open_asf(bgav_demuxer_context_t * ctx)
         
         //        gavl_video_format_dump(&bgav_vs->format);
         if(pos - buf < type_specific_size)
-          {
-          bgav_vs->ext_size = type_specific_size - (pos - buf);
-          bgav_vs->ext_data = malloc(bgav_vs->ext_size);
-          memcpy(bgav_vs->ext_data, pos, bgav_vs->ext_size);
-          }
-
+          bgav_stream_set_extradata(bgav_vs, pos, type_specific_size - (pos - buf));
+        
         /* Stream specific (not known for now) */
 
         if(stream_specific_size)
@@ -739,8 +737,9 @@ static int open_asf(bgav_demuxer_context_t * ctx)
   if(ctx->input->input->seek_byte && asf->hdr.packets_count)
     ctx->flags |= (BGAV_DEMUXER_CAN_SEEK | BGAV_DEMUXER_SEEK_ITERATIVE);
   
-  ctx->stream_description = bgav_sprintf("Windows media format (ASF)");
-  
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "ASF");
+
   return 1;
   fail:
   if(buf)
@@ -975,7 +974,7 @@ static void add_packet(bgav_demuxer_context_t * ctx,
   {
   bgav_stream_t * s;
   asf_audio_stream_t * as;
-  asf_t * asf = (asf_t *)(ctx->priv);
+  asf_t * asf = ctx->priv;
 
   s = bgav_track_find_stream(ctx, id);
   
@@ -998,7 +997,7 @@ static void add_packet(bgav_demuxer_context_t * ctx,
       {
       if(s->type == BGAV_STREAM_AUDIO)
         {
-        as = (asf_audio_stream_t*)(s->priv);
+        as = s->priv;
         if((as->descramble_w > 1) && (as->descramble_h > 1))
           asf_descramble(as,
                          s->packet->data,
@@ -1017,7 +1016,7 @@ static void add_packet(bgav_demuxer_context_t * ctx,
         }
 #endif
       bgav_stream_done_packet_write(s, s->packet);
-      s->packet = (bgav_packet_t*)0;
+      s->packet = NULL;
       }
     else
       {
@@ -1070,7 +1069,7 @@ static int next_packet_asf(bgav_demuxer_context_t * ctx)
   uint8_t * data_ptr;
   asf_packet_header_t pkt_hdr;
   asf_segment_header_t seg_hdr;
-  asf = (asf_t*)(ctx->priv);
+  asf = ctx->priv;
 
   //  if(ctx->input->position >= asf->data_start + asf->data_size)
   //    return 0;
@@ -1137,7 +1136,7 @@ static int next_packet_asf(bgav_demuxer_context_t * ctx)
 static void seek_asf(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   {
   int64_t filepos;
-  asf_t * asf = (asf_t*)(ctx->priv);
+  asf_t * asf = ctx->priv;
   /* Reset all the streams */
   
   asf->packets_read =
@@ -1163,7 +1162,7 @@ static void seek_asf(bgav_demuxer_context_t * ctx, int64_t time, int scale)
 
 static void close_asf(bgav_demuxer_context_t * ctx)
   {
-  asf_t * asf = (asf_t *)(ctx->priv);
+  asf_t * asf = ctx->priv;
 
   if(asf->packet_buffer)
     free(asf->packet_buffer);
@@ -1176,7 +1175,7 @@ static void close_asf(bgav_demuxer_context_t * ctx)
 
 static int select_track_asf(bgav_demuxer_context_t * ctx, int track)
   {
-  asf_t * asf = (asf_t *)(ctx->priv);
+  asf_t * asf = ctx->priv;
   asf->packets_read = 0;
   return 1;
   }
