@@ -32,9 +32,8 @@
 /* Define the variables below to get a detailed file dump
    on each open call */
 
-// #define DUMP_HEADERS
-// #define DUMP_INDICES
 // #define DUMP_AUDIO_TYPE
+// #define DUMP_CHUNK_HEADERS
 
 /* AVI Flags */
 
@@ -396,7 +395,6 @@ static int read_chunk_header(bgav_input_context_t * input,
     bgav_input_read_32_le(input, &chunk->ckSize);
   }
 
-#ifdef DUMP_HEADERS
 static void dump_chunk_header(chunk_header_t * chunk)
   {
   bgav_dprintf("chunk header:\n");
@@ -404,7 +402,6 @@ static void dump_chunk_header(chunk_header_t * chunk)
   bgav_dump_fourcc(chunk->ckID);
   bgav_dprintf("\n  ckSize %d\n", chunk->ckSize);
   }
-#endif
 
 #ifdef DUMP_AUDIO_TYPE
 static int do_msg = 1;
@@ -419,7 +416,7 @@ static void add_index_packet(bgav_superindex_t * si, bgav_stream_t * stream,
   
   if(stream->type == BGAV_STREAM_AUDIO)
     {
-    avi_as = (audio_priv_t*)(stream->priv);
+    avi_as = stream->priv;
     samplerate = stream->data.audio.format.samplerate;
 
     bgav_superindex_add_packet(si,
@@ -507,7 +504,6 @@ static void add_index_packet(bgav_superindex_t * si, bgav_stream_t * stream,
     else /* If we have zero size, the framerate will be nonconstant */
       {
       stream->data.video.format.framerate_mode = GAVL_FRAMERATE_VARIABLE;
-      stream->data.video.frametime_mode = BGAV_FRAMETIME_PACKET;
       }
     stream->duration += stream->data.video.format.frame_duration;
     }
@@ -526,7 +522,6 @@ static int read_riff_header(bgav_input_context_t * input,
   }
 
 
-#ifdef DUMP_HEADERS
 static void dump_avih(avih_t * h)
   {
   bgav_dprintf("avih:\n");
@@ -544,7 +539,6 @@ static void dump_avih(avih_t * h)
   bgav_dprintf("  dwRate: %d\n",                h->dwRate);
   bgav_dprintf("  dwLength: %d\n",              h->dwLength);
   }
-#endif
 
 static int read_avih(bgav_input_context_t* input,
               avih_t * ret, chunk_header_t * ch)
@@ -572,9 +566,8 @@ static int read_avih(bgav_input_context_t* input,
     {
     bgav_input_skip(input, PADD(ch->ckSize) - (input->position - start_pos));
     }
-#ifdef DUMP_HEADERS
-  dump_avih(ret);
-#endif
+  if(input->opt->dump_headers)
+    dump_avih(ret);
   return result;
   }
 
@@ -586,7 +579,6 @@ static void free_idx1(idx1_t * idx1)
     free(idx1->entries);
   }
 
-#ifdef DUMP_INDICES
 static void dump_idx1(idx1_t * idx1)
   {
   int i;
@@ -600,7 +592,6 @@ static void dump_idx1(idx1_t * idx1)
     bgav_dprintf(" Size: %d\n", idx1->entries[i].dwChunkLength);
     }
   }
-#endif
 
 static int probe_idx1(bgav_input_context_t * input)
   {
@@ -637,7 +628,6 @@ static int read_idx1(bgav_input_context_t * input, idx1_t * ret)
 /* strh */
 
 
-#ifdef DUMP_HEADERS
 static void dump_strh(strh_t * ret)
   {
   bgav_dprintf("strh\n  fccType: ");
@@ -660,7 +650,6 @@ static void dump_strh(strh_t * ret)
   bgav_dprintf("  dwQuality: %d (%08x)\n", ret->dwQuality, ret->dwQuality);
   bgav_dprintf("  dwSampleSize: %d (%08x)\n", ret->dwSampleSize, ret->dwSampleSize);
   }
-#endif
 
 static int read_strh(bgav_input_context_t * input, strh_t * ret,
                      chunk_header_t * ch)
@@ -688,9 +677,8 @@ static int read_strh(bgav_input_context_t * input, strh_t * ret,
     {
     bgav_input_skip(input, PADD(ch->ckSize) - (input->position - start_pos));
     }
-#ifdef DUMP_HEADERS
-  dump_strh(ret);
-#endif
+  if(input->opt->dump_headers)
+    dump_strh(ret);
   return result;
   }
 
@@ -699,13 +687,11 @@ static int read_strh(bgav_input_context_t * input, strh_t * ret,
 /* dmlh */
 
 
-#ifdef DUMP_HEADERS
 static void dump_dmlh(dmlh_t * dmlh)
   {
   bgav_dprintf("dmlh:\n");
   bgav_dprintf("  dwTotalFrames: %d\n", dmlh->dwTotalFrames);
   }
-#endif
 
 static int read_dmlh(bgav_input_context_t * input, dmlh_t * ret,
                      chunk_header_t * ch)
@@ -724,14 +710,12 @@ static int read_dmlh(bgav_input_context_t * input, dmlh_t * ret,
   }
 /* odml */
 
-#ifdef DUMP_HEADERS
 static void dump_odml(odml_t * odml)
   {
   bgav_dprintf("odml:\n");
   if(odml->has_dmlh)
     dump_dmlh(&odml->dmlh);
   }
-#endif
 
 static int read_odml(bgav_input_context_t * input, odml_t * ret,
                      chunk_header_t * ch)
@@ -767,10 +751,8 @@ static int read_odml(bgav_input_context_t * input, odml_t * ret,
     {
     bgav_input_skip(input, ch->ckSize - 4 - input->position - start_pos);
     }
-#ifdef DUMP_HEADERS
-  dump_odml(ret);
-#endif
-
+  if(input->opt->dump_headers)
+    dump_odml(ret);
   return 1;
   }
 
@@ -871,7 +853,6 @@ static int read_indx(bgav_input_context_t * input, indx_t * ret,
   return 1;
   }
 
-#ifdef DUMP_INDICES
 static void dump_indx(indx_t * indx)
   {
   int i;
@@ -942,7 +923,6 @@ static void dump_indx(indx_t * indx)
       break;
     }
   }
-#endif
 
 static void free_indx(indx_t * indx)
   {
@@ -981,7 +961,7 @@ static void indx_build_superindex(bgav_demuxer_context_t * ctx)
   video_priv_t * avi_vs;
   uint32_t size = 0, test_size;
 
-  avi_priv_t * priv = (avi_priv_t *)(ctx->priv);
+  avi_priv_t * priv = ctx->priv;
   
   struct
     {
@@ -1001,7 +981,7 @@ static void indx_build_superindex(bgav_demuxer_context_t * ctx)
     {
     for(i = 0; i < ctx->tt->cur->num_audio_streams; i++)
       {
-      avi_as = (audio_priv_t *)(ctx->tt->cur->audio_streams[i].priv);
+      avi_as = ctx->tt->cur->audio_streams[i].priv;
       
       if(!avi_as->has_indx)
         return;
@@ -1013,7 +993,7 @@ static void indx_build_superindex(bgav_demuxer_context_t * ctx)
     }
   for(i = 0; i < ctx->tt->cur->num_video_streams; i++)
     {
-    avi_vs = (video_priv_t *)(ctx->tt->cur->video_streams[i].priv);
+    avi_vs = ctx->tt->cur->video_streams[i].priv;
     ctx->tt->cur->video_streams[i].duration = 0;
     if(!avi_vs->has_indx)
       return;
@@ -1035,7 +1015,7 @@ static void indx_build_superindex(bgav_demuxer_context_t * ctx)
     {
     streams[i].s = &ctx->tt->cur->audio_streams[i];
     
-    avi_as = (audio_priv_t *)(streams[i].s->priv);
+    avi_as = streams[i].s->priv;
 
     streams[i].indx = &avi_as->indx;
     
@@ -1059,7 +1039,7 @@ static void indx_build_superindex(bgav_demuxer_context_t * ctx)
     {
     streams[i].s = &ctx->tt->cur->video_streams[i-num_audio_streams];
 
-    avi_vs = (video_priv_t *)(streams[i].s->priv);
+    avi_vs = streams[i].s->priv;
 
     streams[i].indx = &avi_vs->indx;
     
@@ -1172,7 +1152,7 @@ static void cleanup_stream_avi(bgav_stream_t * s)
   if(s->type == BGAV_STREAM_AUDIO)
     {
     audio_priv_t * avi_as;
-    avi_as = (audio_priv_t*)(s->priv);
+    avi_as = s->priv;
     if(avi_as)
       {
       if(avi_as->has_indx)
@@ -1183,10 +1163,8 @@ static void cleanup_stream_avi(bgav_stream_t * s)
   else if(s->type == BGAV_STREAM_VIDEO)
     {
     video_priv_t * avi_vs;
-    if(s->data.video.palette_size)
-      free(s->data.video.palette);
-
-    avi_vs = (video_priv_t*)(s->priv);
+    
+    avi_vs = s->priv;
 
     if(avi_vs)
       {
@@ -1226,9 +1204,8 @@ static int init_audio_stream(bgav_demuxer_context_t * ctx,
         pos = buf;
         bgav_WAVEFORMAT_read(&wf, buf, ch->ckSize);
         bgav_WAVEFORMAT_get_format(&wf, bg_as);
-#ifdef DUMP_HEADERS
-        bgav_WAVEFORMAT_dump(&wf);
-#endif
+        if(ctx->opt->dump_headers)
+          bgav_WAVEFORMAT_dump(&wf);
         bgav_WAVEFORMAT_free(&wf);
         //        bg_as->fourcc = BGAV_WAVID_2_FOURCC(wf.wFormatTag);
         //        if(!bg_as->data.audio.bits_per_sample)
@@ -1251,9 +1228,8 @@ static int init_audio_stream(bgav_demuxer_context_t * ctx,
       case ID_INDX:
         if(!read_indx(ctx->input, &avi_as->indx, ch))
           return 0;
-#ifdef DUMP_INDICES
-        dump_indx(&avi_as->indx);
-#endif
+        if(ctx->opt->dump_indices)
+          dump_indx(&avi_as->indx);
         avi_as->has_indx = 1;
         break;
       default:
@@ -1303,34 +1279,29 @@ static int init_video_stream(bgav_demuxer_context_t * ctx,
         pos = buf;
         bgav_BITMAPINFOHEADER_read(&bh, &pos);
         bgav_BITMAPINFOHEADER_get_format(&bh, bg_vs);
-#ifdef DUMP_HEADERS
-        bgav_BITMAPINFOHEADER_dump(&bh);
-#endif
-
+        if(ctx->opt->dump_headers)
+          bgav_BITMAPINFOHEADER_dump(&bh);
+        
         /* We don't add extradata if the fourcc is MJPG */
         /* This lets us play blender AVIs. */
                 
         if((ch->ckSize > 40) && (bg_vs->fourcc != BGAV_MK_FOURCC('M','J','P','G')))
-          {
-          bg_vs->ext_size = ch->ckSize - 40;
-          bg_vs->ext_data = calloc(bg_vs->ext_size + 16, 1);
-          memcpy(bg_vs->ext_data, pos, bg_vs->ext_size);
-          }
-
+          bgav_stream_set_extradata(bg_vs, pos, ch->ckSize - 40);
+        
         /* Add palette if depth <= 8 */
 
         
         if((bh.biBitCount <= 8) && (bh.biClrUsed))
           {
-          bg_vs->data.video.palette_size = (ch->ckSize - 40) / 4;
-          bg_vs->data.video.palette      =
-            malloc(bg_vs->data.video.palette_size * sizeof(*(bg_vs->data.video.palette)));
-          for(i = 0; i < bg_vs->data.video.palette_size; i++)
+          bg_vs->data.video.pal.size = (ch->ckSize - 40) / 4;
+          bg_vs->data.video.pal.entries =
+            malloc(bg_vs->data.video.pal.size * sizeof(*(bg_vs->data.video.pal.entries)));
+          for(i = 0; i < bg_vs->data.video.pal.size; i++)
             {
-            bg_vs->data.video.palette[i].b = BYTE_2_COLOR(pos[4*i]);
-            bg_vs->data.video.palette[i].g = BYTE_2_COLOR(pos[4*i+1]);
-            bg_vs->data.video.palette[i].r = BYTE_2_COLOR(pos[4*i+2]);
-            bg_vs->data.video.palette[i].a = 0xffff;
+            bg_vs->data.video.pal.entries[i].b = BYTE_2_COLOR(pos[4*i]);
+            bg_vs->data.video.pal.entries[i].g = BYTE_2_COLOR(pos[4*i+1]);
+            bg_vs->data.video.pal.entries[i].r = BYTE_2_COLOR(pos[4*i+2]);
+            bg_vs->data.video.pal.entries[i].a = 0xffff;
             }
           }
 
@@ -1349,10 +1320,8 @@ static int init_video_stream(bgav_demuxer_context_t * ctx,
       case ID_INDX:
         if(!read_indx(ctx->input, &avi_vs->indx, ch))
           return 0;
-
-#ifdef DUMP_INDICES
-        dump_indx(&avi_vs->indx);
-#endif
+        if(ctx->opt->dump_indices)
+          dump_indx(&avi_vs->indx);
         avi_vs->has_indx = 1;
         break;
       default:
@@ -1402,12 +1371,12 @@ static int process_packet_iavs(bgav_demuxer_context_t * ctx, int64_t position)
   {
   int do_init = 0;
   bgav_stream_t * as, *vs;
-  bgav_packet_t * ap = (bgav_packet_t *)0;
-  bgav_packet_t * vp = (bgav_packet_t *)0;
+  bgav_packet_t * ap = NULL;
+  bgav_packet_t * vp = NULL;
   uint8_t header[DV_HEADER_SIZE];
   avi_priv_t * priv;
   
-  priv = (avi_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
   if(!priv->dv_dec)
     {
@@ -1483,7 +1452,7 @@ static void seek_iavs(bgav_demuxer_context_t * ctx, gavl_time_t time,
                       int scale)
   {
   avi_priv_t * priv;
-  priv = (avi_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
   bgav_superindex_seek(ctx->si,
                        ctx->tt->cur->video_streams,
@@ -1539,7 +1508,7 @@ static void process_packet_iavs_stream(bgav_stream_t * s, bgav_packet_t * p)
   bgav_stream_t * as;
   bgav_stream_t * vs;
   
-  priv = (avi_priv_t*)(s->demuxer->priv);
+  priv = s->demuxer->priv;
   
   if(!priv->dv_dec)
     {
@@ -1617,7 +1586,7 @@ static int init_iavs_stream(bgav_demuxer_context_t * ctx,
   video_priv_t * video_priv;
   bgav_stream_t * bg_vs;
   bgav_stream_t * bg_as;
-  avi_priv_t * priv = (avi_priv_t*)(ctx->priv);
+  avi_priv_t * priv = ctx->priv;
 
   priv->has_iavs = 1;
   
@@ -1667,10 +1636,8 @@ static int init_iavs_stream(bgav_demuxer_context_t * ctx,
       case ID_INDX:
         if(!read_indx(ctx->input, &video_priv->indx, ch))
           return 0;
-
-#ifdef DUMP_INDICES
-        dump_indx(&video_priv->indx);
-#endif
+        if(ctx->opt->dump_indices)
+          dump_indx(&video_priv->indx);
         video_priv->has_indx = 1;
         break;
       default:
@@ -1732,13 +1699,13 @@ static void idx1_build_superindex(bgav_demuxer_context_t * ctx)
   int64_t base_offset;
   int first_pos;
   
-  avi = (avi_priv_t*)(ctx->priv);
+  avi = ctx->priv;
 
   /* Reset timestamps */
   
   for(i = 0; i < ctx->tt->cur->num_audio_streams; i++)
     {
-    avi_as = (audio_priv_t*)(ctx->tt->cur->audio_streams[i].priv);
+    avi_as = ctx->tt->cur->audio_streams[i].priv;
     avi_as->total_bytes = 0;
     avi_as->total_blocks = 0;
     ctx->tt->cur->audio_streams[i].duration = 0;
@@ -1746,7 +1713,7 @@ static void idx1_build_superindex(bgav_demuxer_context_t * ctx)
 
   for(i = 0; i < ctx->tt->cur->num_video_streams; i++)
     {
-    avi_vs = (video_priv_t*)(ctx->tt->cur->video_streams[i].priv);
+    avi_vs = ctx->tt->cur->video_streams[i].priv;
     ctx->tt->cur->video_streams[i].duration = 0;
     }
 
@@ -1956,10 +1923,11 @@ static int open_avi(bgav_demuxer_context_t * ctx)
     if(!read_chunk_header(ctx->input, &ch))
       goto fail;
     }
-#ifdef DUMP_HEADERS
-  bgav_dprintf("movi:\n");
-  dump_chunk_header(&ch);
-#endif
+  if(ctx->opt->dump_headers)
+    {
+    bgav_dprintf("movi:\n");
+    dump_chunk_header(&ch);
+    }
   ctx->data_start = ctx->input->position;
   ctx->flags |= BGAV_DEMUXER_HAS_DATA_START;
 
@@ -1978,9 +1946,8 @@ static int open_avi(bgav_demuxer_context_t * ctx)
     if(probe_idx1(ctx->input) && read_idx1(ctx->input, &p->idx1))
       {
       p->has_idx1 = 1;
-#ifdef DUMP_INDICES
-      dump_idx1(&p->idx1);
-#endif
+      if(ctx->opt->dump_indices)
+        dump_idx1(&p->idx1);
       }
     bgav_input_seek(ctx->input, ctx->data_start, SEEK_SET);
     }
@@ -2076,7 +2043,7 @@ static int open_avi(bgav_demuxer_context_t * ctx)
 
     for(i = 0; i < ctx->tt->cur->num_video_streams; i++)
       {
-      avi_vs = (video_priv_t*)(ctx->tt->cur->video_streams[i].priv);
+      avi_vs = ctx->tt->cur->video_streams[i].priv;
       if(check_codec(ctx->tt->cur->video_streams[i].fourcc,
                      video_codecs_msmpeg4v1))
         avi_vs->is_keyframe = is_keyframe_msmpeg4v1;
@@ -2124,8 +2091,9 @@ static int open_avi(bgav_demuxer_context_t * ctx)
 
   if(p->info)
     bgav_RIFFINFO_get_metadata(p->info, &ctx->tt->cur->metadata);
-  
-  ctx->stream_description = bgav_sprintf("AVI");
+
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "AVI");
   
   return 1;
   fail:
@@ -2150,7 +2118,7 @@ static void close_avi(bgav_demuxer_context_t * ctx)
   {
   avi_priv_t * priv;
   
-  priv = (avi_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
   if(priv)
     {
@@ -2171,7 +2139,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
   {
   chunk_header_t ch;
   bgav_packet_t * p;
-  bgav_stream_t * s = (bgav_stream_t*)0;
+  bgav_stream_t * s = NULL;
   uint32_t fourcc;
   int stream_id;
   avi_priv_t * priv;
@@ -2179,7 +2147,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
   audio_priv_t* avi_as;
   int result = 1;
   int64_t position;
-  priv = (avi_priv_t*)(ctx->priv);
+  priv = ctx->priv;
 
   if(priv->has_iavs && ctx->si)
     {
@@ -2198,7 +2166,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
       return 0;
       }
 
-#ifdef DUMP_HEADERS
+#ifdef DUMP_CHUNK_HEADERS
     dump_chunk_header(&ch);
 #endif
 
@@ -2239,7 +2207,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
       
       if(s->type == BGAV_STREAM_VIDEO)
         {
-        avi_vs = (video_priv_t*)s->priv;
+        avi_vs = s->priv;
         p->pts = avi_vs->frame_counter * s->data.video.format.frame_duration;
         avi_vs->frame_counter++;
         if(s->action == BGAV_STREAM_PARSE)
@@ -2252,7 +2220,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
         {
         if(s->index_mode == INDEX_MODE_SIMPLE)
           {
-          avi_as = (audio_priv_t*)s->priv;
+          avi_as = s->priv;
           p->pts = avi_as->sample_counter;
           avi_as->sample_counter += p->data_size / s->data.audio.block_align;
           if(s->action == BGAV_STREAM_PARSE)
@@ -2267,7 +2235,7 @@ static int next_packet_avi(bgav_demuxer_context_t * ctx)
     }
   else if(s->type == BGAV_STREAM_VIDEO) // Increase timestamp for empty frames
     {
-    avi_vs = (video_priv_t*)s->priv;
+    avi_vs = s->priv;
     avi_vs->frame_counter++;
     if(s->action == BGAV_STREAM_PARSE)
       s->duration = avi_vs->frame_counter * s->data.video.format.frame_duration;
@@ -2285,11 +2253,11 @@ static void resync_avi(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
   switch(s->type)
     {
     case BGAV_STREAM_AUDIO:
-      avi_as = (audio_priv_t*)(s->priv);
+      avi_as = s->priv;
       avi_as->sample_counter = STREAM_GET_SYNC(s);
       break;
     case BGAV_STREAM_VIDEO:
-      avi_vs = (video_priv_t*)(s->priv);
+      avi_vs = s->priv;
       avi_vs->frame_counter = STREAM_GET_SYNC(s) /
         s->data.video.format.frame_duration;
       break;
@@ -2303,7 +2271,7 @@ static void resync_avi(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
 static void seek_avi(bgav_demuxer_context_t * ctx, gavl_time_t time, int scale)
   {
   avi_priv_t * priv;
-  priv = (avi_priv_t*)ctx->priv;
+  priv = ctx->priv;
   if(ctx->si && priv->has_iavs)
     seek_iavs(ctx, time, scale);
   }
