@@ -29,6 +29,48 @@
 
 #define PROBE_SIZE 2048 /* Same as in MPlayer */
 
+#if LIBAVFORMAT_VERSION_MAJOR >= 53
+#define NEW_IO_API
+#endif
+
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)+(2<<8)+0)
+#define BRANDNEW_IO_API
+#endif
+
+#if LIBAVFORMAT_VERSION_MAJOR >= 53
+#define NEW_METADATA
+#endif
+
+#if LIBAVFORMAT_VERSION_MAJOR >= 54
+#define AV_METADATA_MATCH_CASE      AV_DICT_MATCH_CASE
+#define AV_METADATA_IGNORE_SUFFIX   AV_DICT_IGNORE_SUFFIX
+#define AV_METADATA_DONT_STRDUP_KEY AV_DICT_DONT_STRDUP_KEY
+#define AV_METADATA_DONT_STRDUP_VAL AV_DICT_DONT_STRDUP_VAL
+#define AV_METADATA_DONT_OVERWRITE  AV_DICT_DONT_OVERWRITE
+
+
+#define AVMetadata    AVDictionary
+#define AVMetadataTag AVDictionaryEntry
+
+#define av_metadata_get(m, key, prev, flags) av_dict_get(m, key, prev, flags)
+
+#endif
+
+#if LIBAVCODEC_VERSION_MAJOR >= 53
+#define CodecType AVMediaType
+#define CODEC_TYPE_UNKNOWN    AVMEDIA_TYPE_UNKNOWN
+#define CODEC_TYPE_VIDEO      AVMEDIA_TYPE_VIDEO
+#define CODEC_TYPE_AUDIO      AVMEDIA_TYPE_AUDIO
+#define CODEC_TYPE_DATA       AVMEDIA_TYPE_DATA
+#define CODEC_TYPE_SUBTITLE   AVMEDIA_TYPE_SUBTITLE
+#define CODEC_TYPE_ATTACHMENT AVMEDIA_TYPE_ATTACHMENT
+#define CODEC_TYPE_NB         AVMEDIA_TYPE_NB
+#endif
+
+#if LIBAVCODEC_VERSION_MAJOR >= 53
+#define PKT_FLAG_KEY AV_PKT_FLAG_KEY
+#endif
+
 static void cleanup_stream_ffmpeg(bgav_stream_t * s)
   {
   if(s->type == BGAV_STREAM_VIDEO)
@@ -42,15 +84,35 @@ typedef struct
   {
   AVInputFormat *avif;
   AVFormatContext *avfc;
-#if LIBAVFORMAT_VERSION_INT < ((52<<16)+(0<<8)+0)
-  ByteIOContext pb;
-#else
+#ifdef NEW_IO_API
+#define BUFFER_SIZE 1024 * 4
+  AVIOContext * pb;
+  unsigned char * buffer;
+#else  
   ByteIOContext * pb;
-#endif
+#endif // OLD_IO_API  
   } ffmpeg_priv_t;
 
 /* Callbacks for URLProtocol */
 
+#ifdef NEW_IO_API
+// TODO
+static int lavf_read(void * opaque, uint8_t *buf, int buf_size)
+  {
+  return bgav_input_read_data(opaque, buf, buf_size);
+  }
+
+static int64_t lavf_seek(void *opaque, int64_t offset, int whence)
+  {
+  bgav_input_context_t * input = opaque;
+  if(whence == AVSEEK_SIZE)
+    return input->total_bytes;
+  bgav_input_seek(input, offset, whence);
+  return input->position;
+  }
+  
+#else
+
 static int lavf_open(URLContext *h, const char *filename, int flags)
   {
   return 0;
@@ -60,7 +122,7 @@ static int lavf_read(URLContext *h, unsigned char *buf, int size)
   {
   bgav_input_context_t * input;
   int result;
-  input = (bgav_input_context_t *)h->priv_data;
+  input = h->priv_data;
 
   result = bgav_input_read_data(input, buf, size);
   if(!result)
@@ -68,7 +130,7 @@ static int lavf_read(URLContext *h, unsigned char *buf, int size)
   return result;
   }
 
-static int lavf_write(URLContext *h, unsigned char *buf, int size)
+static int lavf_write(URLContext *h, const unsigned char *buf, int size)
   {
   return -1;
   }
@@ -76,7 +138,7 @@ static int lavf_write(URLContext *h, unsigned char *buf, int size)
 static int64_t lavf_seek(URLContext *h, int64_t pos, int whence)
   {
   bgav_input_context_t * input;
-  input = (bgav_input_context_t *)h->priv_data;
+  input = h->priv_data;
 
   if(!input->input->seek_byte)
     return -1;
@@ -104,6 +166,8 @@ static URLProtocol bgav_protocol = {
     lavf_close,
 };
 
+#endif
+
 /* Demuxer functions */
 
 static AVInputFormat * get_format(bgav_input_context_t * input)
@@ -260,6 +324,8 @@ static audio_codec_map_t audio_codecs[] =
 #endif
     //    { CODEC_ID_MUSEPACK7, },
     //    { CODEC_ID_MLP, },
+    { CODEC_ID_MLP, BGAV_MK_FOURCC('.', 'm', 'l', 'p') },
+    
 #if LIBAVCODEC_BUILD >= ((51<<16)+(34<<8)+0)
     { CODEC_ID_GSM_MS, BGAV_WAVID_2_FOURCC(0x31) }, /* as found in WAV */
 #endif
@@ -439,7 +505,7 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx,
   {
   bgav_stream_t * s;
   int i;
-  audio_codec_map_t * map = (audio_codec_map_t *)0;
+  audio_codec_map_t * map = NULL;
   AVCodecContext *codec= st->codec;
   
   /* Get fourcc */
@@ -481,9 +547,9 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx,
   
   s->data.audio.format.num_channels = codec->channels;
   s->data.audio.format.samplerate = codec->sample_rate;
-  s->ext_size = codec->extradata_size;
-  s->ext_data = malloc(s->ext_size);
-  memcpy(s->ext_data, codec->extradata, codec->extradata_size);
+  
+  bgav_stream_set_extradata(s, codec->extradata, codec->extradata_size);
+  
   s->container_bitrate = codec->bit_rate;
   s->stream_id = index;
   
@@ -493,7 +559,7 @@ static void init_video_stream(bgav_demuxer_context_t * ctx,
                               AVStream * st, int index)
   {
   bgav_stream_t * s;
-  video_codec_map_t * map = (video_codec_map_t *)0;
+  video_codec_map_t * map = NULL;
   int i;
   uint32_t tag;
   AVCodecContext *codec= st->codec;
@@ -552,19 +618,18 @@ static void init_video_stream(bgav_demuxer_context_t * ctx,
 #else
   s->data.video.depth = codec->bits_per_coded_sample;
 #endif
-  s->ext_size = codec->extradata_size;
-  s->ext_data = malloc(s->ext_size);
-  memcpy(s->ext_data, codec->extradata, codec->extradata_size);
-  
+  bgav_stream_set_extradata(s, codec->extradata, codec->extradata_size);
+    
   s->container_bitrate = codec->bit_rate;
   s->stream_id = index;
-
+#if 0
   if(codec->palctrl)
     {
     s->priv = calloc(AVPALETTE_COUNT, sizeof(bgav_palette_entry_t));
     s->data.video.palette = s->priv;
     s->data.video.palette_size = AVPALETTE_COUNT;
     }
+#endif
   }
 
 static int open_ffmpeg(bgav_demuxer_context_t * ctx)
@@ -572,57 +637,88 @@ static int open_ffmpeg(bgav_demuxer_context_t * ctx)
   int i;
   ffmpeg_priv_t * priv;
   AVFormatContext *avfc;
-  AVFormatParameters ap;
+
   char * tmp_filename;
+#ifdef NEW_METADATA
+  AVMetadataTag * tag;
+#endif
+
+#ifndef BRANDNEW_IO_API
+  AVFormatParameters ap;
   memset(&ap, 0, sizeof(ap));
+#endif
+  
   priv = calloc(1, sizeof(*priv));
   ctx->priv = priv;
 
   /* With the current implementation in ffmpeg, this can be
      called multiple times */
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 29, 0)
-  register_protocol(&bgav_protocol);
-#else
-  av_register_protocol(&bgav_protocol);
-#endif
-  
-#if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(52, 26, 0)
-  avfc = av_alloc_format_context();
-#else
-  avfc = avformat_alloc_context();
-#endif
-  
+
   tmp_filename = bgav_sprintf("bgav:%s", ctx->input->filename);
+  
+#ifdef NEW_IO_API
+  // TODO
+  priv->buffer = malloc(BUFFER_SIZE);
+  priv->pb =
+    avio_alloc_context(priv->buffer,
+                       BUFFER_SIZE,
+                       0,
+                       ctx->input,
+                       lavf_read,
+                       NULL,
+                       ctx->input->input->seek_byte ? lavf_seek : NULL);
+#else
+  
+  av_register_protocol(&bgav_protocol);
 
   url_fopen(&priv->pb, tmp_filename, URL_RDONLY);
-#if LIBAVFORMAT_VERSION_INT < ((52<<16)+(0<<8)+0)
-  ((URLContext*)(priv->pb.opaque))->priv_data= ctx->input;
-#else
+
   ((URLContext*)(priv->pb->opaque))->priv_data= ctx->input;
-#endif
+
+#endif // !NEW_IO_API
+
+  avfc = avformat_alloc_context();
+
   priv->avif = get_format(ctx->input);
-  
-#if LIBAVFORMAT_VERSION_INT < ((52<<16)+(0<<8)+0)
-  if(av_open_input_stream(&avfc, &priv->pb, tmp_filename, priv->avif, &ap)<0)
-#else
+
+#ifdef BRANDNEW_IO_API
+  avfc->pb = priv->pb;
+
+  if(avformat_open_input(&avfc, tmp_filename, priv->avif, NULL)<0)
+    {
+    bgav_log(ctx->opt,BGAV_LOG_ERROR,LOG_DOMAIN,
+             "avformat_open_input failed");
+    free(tmp_filename);
+    return 0;
+    }
+#else 
   if(av_open_input_stream(&avfc, priv->pb, tmp_filename, priv->avif, &ap)<0)
-#endif
     {
     bgav_log(ctx->opt,BGAV_LOG_ERROR,LOG_DOMAIN,
              "av_open_input_stream failed");
     free(tmp_filename);
     return 0;
     }
+#endif
   free(tmp_filename);
   priv->avfc= avfc;
   /* Get the streams */
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)|(6<<8))|0
+  if(avformat_find_stream_info(avfc, NULL) < 0)
+    {
+    bgav_log(ctx->opt,BGAV_LOG_ERROR,LOG_DOMAIN,
+             "avformat_find_stream_info failed");
+    return 0;
+    }
+#else
   if(av_find_stream_info(avfc) < 0)
     {
     bgav_log(ctx->opt,BGAV_LOG_ERROR,LOG_DOMAIN,
              "av_find_stream_info failed");
     return 0;
     }
-
+#endif
+  
   ctx->tt = bgav_track_table_create(1);
   
   for(i = 0; i < avfc->nb_streams; i++)
@@ -649,10 +745,33 @@ static int open_ffmpeg(bgav_demuxer_context_t * ctx)
     if(priv->avfc->iformat->read_seek)
       ctx->flags |= BGAV_DEMUXER_CAN_SEEK;
     }
+  
+#ifdef NEW_METADATA
+#define GET_METADATA_STRING(gavl_name, ffmpeg_name) \
+  tag = av_metadata_get(avfc->metadata, ffmpeg_name, NULL, \
+                        AV_METADATA_IGNORE_SUFFIX); \
+  if(tag) \
+    gavl_metadata_set(&ctx->tt->cur->metadata, gavl_name, tag->value);
 
-  ctx->stream_description = bgav_sprintf(TRD("%s (via ffmpeg)"),
-                                         priv->avfc->iformat->long_name);
+#define GET_METADATA_INT(gavl_name, ffmpeg_name) \
+  tag = av_metadata_get(avfc->metadata, ffmpeg_name, NULL, \
+                        AV_METADATA_IGNORE_SUFFIX); \
+  if(tag) \
+    gavl_metadata_set_int(&ctx->tt->cur->metadata, gavl_name, atoi(tag->value));
 
+  
+  if(avfc->metadata)
+    {
+    GET_METADATA_STRING(GAVL_META_TITLE,     "title");
+    GET_METADATA_STRING(GAVL_META_AUTHOR,    "author");
+    GET_METADATA_STRING(GAVL_META_COPYRIGHT, "copyright");
+    GET_METADATA_STRING(GAVL_META_GENRE,     "genre");
+    GET_METADATA_STRING(GAVL_META_ALBUM,     "album");
+    GET_METADATA_INT(GAVL_META_TRACKNUMBER,  "track");
+    }
+
+#else
+  
   /* Metadata */
   if(avfc->title[0])
     ctx->tt->cur->metadata.title = bgav_strdup(avfc->title);
@@ -664,6 +783,14 @@ static int open_ffmpeg(bgav_demuxer_context_t * ctx)
     ctx->tt->cur->metadata.album = bgav_strdup(avfc->album);
   if(avfc->genre[0])
     ctx->tt->cur->metadata.genre = bgav_strdup(avfc->genre);
+
+#endif
+
+  gavl_metadata_set_nocpy(&ctx->tt->cur->metadata,
+                          GAVL_META_FORMAT,
+                          bgav_sprintf(TRD("%s (via ffmpeg)"),
+                                       priv->avfc->iformat->long_name));
+  
   
   return 1;
   }
@@ -672,9 +799,17 @@ static int open_ffmpeg(bgav_demuxer_context_t * ctx)
 static void close_ffmpeg(bgav_demuxer_context_t * ctx)
   {
   ffmpeg_priv_t * priv;
-  priv = (ffmpeg_priv_t*)ctx->priv;
-
+  priv = ctx->priv;
+#if LIBAVFORMAT_VERSION_INT >= ((53<<16)|(17<<8)|0)
+  avformat_close_input(&priv->avfc);
+#else
   av_close_input_file(priv->avfc);
+#endif
+  
+#ifdef NEW_IO
+  if(priv->buffer)
+    free(priv->buffer);
+#endif
   if(priv)
     free(priv);
   }
@@ -689,7 +824,15 @@ static int next_packet_ffmpeg(bgav_demuxer_context_t * ctx)
   bgav_packet_t * p;
   bgav_stream_t * s;
   int i_tmp;
-  priv = (ffmpeg_priv_t*)ctx->priv;
+  uint32_t * pal_i;
+
+#if LIBAVCODEC_VERSION_MAJOR >= 54
+  int pal_i_len;
+#else
+  const int pal_i_len = AVPALETTE_COUNT;
+#endif
+  
+  priv = ctx->priv;
   
   if(av_read_frame(priv->avfc, &pkt) < 0)
     return 0;
@@ -719,26 +862,41 @@ static int next_packet_ffmpeg(bgav_demuxer_context_t * ctx)
     }
   /* Handle palette */
   if((s->type == BGAV_STREAM_VIDEO) &&
+#if LIBAVCODEC_VERSION_MAJOR < 54
      avs->codec->palctrl &&
-     avs->codec->palctrl->palette_changed)
+     avs->codec->palctrl->palette_changed
+#else
+     (pal_i = (uint32_t*)av_packet_get_side_data(&pkt,
+                                                 AV_PKT_DATA_PALETTE,
+                                                 &pal_i_len))
+#endif
+     )
     {
-    pal = s->priv;
     
-    for(i = 0; i < AVPALETTE_COUNT; i++)
+#if LIBAVCODEC_VERSION_MAJOR < 54
+    pal_i = avs->codec->palctrl->palette;
+#else
+    pal_i_len /= 4;
+
+    bgav_packet_alloc_palette(p, pal_i_len);
+    
+    pal = p->palette;
+#endif
+    for(i = 0; i < pal_i_len; i++)
       {
-      i_tmp = (avs->codec->palctrl->palette[i] >> 24) & 0xff;
+      i_tmp = (pal_i[i] >> 24) & 0xff;
       pal[i].a = i_tmp | i_tmp << 8;
-      i_tmp = (avs->codec->palctrl->palette[i] >> 16) & 0xff;
+      i_tmp = (pal_i[i] >> 16) & 0xff;
       pal[i].r = i_tmp | i_tmp << 8;
-      i_tmp = (avs->codec->palctrl->palette[i] >> 8) & 0xff;
+      i_tmp = (pal_i[i] >> 8) & 0xff;
       pal[i].g = i_tmp | i_tmp << 8;
-      i_tmp = (avs->codec->palctrl->palette[i]) & 0xff;
+      i_tmp = (pal_i[i]) & 0xff;
       pal[i].b = i_tmp | i_tmp << 8;
       }
+#if LIBAVCODEC_VERSION_MAJOR < 54
     avs->codec->palctrl->palette_changed = 0;
-    s->data.video.palette_changed = 1;
+#endif
     }
-  
   if(pkt.flags&PKT_FLAG_KEY)
     PACKET_SET_KEYFRAME(p);
   bgav_stream_done_packet_write(s, p);
@@ -751,7 +909,7 @@ static int next_packet_ffmpeg(bgav_demuxer_context_t * ctx)
 static void seek_ffmpeg(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   {
   ffmpeg_priv_t * priv;
-  priv = (ffmpeg_priv_t*)ctx->priv;
+  priv = ctx->priv;
 
   av_seek_frame(priv->avfc, -1,
                 gavl_time_rescale(scale, AV_TIME_BASE, time), 0);
