@@ -40,8 +40,6 @@
 #define TYPE_DATE        11
 #define TYPE_TERMINATOR   9
 
-//#define DUMP_METADATA
-
 #define FOURCC_AAC  BGAV_MK_FOURCC('m', 'p', '4', 'a')
 // #define FOURCC_H264 BGAV_MK_FOURCC('h', '2', '6', '4')
 
@@ -140,7 +138,7 @@ static void flv_tag_dump(flv_tag * t)
 
 static void add_audio_stream(bgav_demuxer_context_t * ctx)
   {
-  bgav_stream_t * as = (bgav_stream_t*)0;
+  bgav_stream_t * as = NULL;
   as = bgav_track_add_audio_stream(ctx->tt->cur, ctx->opt);
   as->stream_id = AUDIO_ID;
   as->timescale = 1000;
@@ -149,9 +147,8 @@ static void add_audio_stream(bgav_demuxer_context_t * ctx)
 
 static void add_video_stream(bgav_demuxer_context_t * ctx)
   {
-  bgav_stream_t * vs = (bgav_stream_t*)0;
+  bgav_stream_t * vs = NULL;
   vs = bgav_track_add_video_stream(ctx->tt->cur, ctx->opt);
-  vs->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
   vs->flags |= STREAM_NO_DURATIONS;
 
   vs->stream_id = VIDEO_ID;
@@ -165,7 +162,7 @@ static int init_audio_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   int adpcm_bits;
   flv_priv_t * priv;
 
-  priv = (flv_priv_t*)(ctx->priv);
+  priv = ctx->priv;
 
   s->flags |= STREAM_NEED_START_TIME;
     
@@ -244,7 +241,7 @@ static int init_video_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   {
   flv_priv_t * priv;
   uint8_t header[1];
-  priv = (flv_priv_t*)(ctx->priv);
+  priv = ctx->priv;
 
   s->flags |= STREAM_NEED_START_TIME;
   
@@ -308,8 +305,6 @@ static double int2dbl(int64_t v){
     return ldexp(((v&((1LL<<52)-1)) + (1LL<<52)) * (v>>63|1), (v>>52&0x7FF)-1075);
 }
 
-#ifdef DUMP_METADATA
-
 static void dump_meta_object(meta_object_t * obj, int num_spaces)
   {
   int i;
@@ -350,7 +345,6 @@ static void dump_meta_object(meta_object_t * obj, int num_spaces)
       break;
     }
   }
-#endif
 
 static int read_meta_object(bgav_input_context_t * input,
                             meta_object_t * ret, int read_name, int read_type, int64_t end_pos)
@@ -501,7 +495,7 @@ static meta_object_t * meta_object_find(meta_object_t * obj, int num, const char
     if(obj[i].name && !strcasecmp(obj[i].name, name))
       return &obj[i];
     }
-  return (meta_object_t*)0;
+  return NULL;
   }
 
 static int
@@ -524,7 +518,7 @@ static int read_metadata(bgav_demuxer_context_t * ctx, flv_tag * t)
   int64_t end_pos;
   int ret;
   
-  priv = (flv_priv_t*)(ctx->priv);
+  priv = ctx->priv;
 
   end_pos = ctx->input->position + t->data_size;
   
@@ -565,7 +559,7 @@ static int next_packet_flv(bgav_demuxer_context_t * ctx)
   int32_t cts;
   int has_cts = 0;
 
-  priv = (flv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   position = ctx->input->position;
   /* Previous tag size (ignore this?) */
@@ -616,11 +610,8 @@ static int next_packet_flv(bgav_demuxer_context_t * ctx)
           init_video_stream(ctx);
         }
 #endif
-
-#ifdef DUMP_METADATA
-      dump_meta_object(&priv->metadata, 0);
-#endif
-      //    bgav_input_skip_dump(ctx->input, t.data_size);
+      if(ctx->opt->dump_headers)
+        dump_meta_object(&priv->metadata, 0);
       }
     else
       bgav_input_skip(ctx->input, t.data_size);
@@ -791,7 +782,7 @@ static void handle_metadata(bgav_demuxer_context_t * ctx)
   int num_obj;
   flv_priv_t * priv;
   int as, vs;
-  priv = (flv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   obj = priv->metadata.data.object.children;
   num_obj = priv->metadata.data.object.num_children;
@@ -833,7 +824,7 @@ static void seek_flv(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   int64_t file_pos;
   uint32_t i;
   flv_priv_t * priv;
-  priv = (flv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   /* Get index objects */
 
@@ -886,7 +877,7 @@ static int open_flv(bgav_demuxer_context_t * ctx)
   int64_t pos;
   uint8_t flags;
   uint32_t data_offset, tmp;
-  bgav_input_context_t * input_save = (bgav_input_context_t*)0;
+  bgav_input_context_t * input_save = NULL;
   flv_priv_t * priv;
   flv_tag t;
   
@@ -1004,8 +995,9 @@ static int open_flv(bgav_demuxer_context_t * ctx)
     bgav_input_seek(ctx->input, pos, SEEK_SET);
     }
   
-  ctx->stream_description = bgav_sprintf("Flash video (FLV)");
-  
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "FLV");
+
   return 1;
   
   fail:
@@ -1015,7 +1007,7 @@ static int open_flv(bgav_demuxer_context_t * ctx)
 static void resync_flv(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
   {
   flv_priv_t * priv;
-  priv = (flv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   if(s->type == BGAV_STREAM_AUDIO)
     {
@@ -1027,7 +1019,7 @@ static void resync_flv(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
 static void close_flv(bgav_demuxer_context_t * ctx)
   {
   flv_priv_t * priv;
-  priv = (flv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   free_meta_object(&priv->metadata);
 
