@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <zlib.h>
 
 #include <avdec_private.h>
 #include <matroska.h>
@@ -41,6 +42,9 @@ typedef struct
   bgav_mkv_track_t * tracks;
   int num_tracks;
 
+  bgav_mkv_tag_t * tags;
+  int num_tags;
+  
   int64_t segment_start;
 
   bgav_mkv_cluster_t cluster;
@@ -56,6 +60,8 @@ typedef struct
   
   int64_t cluster_pos; // Start position of last cluster
   
+  bgav_mkv_chapters_t chapters;
+  
   } mkv_t;
  
 static int probe_matroska(bgav_input_context_t * input)
@@ -186,12 +192,8 @@ static void init_vfw(bgav_stream_t * s)
   s->fourcc = bgav_BITMAPINFOHEADER_get_fourcc(&bh);
 
   if(data < end)
-    {
-    s->ext_size = end - data;
-    s->ext_data = malloc(s->ext_size);
-    memcpy(s->ext_data, data, s->ext_size);
-    }
-
+    bgav_stream_set_extradata(s, data, end - data);
+  
   if(bgav_video_is_divx4(s->fourcc))
     s->flags |= (STREAM_B_FRAMES|STREAM_PARSE_FRAME);
   }
@@ -199,7 +201,17 @@ static void init_vfw(bgav_stream_t * s)
 static void init_theora(bgav_stream_t * s)
   {
   setup_ogg_extradata(s);
-  s->fourcc = BGAV_MK_FOURCC('T', 'H', 'R', 'A');
+  }
+
+static void init_mpeg(bgav_stream_t * s)
+  {
+  bgav_mkv_track_t * track = s->priv;
+
+  s->flags |= STREAM_NEED_FRAMETYPES|STREAM_B_FRAMES;
+  if(track->CodecPrivateLen)
+    bgav_stream_set_extradata(s,
+                              track->CodecPrivate,
+                              track->CodecPrivateLen);
   }
 
 static const codec_info_t video_codecs[] =
@@ -216,8 +228,10 @@ static const codec_info_t video_codecs[] =
     { "V_REAL/RV30",     BGAV_MK_FOURCC('R','V','3','0'), NULL,     0 },
     { "V_REAL/RV40",     BGAV_MK_FOURCC('R','V','4','0'), NULL,     0 },
     { "V_VP8",           BGAV_MK_FOURCC('V','P','8','0'), NULL,     0 },
-    { "V_THEORA",        0x00,                            init_theora, 0 },
-    { "V_MPEG4/ISO/AVC", BGAV_MK_FOURCC('a','v','c','1'), NULL,     0 },
+    { "V_THEORA",        BGAV_MK_FOURCC('T','H','R','A'), init_theora, 0 },
+    { "V_MPEG4/ISO/AVC", BGAV_MK_FOURCC('a','v','c','1'), init_mpeg, 0 },
+    { "V_MPEG1",         BGAV_MK_FOURCC('m','p','v','1'), init_mpeg, 0 },
+    { "V_MPEG2",         BGAV_MK_FOURCC('m','p','v','2'), init_mpeg, 0 },
     { /* End */ }
   };
 
@@ -275,9 +289,7 @@ static void init_aac(bgav_stream_t * s)
   bgav_mkv_track_t * p = s->priv;
   if(p->CodecPrivate)
     {
-    s->ext_data = malloc(p->CodecPrivateLen);
-    memcpy(s->ext_data, p->CodecPrivate, p->CodecPrivateLen);
-    s->ext_size = p->CodecPrivateLen;
+    bgav_stream_set_extradata(s, p->CodecPrivate, p->CodecPrivateLen);
     }
   else
     {
@@ -339,8 +351,17 @@ static void init_ac3(bgav_stream_t * s)
   {
   bgav_mkv_track_t * p = s->priv;
   p->frame_samples = 1536;
+  s->flags |= STREAM_PARSE_FRAME; // Detect bitrate
   }
 
+static void init_dts(bgav_stream_t * s)
+  {
+  // bgav_mkv_track_t * p = s->priv;
+  s->flags |= STREAM_PARSE_FRAME;
+  s->index_mode = INDEX_MODE_SIMPLE;
+  }
+
+
 static const codec_info_t audio_codecs[] =
   {
     { "A_MS/ACM",        0x00,                            init_acm,    0  },
@@ -348,6 +369,7 @@ static const codec_info_t audio_codecs[] =
     { "A_MPEG/",         0x00,                            init_mpa,    CODEC_FLAG_INCOMPLETE },
     { "A_AAC/",          BGAV_MK_FOURCC('m','p','4','a'), init_aac,    CODEC_FLAG_INCOMPLETE },
     { "A_AC3",           BGAV_MK_FOURCC('.','a','c','3'), init_ac3,    0 },
+    { "A_DTS",           BGAV_MK_FOURCC('d','t','s',' '), init_dts,    0 },
     { /* End */ }
   };
 
@@ -360,7 +382,16 @@ static void init_stream_common(mkv_t * m,
   const codec_info_t * info = NULL;
   
   s->priv = track;
-
+  s->stream_id = track->TrackNumber;
+  s->timescale = 1000000000 / m->segment_info.TimecodeScale;
+  s->flags |= STREAM_NEED_START_TIME;
+  
+  if(track->Language)
+    gavl_metadata_set(&s->m, GAVL_META_LANGUAGE, track->Language);
+  
+  if(!codecs)
+    return;
+  
   while(codecs[i].id)
     {
     if(((codecs[i].flags & CODEC_FLAG_INCOMPLETE) &&
@@ -373,20 +404,20 @@ static void init_stream_common(mkv_t * m,
     i++;
     }
 
+  if(!info)
+    {
+    bgav_log(s->opt, BGAV_LOG_WARNING, LOG_DOMAIN, "Unknown codec: %s",
+             track->CodecID);
+    }
+  
   if(info)
     {
     s->fourcc = info->fourcc;
     if(info->init_func)
       info->init_func(s);
     else if(track->CodecPrivateLen)
-      {
-      s->ext_data = malloc(track->CodecPrivateLen);
-      memcpy(s->ext_data, track->CodecPrivate, track->CodecPrivateLen);
-      s->ext_size = track->CodecPrivateLen;
-      }
+      bgav_stream_set_extradata(s, track->CodecPrivate, track->CodecPrivateLen);
     }
-  s->stream_id = track->TrackNumber;
-  s->timescale = 1000000000 / m->segment_info.TimecodeScale;
   }
 
 static int init_audio(bgav_demuxer_context_t * ctx,
@@ -471,12 +502,179 @@ static int init_video(bgav_demuxer_context_t * ctx,
   fmt->timescale = s->timescale;
   fmt->framerate_mode = GAVL_FRAMERATE_VARIABLE;
   
-  s->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
   s->flags |= STREAM_NO_DURATIONS;
   
   return 1;
   }
 
+static int init_subtitle(bgav_demuxer_context_t * ctx,
+                         bgav_mkv_track_t * track)
+  {
+  bgav_stream_t * s = NULL;
+  mkv_t * m = ctx->priv;
+  
+  //  fprintf(stderr, "Init subtitles\n");
+  //  bgav_mkv_track_dump(track);
+
+  if(!strcmp(track->CodecID, "S_TEXT/UTF8"))
+    {
+    // fprintf(stderr, "UTF-8 subtitles\n");
+    s = bgav_track_add_subtitle_stream(ctx->tt->cur, ctx->opt, 1, "UTF-8");
+    gavl_metadata_set(&s->m, GAVL_META_FORMAT, "SRT");
+
+    }
+  else if(!strcmp(track->CodecID, "S_VOBSUB"))
+    {
+    char * line = NULL;
+    int line_alloc = 0;
+    uint32_t * pal = NULL;
+    bgav_input_context_t * input_mem;
+    
+    input_mem = bgav_input_open_memory(track->CodecPrivate, track->CodecPrivateLen,
+                                       ctx->opt);
+    
+    /* Get the palette from the codec data */
+    while(bgav_input_read_line(input_mem, &line, &line_alloc, 0, NULL))
+      {
+      //      fprintf(stderr, "Got line: %s\n", line);
+      if(!strncmp(line, "palette:", 8))
+        {
+        int index;
+        
+        float r, g, b;
+        int y, u, v;
+        
+        char ** colors = bgav_stringbreak(line + 8, ',');
+
+        index = 0;
+        while(colors[index])
+          index++;
+        if(index == 16)
+          {
+          index = 0;
+          pal = malloc(16 * sizeof(*pal));
+
+          for(index = 0; index < 16; index++)
+            {
+            pal[index] = strtol(colors[index], NULL, 16);
+
+            /* Now it gets insane: The vobsub program
+               converts the YCbCr palette from the IFO file to RGB,
+               so we need to convert it back
+
+               http://guliverkli.svn.sourceforge.net/viewvc/guliverkli/
+                      trunk/guliverkli/src/subtitles/
+                      VobSubFile.cpp?revision=605&view=markup
+               (line 821)
+             */
+            
+            r = (pal[index] >> 16) & 0xff;
+            g = (pal[index] >>  8) & 0xff;
+            b = pal[index] & 0xff;
+
+            y =  (int)((0.257 * r) + (0.504 * g) + (0.098 * b) + 16);
+            u =  (int)(-(0.148 * r) - (0.291 * g) + (0.439 * b) + 128);
+            v =  (int)((0.439 * r) - (0.368 * g) - (0.071 * b) + 128);
+            pal[index] = y << 16 | u << 8 | v;
+            }
+          }
+        bgav_stringbreak_free(colors);
+        }
+      if(pal)
+        break;
+      }
+    if(line)
+      free(line);
+    bgav_input_close(input_mem);
+    bgav_input_destroy(input_mem);
+
+    if(pal)
+      {
+      s = bgav_track_add_subtitle_stream(ctx->tt->cur, ctx->opt, 0, NULL);
+
+      gavl_metadata_set(&s->m, GAVL_META_FORMAT, "DVD subtitles");
+      s->fourcc = BGAV_MK_FOURCC('D', 'V', 'D', 'S');
+      s->ext_data = (uint8_t*)pal;
+      s->ext_size = 16 * 4; // 64
+      }
+    
+    }
+  else
+    {
+    bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+             "Subtitle format %s not suppported yet",
+             track->CodecID);
+    }
+  
+  if(!s)
+    return 1;
+  
+  init_stream_common(m, s, track, NULL);
+  
+  return 1;
+  }
+
+static bgav_chapter_list_t * create_chapter_list(bgav_mkv_chapters_t * chap)
+  {
+  int i;
+  /* We support only the first (and only?) EditionEntry */
+  bgav_chapter_list_t * ret = NULL;
+
+  if(!chap->num_editions || !chap->editions[0].num_atoms)
+    return NULL;
+
+  ret = bgav_chapter_list_create(GAVL_TIME_SCALE, chap->editions[0].num_atoms);
+
+  for(i = 0; i < chap->editions[0].num_atoms; i++)
+    {
+    // No ns precision
+    ret->chapters[i].time = chap->editions[0].atoms[i].ChapterTimeStart / 1000; 
+
+    /* Take first language */
+    if(chap->editions[0].atoms[i].num_displays)
+      ret->chapters[i].name =
+        bgav_strdup(chap->editions[0].atoms[i].displays[0].ChapString);
+    }
+  
+  return ret;
+  }
+
+#define SET_TAG_STRING(name, gavl_name) \
+  else if(!strcmp(tags[i].st[j].TagName, name)) \
+    gavl_metadata_set(ret, gavl_name, tags[i].st[j].TagString)
+
+#define SET_TAG_INT(name, gavl_name) \
+  else if(!strcmp(tags[i].st[j].TagName, name)) \
+    gavl_metadata_set_int(ret, gavl_name, atoi(tags[i].st[j].TagString))
+
+static void init_metadata(bgav_mkv_tag_t * tags, int num_tags,
+                          gavl_metadata_t * ret)
+  {
+  int i, j;
+  for(i = 0; i < num_tags; i++)
+    {
+    /* Pick just the global tags */
+    if(!tags[i].targets.num_TagTrackUID &&
+       !tags[i].targets.num_TagEditionUID &&
+       !tags[i].targets.num_TagChapterUID &&
+       !tags[i].targets.num_TagAttachmentUID)
+      {
+      for(j = 0; j < tags[i].num_st; j++)
+        {
+        if(!tags[i].st[j].TagName)
+          return;
+        SET_TAG_STRING("COMPOSER", GAVL_META_AUTHOR);
+        SET_TAG_STRING("ALBUM", GAVL_META_ALBUM);
+        SET_TAG_STRING("COPYRIGHT", GAVL_META_COPYRIGHT);
+        SET_TAG_STRING("COMMENT", GAVL_META_COMMENT);
+        SET_TAG_STRING("GENRE", GAVL_META_GENRE);
+        SET_TAG_STRING("DATE", GAVL_META_DATE);
+        SET_TAG_INT("PART_NUMBER", GAVL_META_TRACKNUMBER);
+        }
+      }
+    }
+  }
+
 #define MAX_HEADER_LEN 16
 
 static int open_matroska(bgav_demuxer_context_t * ctx)
@@ -550,24 +748,33 @@ static int open_matroska(bgav_demuxer_context_t * ctx)
                                          &p->meta_seek_info,
                                          &e))
           return 0;
-        // bgav_mkv_meta_seek_info_dump(&p->meta_seek_info);
+        if(ctx->opt->dump_headers)
+          bgav_mkv_meta_seek_info_dump(&p->meta_seek_info);
         break;
       case MKV_ID_Info:
         bgav_input_skip(ctx->input, head_len);
         e.end += pos;
         if(!bgav_mkv_segment_info_read(ctx->input, &p->segment_info, &e))
           return 0;
-        //        bgav_mkv_segment_info_dump(&p->segment_info);
+        if(ctx->opt->dump_headers)
+          bgav_mkv_segment_info_dump(&p->segment_info);
         break;
       case MKV_ID_Tracks:
         bgav_input_skip(ctx->input, head_len);
         e.end += pos;
         if(!bgav_mkv_tracks_read(ctx->input, &p->tracks, &p->num_tracks, &e))
           return 0;
+        if(ctx->opt->dump_headers)
+          {
+          for(i = 0; i < p->num_tracks; i++)
+            bgav_mkv_track_dump(&p->tracks[i]);
+          }
         break;
       case MKV_ID_Cues:
         if(!bgav_mkv_cues_read(ctx->input, &p->cues, p->num_tracks))
           return 0;
+        if(ctx->opt->dump_indices)
+          bgav_mkv_cues_dump(&p->cues);
         p->have_cues = 1;
         break;
       case MKV_ID_Cluster:
@@ -575,8 +782,22 @@ static int open_matroska(bgav_demuxer_context_t * ctx)
         ctx->data_start = pos;
         ctx->flags |= BGAV_DEMUXER_HAS_DATA_START;
         break;
-
-        
+      case MKV_ID_Chapters:
+        bgav_input_skip(ctx->input, head_len);
+        e.end += pos;
+        if(!bgav_mkv_chapters_read(ctx->input, &p->chapters, &e)) 
+          return 0;
+        if(ctx->opt->dump_headers)
+          bgav_mkv_chapters_dump(&p->chapters);
+        break;
+      case MKV_ID_Tags:
+        bgav_input_skip(ctx->input, head_len);
+        e.end += pos;
+        if(!bgav_mkv_tags_read(ctx->input, &p->tags, &p->num_tags, &e))
+          return 0;
+        if(ctx->opt->dump_headers)
+          bgav_mkv_tags_dump(p->tags, p->num_tags);
+        break;
       default:
         bgav_input_skip(ctx->input, head_len);
         bgav_mkv_element_skip(ctx->input, &e, "segment");
@@ -600,27 +821,37 @@ static int open_matroska(bgav_demuxer_context_t * ctx)
         break;
       case MKV_TRACK_COMPLEX:
         bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-                 "Complex tracks not supported yet\n");
+                 "Complex tracks not supported yet");
         break;
       case MKV_TRACK_LOGO:
         bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-                 "Logo tracks not supported yet\n");
+                 "Logo tracks not supported yet");
         break;
       case MKV_TRACK_SUBTITLE:
-        bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-                 "Subtitle tracks not supported yet\n");
+        if(!init_subtitle(ctx, &p->tracks[i]))
+          return 0;
+        
+        //        bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+        //                 "Subtitle tracks not supported yet");
         break;
       case MKV_TRACK_BUTTONS:
         bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-                 "Button tracks not supported yet\n");
+                 "Button tracks not supported yet");
         break;
       case MKV_TRACK_CONTROL:
         bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-                 "Control tracks not supported yet\n");
+                 "Control tracks not supported yet");
         break;
       }
     }
 
+  /* Look for chapters */
+  ctx->tt->cur->chapter_list = create_chapter_list(&p->chapters);
+
+  /* Metadata */
+  init_metadata(p->tags, p->num_tags,
+                &ctx->tt->cur->metadata);
+  
   /* Look for file index (cues) */
   if(p->meta_seek_info.num_entries && ctx->input->input->seek_byte &&
      !p->have_cues)
@@ -632,15 +863,23 @@ static int open_matroska(bgav_demuxer_context_t * ctx)
         // fprintf(stderr, "Found index at %"PRId64"\n",
         //         p->meta_seek_info.entries[i].SeekPosition);
 
-        pos = ctx->input->position;
-
-        bgav_input_seek(ctx->input,
-                        p->segment_start +
-                        p->meta_seek_info.entries[i].SeekPosition, SEEK_SET);
-        
-        if(bgav_mkv_cues_read(ctx->input, &p->cues, p->num_tracks))
-          p->have_cues = 1;
-        bgav_input_seek(ctx->input, pos, SEEK_SET);
+        if(p->segment_start + p->meta_seek_info.entries[i].SeekPosition > ctx->input->total_bytes)
+          {
+          bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+                   "Didn't find cues (truncated file?)");
+          }
+        else
+          {
+          pos = ctx->input->position;
+          
+          bgav_input_seek(ctx->input,
+                          p->segment_start +
+                          p->meta_seek_info.entries[i].SeekPosition, SEEK_SET);
+          
+          if(bgav_mkv_cues_read(ctx->input, &p->cues, p->num_tracks))
+            p->have_cues = 1;
+          bgav_input_seek(ctx->input, pos, SEEK_SET);
+          }
         }
       }
     }
@@ -656,10 +895,12 @@ static int open_matroska(bgav_demuxer_context_t * ctx)
     ctx->flags |= BGAV_DEMUXER_CAN_SEEK;
 
   if(!strcmp(p->ebml_header.DocType, "matroska"))
-    ctx->stream_description = bgav_sprintf("Matroska (version %d)", p->ebml_header.DocTypeVersion);
+    gavl_metadata_set(&ctx->tt->cur->metadata, 
+                      GAVL_META_FORMAT, "Matroska");
   else if(!strcmp(p->ebml_header.DocType, "webm"))
-    ctx->stream_description = bgav_sprintf("Matroska (webm)");
-
+    gavl_metadata_set(&ctx->tt->cur->metadata, 
+                      GAVL_META_FORMAT, "Webm");
+  
   bgav_input_close(input_mem);
   bgav_input_destroy(input_mem);
   
@@ -704,14 +945,53 @@ static int get_ebml_frame_size_int(uint8_t * ptr1, int size, int64_t * ret)
   return bytes;
   }
 
-static void append_packet_data(bgav_stream_t * s,
-                               bgav_packet_t * p,
-                               const uint8_t * data,
-                               int len)
+static void set_packet_data(bgav_stream_t * s,
+                            bgav_packet_t * p,
+                            const uint8_t * data,
+                            int len)
   {
-  bgav_packet_alloc(p, len);
-  memcpy(p->data, data, len);
-  p->data_size = len;
+  bgav_mkv_track_t * t = s->priv;
+
+  if((t->num_encodings == 1) &&
+     (t->encodings[0].ContentEncodingType == MKV_CONTENT_ENCODING_COMPRESSION) &&
+     (t->encodings[0].ContentCompression.ContentCompAlgo == MKV_CONTENT_COMP_ALGO_ZLIB))
+    {
+    uLongf out_len;
+    int err;
+    /* zlib decompression (probably the dumbest possible routine,
+       but it seems that this is used just for subtitles) */
+
+    bgav_packet_alloc(p, len * 5); // Optimistically assume 1:5 ratio
+    
+    while(1)
+      {
+      out_len = p->data_alloc;
+      err = uncompress(p->data, &out_len, data, len);
+
+      if(err == Z_OK)
+        {
+        p->data_size = out_len;
+        // fprintf(stderr, "Uncompressed packet %d -> %d\n", len, p->data_size);
+        break;
+        }
+      else if(err == Z_BUF_ERROR)
+        bgav_packet_alloc(p, p->data_alloc * 2); // Double the compression ratio
+      else
+        {
+        bgav_log(s->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
+                 "Decompression of matroska packet failed");
+        p->data_size = 0;
+        break;
+        }
+      }
+    }
+  else
+    {
+    /* Plain packet */
+    bgav_packet_alloc(p, len);
+    memcpy(p->data, data, len);
+    p->data_size = len;
+    }
   }
 
 static void setup_packet(mkv_t * m, bgav_stream_t * s,
@@ -786,8 +1066,15 @@ static int process_block(bgav_demuxer_context_t * ctx,
     case MKV_LACING_NONE:
       p = bgav_stream_get_packet_write(s);
       p->data_size = 0;
-      append_packet_data(s, p, b->data, b->data_size);
+      set_packet_data(s, p, b->data, b->data_size);
       setup_packet(m, s, p, pts, keyframe, 0);
+
+      if(s->type == BGAV_STREAM_SUBTITLE_TEXT)
+        {
+        if(bg && bg->BlockDuration)
+          p->duration = bg->BlockDuration;
+        }
+      
       bgav_stream_done_packet_write(s, p);
       break;
     case MKV_LACING_EBML:
@@ -841,7 +1128,7 @@ static int process_block(bgav_demuxer_context_t * ctx,
         {
         p = bgav_stream_get_packet_write(s);
         p->data_size = 0;
-        append_packet_data(s, p, ptr, m->lace_sizes[i]);
+        set_packet_data(s, p, ptr, m->lace_sizes[i]);
         ptr += m->lace_sizes[i];
         setup_packet(m, s, p, pts, keyframe, i);
         bgav_stream_done_packet_write(s, p);
@@ -884,7 +1171,7 @@ static int process_block(bgav_demuxer_context_t * ctx,
         {
         p = bgav_stream_get_packet_write(s);
         p->data_size = 0;
-        append_packet_data(s, p, ptr, m->lace_sizes[i]);
+        set_packet_data(s, p, ptr, m->lace_sizes[i]);
         ptr += m->lace_sizes[i];
         setup_packet(m, s, p, pts, keyframe, i);
         bgav_stream_done_packet_write(s, p);
@@ -906,7 +1193,7 @@ static int process_block(bgav_demuxer_context_t * ctx,
         {
         p = bgav_stream_get_packet_write(s);
         p->data_size = 0;
-        append_packet_data(s, p, ptr, frame_size);
+        set_packet_data(s, p, ptr, frame_size);
         ptr += frame_size;
         setup_packet(m, s, p, pts, keyframe, i);
         bgav_stream_done_packet_write(s, p);
@@ -1007,8 +1294,9 @@ static void close_matroska(bgav_demuxer_context_t * ctx)
   bgav_mkv_meta_seek_info_free(&priv->meta_seek_info);
   
   bgav_mkv_cues_free(&priv->cues);
-
+  bgav_mkv_chapters_free(&priv->chapters);
   bgav_mkv_cluster_free(&priv->cluster);
+  bgav_mkv_tags_free(priv->tags, priv->num_tags);
 
   bgav_mkv_block_group_free(&priv->bg);
   
