@@ -103,7 +103,7 @@ static bgav_albw_t * bgav_albw_read(bgav_input_context_t * input)
   char buffer[512];
   int64_t diff;
 
-  bgav_albw_t * ret = (bgav_albw_t *)0;
+  bgav_albw_t * ret = NULL;
   
   if(bgav_input_read_data(input, (uint8_t*)buffer, 12) < 12)
     goto fail;
@@ -160,7 +160,7 @@ static bgav_albw_t * bgav_albw_read(bgav_input_context_t * input)
   fail:
   if(ret)
     bgav_albw_destroy(ret);
-  return (bgav_albw_t*)0;
+  return NULL;
   }
 
 /* This is the actual demuxer */
@@ -173,7 +173,7 @@ typedef struct
   bgav_albw_t * albw;
   
   /* Global metadata */
-  bgav_metadata_t metadata;
+  gavl_metadata_t metadata;
   
   bgav_xing_header_t xing;
   int have_xing;
@@ -228,7 +228,7 @@ static int resync(bgav_demuxer_context_t * ctx, int check_next)
   int skipped_bytes = 0;
   bgav_mpa_header_t next_header;
     
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
 
   while(1)
     {
@@ -273,7 +273,7 @@ static gavl_time_t get_duration(bgav_demuxer_context_t * ctx,
   if(!(ctx->input->input->seek_byte))
     return GAVL_TIME_UNDEFINED;
   
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
   bgav_input_seek(ctx->input, start_offset, SEEK_SET);
   if(!resync(ctx, 1))
@@ -303,13 +303,11 @@ static gavl_time_t get_duration(bgav_demuxer_context_t * ctx,
 static int set_stream(bgav_demuxer_context_t * ctx)
      
   {
-  char * bitrate_string;
-  const char * version_string;
   bgav_stream_t * s;
   uint8_t frame[MAX_FRAME_BYTES]; /* Max possible mpeg audio frame size */
   mpegaudio_priv_t * priv;
   
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   if(!resync(ctx, 1))
     return 0;
   
@@ -344,59 +342,20 @@ static int set_stream(bgav_demuxer_context_t * ctx)
   if(!s->container_bitrate)
     {
     if(priv->have_xing)
-      {
       s->container_bitrate = BGAV_BITRATE_VBR;
-      // s->codec_bitrate     = BGAV_BITRATE_VBR;
-      }
     else
-      {
       s->container_bitrate = priv->header.bitrate;
-      // s->codec_bitrate     = priv->header.bitrate;
-      }
     }
-  
-  if(s->description)
-    free(s->description);
-    
-  switch(priv->header.version)
-    {
-    case MPEG_VERSION_1:
-      version_string = "1";
-      break;
-    case MPEG_VERSION_2:
-      version_string = "2";
-      break;
-    case MPEG_VERSION_2_5:
-      version_string = "2.5";
-      break;
-    default:
-      version_string = "Not specified";
-      break;
-    }
-    
-  if(s->container_bitrate == BGAV_BITRATE_VBR)
-    bitrate_string = bgav_strdup("Variable");
-  else
-    bitrate_string =
-      bgav_sprintf("%d kb/s",
-                   s->container_bitrate/1000);
-
-  ctx->stream_description =
-    bgav_sprintf("MPEG-%s layer %d, bitrate: %s",
-                 version_string, 
-                 priv->header.layer, bitrate_string);
-  free(bitrate_string);
-
   return 1;
   }
 
 static void get_metadata_albw(bgav_input_context_t* input,
                               int64_t * start_position,
                               int64_t * end_position,
-                              bgav_metadata_t * metadata)
+                              gavl_metadata_t * metadata)
   {
-  bgav_metadata_t metadata_v1;
-  bgav_metadata_t metadata_v2;
+  gavl_metadata_t metadata_v1;
+  gavl_metadata_t metadata_v2;
 
   bgav_id3v1_tag_t * id3v1 = NULL;
   bgav_id3v2_tag_t * id3v2 = NULL;
@@ -427,9 +386,9 @@ static void get_metadata_albw(bgav_input_context_t* input,
       bgav_id3v1_2_metadata(id3v1, &metadata_v1);
       }
     }
-  bgav_metadata_merge(metadata, &metadata_v2, &metadata_v1);
-  bgav_metadata_free(&metadata_v1);
-  bgav_metadata_free(&metadata_v2);
+  gavl_metadata_merge(metadata, &metadata_v2, &metadata_v1);
+  gavl_metadata_free(&metadata_v1);
+  gavl_metadata_free(&metadata_v2);
 
   if(id3v1)
     bgav_id3v1_destroy(id3v1);
@@ -440,19 +399,19 @@ static void get_metadata_albw(bgav_input_context_t* input,
 
 static bgav_track_table_t * albw_2_track(bgav_demuxer_context_t* ctx,
                                          bgav_albw_t * albw,
-                                         bgav_metadata_t * global_metadata)
+                                         gavl_metadata_t * global_metadata)
   {
   int i;
   const char * end_pos;
   bgav_track_table_t * ret;
   bgav_stream_t * s;
-  bgav_metadata_t track_metadata;
+  gavl_metadata_t track_metadata;
 
   memset(&track_metadata, 0, sizeof(track_metadata));
     
   if(!ctx->input->input->seek_byte)
     {
-    return (bgav_track_table_t *)0;
+    return NULL;
     }
   
   ret = bgav_track_table_create(albw->num_tracks);
@@ -468,15 +427,13 @@ static bgav_track_table_t * albw_2_track(bgav_demuxer_context_t* ctx,
                       &albw->tracks[i].start_pos,
                       &albw->tracks[i].end_pos,
                       &track_metadata);
-    bgav_metadata_merge(&ret->tracks[i].metadata,
+    gavl_metadata_merge(&ret->tracks[i].metadata,
                         &track_metadata, global_metadata);
-    bgav_metadata_free(&track_metadata);
+    gavl_metadata_free(&track_metadata);
     
     ret->tracks[i].duration = get_duration(ctx,
                                            albw->tracks[i].start_pos,
                                            albw->tracks[i].end_pos);
-      
-    
     }
   
   return ret;
@@ -484,15 +441,17 @@ static bgav_track_table_t * albw_2_track(bgav_demuxer_context_t* ctx,
 
 static int open_mpegaudio(bgav_demuxer_context_t * ctx)
   {
-  bgav_metadata_t metadata_v1;
-  bgav_metadata_t metadata_v2;
+  int i;
+  gavl_metadata_t metadata_v1;
+  gavl_metadata_t metadata_v2;
 
   bgav_id3v1_tag_t * id3v1 = NULL;
   bgav_stream_t * s;
   
   mpegaudio_priv_t * priv;
   int64_t oldpos;
-    
+  const char * title;
+  
   memset(&metadata_v1, 0, sizeof(metadata_v1));
   memset(&metadata_v2, 0, sizeof(metadata_v2));
   
@@ -525,7 +484,7 @@ static int open_mpegaudio(bgav_demuxer_context_t * ctx)
       }
     bgav_input_seek(ctx->input, oldpos, SEEK_SET);
     }
-  bgav_metadata_merge(&priv->metadata, &metadata_v2, &metadata_v1);
+  gavl_metadata_merge(&priv->metadata, &metadata_v2, &metadata_v1);
   
   if(priv->albw)
     {
@@ -548,22 +507,30 @@ static int open_mpegaudio(bgav_demuxer_context_t * ctx)
     ctx->tt->tracks[0].duration = get_duration(ctx,
                                                priv->data_start,
                                                priv->data_end);
-    bgav_metadata_merge(&ctx->tt->tracks[0].metadata,
+    gavl_metadata_merge(&ctx->tt->tracks[0].metadata,
                         &metadata_v2, &metadata_v1);
     }
 
   if(id3v1)
     bgav_id3v1_destroy(id3v1);
-  bgav_metadata_free(&metadata_v1);
-  bgav_metadata_free(&metadata_v2);
+  gavl_metadata_free(&metadata_v1);
+  gavl_metadata_free(&metadata_v2);
   
   if(ctx->input->input->seek_byte)
     ctx->flags |= BGAV_DEMUXER_CAN_SEEK;
 
-  if(!ctx->tt->tracks[0].name && ctx->input->metadata.title)
+  if(!ctx->tt->tracks[0].name &&
+     (title = gavl_metadata_get(&ctx->input->metadata, GAVL_META_TITLE)))
     {
-    ctx->tt->tracks[0].name = bgav_strdup(ctx->input->metadata.title);
+    ctx->tt->tracks[0].name = bgav_strdup(title);
     }
+
+  /* Set the format for each track */
+
+  for(i = 0; i < ctx->tt->num_tracks; i++)
+    gavl_metadata_set(&ctx->tt->tracks[i].metadata, 
+                      GAVL_META_FORMAT, "MPEG Audio");
+  
   ctx->index_mode = INDEX_MODE_SIMPLE;
   return 1;
   }
@@ -574,7 +541,7 @@ static int next_packet_mpegaudio(bgav_demuxer_context_t * ctx)
   bgav_stream_t * s;
   mpegaudio_priv_t * priv;
   int64_t bytes_left = -1;
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
   if(priv->data_end && (priv->data_end - ctx->input->position < 4))
     return 0;
@@ -615,7 +582,7 @@ static int next_packet_mpegaudio(bgav_demuxer_context_t * ctx)
 static void resync_mpegaudio(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
   {
   mpegaudio_priv_t * priv;
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   priv->frames = STREAM_GET_SYNC(s) / s->data.audio.format.samples_per_frame;
   }
 
@@ -626,7 +593,7 @@ static void seek_mpegaudio(bgav_demuxer_context_t * ctx, int64_t time,
   mpegaudio_priv_t * priv;
   bgav_stream_t * s;
   
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   s = ctx->tt->cur->audio_streams;
 
   time -= gavl_time_rescale(scale,
@@ -660,9 +627,9 @@ static void seek_mpegaudio(bgav_demuxer_context_t * ctx, int64_t time,
 static void close_mpegaudio(bgav_demuxer_context_t * ctx)
   {
   mpegaudio_priv_t * priv;
-  priv = (mpegaudio_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   
-  bgav_metadata_free(&priv->metadata);
+  gavl_metadata_free(&priv->metadata);
   
   if(priv->albw)
     bgav_albw_destroy(priv->albw);
@@ -674,7 +641,7 @@ static int select_track_mpegaudio(bgav_demuxer_context_t * ctx,
   {
   mpegaudio_priv_t * priv;
 
-  priv = (mpegaudio_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   if(priv->albw)
     {
