@@ -51,9 +51,6 @@
 /* Maximum number of consecutive error packets */
 #define MAX_ERROR_PACKETS   10
 
-// #define DUMP_PMT_PAT
-
-
 #if 0
 
 // #define DUMP_HDV_AUX
@@ -157,14 +154,14 @@ static void reset_streams_priv(bgav_track_t * track)
   stream_priv_t * priv;
   for(i = 0; i < track->num_audio_streams; i++)
     {
-    priv = (stream_priv_t*)(track->audio_streams[i].priv);
+    priv = track->audio_streams[i].priv;
     priv->last_pts = BGAV_TIMESTAMP_UNDEFINED;
     priv->pts_offset = 0;
     }
 
   for(i = 0; i < track->num_video_streams; i++)
     {
-    priv = (stream_priv_t*)(track->video_streams[i].priv);
+    priv = track->video_streams[i].priv;
     priv->last_pts = BGAV_TIMESTAMP_UNDEFINED;
     priv->pts_offset = 0;
     }
@@ -177,7 +174,7 @@ static void check_pts_wrap(bgav_stream_t * s, int64_t * pts)
   char tmp_string1[128];
   char tmp_string2[128];
   stream_priv_t * priv;
-  priv = (stream_priv_t*)(s->priv);
+  priv = s->priv;
 
   if(priv->last_pts == BGAV_TIMESTAMP_UNDEFINED)
     {
@@ -233,16 +230,101 @@ typedef struct
   int do_sync;
 
   int error_counter;
+
+  int discontinuous;
   
   } mpegts_t;
 
-static inline int
-parse_transport_packet(const bgav_options_t * opt,
-                       mpegts_t * priv)
+static int resync_file(bgav_demuxer_context_t * ctx,
+                       uint8_t * ptr, int offset, int * len)
   {
-  if(!bgav_transport_packet_parse(opt, &priv->ptr, &priv->packet))
-    return 0;
+  int i;
+  mpegts_t * priv = ctx->priv;
+  uint8_t * ptr1 = ptr + offset + 1;
+  
+  int bytes_skipped = 1;
+  
+  for(i = 1; i < priv->packet_size; i++)
+    {
+    if(*ptr1 == 0x47)
+      {
+      if(*len - offset - bytes_skipped > 0)
+        memmove(ptr + offset, ptr1, *len - offset - bytes_skipped);
+      *len -= bytes_skipped;
+      *len += bgav_input_read_data(ctx->input, ptr + *len, bytes_skipped);
+      bgav_log(ctx->opt, BGAV_LOG_INFO, LOG_DOMAIN,
+               "Skipped discontinuity %d bytes", bytes_skipped);
+      priv->discontinuous = 1;
+      return bytes_skipped;
+      }
+    ptr1++;
+    bytes_skipped++;
+    }
+  return 0;
+  }
 
+static int read_data(bgav_demuxer_context_t * ctx, int num_packets)
+  {
+  int i;
+  int bytes_read;
+  mpegts_t * priv = ctx->priv;
+  uint8_t * ptr;
+  
+  /* Read data */
+  bytes_read =
+    bgav_input_read_data(ctx->input,
+                         priv->buffer, num_packets * priv->packet_size);
+  
+  /* Check if we are in sync */
+  
+  ptr = priv->buffer;
+
+  num_packets = bytes_read / priv->packet_size;
+  
+  for(i = 0; i < num_packets; i++)
+    {
+    if(*ptr != 0x47)
+      {
+      if(!resync_file(ctx, priv->buffer, ptr - priv->buffer, &bytes_read))
+        break;
+      }
+    ptr += priv->packet_size;
+    }
+  num_packets = bytes_read / priv->packet_size;
+  return num_packets * priv->packet_size;
+  }
+
+#if 0
+static int get_data(bgav_demuxer_context_t * ctx, int num_packets)
+  {
+  return 0;
+  }
+#endif
+
+static inline int
+parse_transport_packet(bgav_demuxer_context_t * ctx)
+  {
+  const bgav_options_t * opt = ctx->opt;
+  mpegts_t * priv = ctx->priv;
+  
+  if(!bgav_transport_packet_parse(opt, &priv->ptr, &priv->packet))
+    {
+    if(ctx->input->total_bytes > 0)
+      {
+      if(ctx->input->total_bytes - ctx->input->position > 1024)
+        {
+        bgav_log(opt, BGAV_LOG_ERROR, LOG_DOMAIN,
+                 "Lost sync %"PRId64" bytes before file end\n",
+                 ctx->input->total_bytes - ctx->input->position);
+        }
+      }
+    else
+      {
+      bgav_log(opt, BGAV_LOG_ERROR, LOG_DOMAIN,
+               "Lost sync\n");
+      }
+    return 0;
+    }
   if(priv->packet.transport_error)
     {
     priv->error_counter++;
@@ -269,22 +351,23 @@ static inline int next_packet(mpegts_t * priv)
   return (priv->ptr - priv->buffer < priv->buffer_size);
   }
 
-static inline int next_packet_scan(bgav_input_context_t * input,
-                                   mpegts_t * priv, int can_seek)
+static inline int next_packet_scan(bgav_demuxer_context_t * ctx)
   {
   int packets_scanned;
+
+  mpegts_t * priv = ctx->priv;
+  int can_seek = ctx->input->input->seek_byte ? 1 : 0;
+  
   if(!next_packet(priv))
     {
     if(can_seek)
       {
       packets_scanned =
-        (input->position - priv->first_packet_pos) / priv->packet_size;
+        (ctx->input->position - priv->first_packet_pos) / priv->packet_size;
 
       if(packets_scanned < SCAN_PACKETS_SEEK)
         {
-        priv->buffer_size =
-          bgav_input_read_data(input, priv->buffer,
-                               priv->packet_size * SCAN_PACKETS);
+        priv->buffer_size =read_data(ctx, SCAN_PACKETS);
         if(!priv->buffer_size)
           return 0;
         
@@ -345,7 +428,7 @@ get_program_timestamp(bgav_demuxer_context_t * ctx, int * program)
   mpegts_t * priv;
   bgav_pes_header_t pes_header;
 
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
 
   /* Get the program to which the packet belongs */
 
@@ -431,7 +514,7 @@ static int get_program_durations(bgav_demuxer_context_t * ctx)
   int64_t total_packets;
   int64_t position;
   
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
   
   bgav_input_seek(ctx->input, priv->first_packet_pos, SEEK_SET);
   
@@ -446,9 +529,7 @@ static int get_program_durations(bgav_demuxer_context_t * ctx)
 
   keep_going = 1;
   
-  priv->buffer_size =
-    bgav_input_read_data(ctx->input, priv->buffer,
-                         priv->packet_size * SCAN_PACKETS);
+  priv->buffer_size = read_data(ctx, SCAN_PACKETS);
   if(!priv->buffer_size)
     return 0;
   
@@ -457,7 +538,7 @@ static int get_program_durations(bgav_demuxer_context_t * ctx)
   
   while(keep_going)
     {
-    if(!parse_transport_packet(ctx->opt, priv))
+    if(!parse_transport_packet(ctx))
       return 0;
 
     pts = get_program_timestamp(ctx, &program_index);
@@ -465,7 +546,7 @@ static int get_program_durations(bgav_demuxer_context_t * ctx)
       {
       priv->programs[program_index].start_pcr = pts;
       }
-    if(!next_packet_scan(ctx->input, priv, 1))
+    if(!next_packet_scan(ctx))
       return 0;
     
     /* Check if we are done */
@@ -493,18 +574,23 @@ static int get_program_durations(bgav_demuxer_context_t * ctx)
     {
     bgav_input_seek(ctx->input, position, SEEK_SET);
 
-    priv->buffer_size =
-      bgav_input_read_data(ctx->input, priv->buffer,
-                           priv->packet_size * SCAN_PACKETS);
+    priv->buffer_size = read_data(ctx, SCAN_PACKETS);
     if(!priv->buffer_size)
       return 0;
+
+    if(priv->discontinuous)
+      {
+      bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+               "Got discontinuities, won't get wrong durations");
+      return 0;
+      }
     
     priv->ptr = priv->buffer;
     priv->packet_start = priv->buffer;
 
     for(i = 0; i < SCAN_PACKETS; i++)
       {
-      if(!parse_transport_packet(ctx->opt, priv))
+      if(!parse_transport_packet(ctx))
         return 0;
       
       pts = get_program_timestamp(ctx, &program_index);
@@ -569,7 +655,7 @@ static int init_psi(bgav_demuxer_context_t * ctx,
   mpegts_t * priv;
   int i, j;
   
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
   //  bgav_hexdump(data, packet.payload_size, 16);
 
   /* We are at the beginning of the payload of a PAT packet */
@@ -584,9 +670,8 @@ static int init_psi(bgav_demuxer_context_t * ctx,
              "PAT section spans multiple packets, please report");
     return 0;
     }
-#ifdef DUMP_PMT_PAT
-  bgav_pat_section_dump(&pats);
-#endif
+  if(ctx->opt->dump_headers)
+    bgav_pat_section_dump(&pats);
   if(pats.section_number || pats.last_section_number)
     {
     bgav_log(ctx->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
@@ -621,7 +706,7 @@ static int init_psi(bgav_demuxer_context_t * ctx,
       }
     }
 
-  if(!next_packet_scan(ctx->input, priv, input_can_seek))
+  if(!next_packet_scan(ctx))
     {
     bgav_log(ctx->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
              "Premature EOF");
@@ -634,7 +719,7 @@ static int init_psi(bgav_demuxer_context_t * ctx,
 
   while(keep_going)
     {
-    if(!parse_transport_packet(ctx->opt, priv))
+    if(!parse_transport_packet(ctx))
       {
       bgav_log(ctx->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
                "Lost sync during initializing");
@@ -661,7 +746,7 @@ static int init_psi(bgav_demuxer_context_t * ctx,
     
     if(program == priv->num_programs)
       {
-      if(!next_packet_scan(ctx->input, priv, input_can_seek))
+      if(!next_packet_scan(ctx))
         break;
       continue;
       }
@@ -678,9 +763,8 @@ static int init_psi(bgav_demuxer_context_t * ctx,
                  "PMT section spans multiple packets, please report");
         return 0;
         }
-#ifdef DUMP_PMT_PAT
-      bgav_pmt_section_dump(&priv->programs[program].pmts);
-#endif
+      if(ctx->opt->dump_headers)
+        bgav_pmt_section_dump(&priv->programs[program].pmts);
       if(priv->programs[program].pmts.section_number ||
          priv->programs[program].pmts.last_section_number)
         {
@@ -689,7 +773,7 @@ static int init_psi(bgav_demuxer_context_t * ctx,
         return 0;
         }
       }
-    if(!next_packet_scan(ctx->input, priv, input_can_seek))
+    if(!next_packet_scan(ctx))
       break;
     }
 
@@ -697,8 +781,11 @@ static int init_psi(bgav_demuxer_context_t * ctx,
     {
     if(bgav_pmt_section_setup_track(&priv->programs[program].pmts,
                                     &ctx->tt->tracks[program],
-                                    ctx->opt, -1, -1, -1, (int*)0, (int*)0))
+                                    ctx->opt, -1, -1, -1, NULL, NULL))
       {
+      gavl_metadata_set(&ctx->tt->tracks[program].metadata, 
+                        GAVL_META_FORMAT, "MPEGTS");
+
       priv->programs[program].pcr_pid = priv->programs[program].pmts.pcr_pid;
       priv->programs[program].initialized = 1;
       init_streams_priv(&priv->programs[program],
@@ -834,10 +921,10 @@ static bgav_stream_t *
 test_streams_detect(test_streams_t * s, bgav_track_t * track,
                     const bgav_options_t * opt)
   {
-  bgav_stream_t * ret = (bgav_stream_t*)0;
+  bgav_stream_t * ret = NULL;
   test_stream_t * st;
   if(s->last_added < 0)
-    return (bgav_stream_t *)0;
+    return NULL;
 
   st = s->streams + s->last_added;
   
@@ -875,7 +962,7 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
 
   memset(&ts, 0, sizeof(ts));
   
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
   
   /* Allocate programs and track table */
   priv->num_programs = 1;
@@ -884,7 +971,7 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
 
   while(1)
     {
-    if(!parse_transport_packet(ctx->opt, priv))
+    if(!parse_transport_packet(ctx))
       break;
 
     /* Find the PCR PID */
@@ -897,7 +984,7 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
     if(bgav_track_find_stream_all(&ctx->tt->tracks[0],
                                   priv->packet.pid))
       {
-      if(!next_packet_scan(ctx->input, priv, input_can_seek))
+      if(!next_packet_scan(ctx))
         break;
       else
         continue;
@@ -917,11 +1004,32 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
       /* MPEG-2 Video */
       if((pes_header.stream_id >= 0xe0) && (pes_header.stream_id <= 0xef))
         {
+        int size = priv->buffer_size - (priv->ptr - priv->buffer);
+        
+        //        bgav_hexdump(priv->ptr, 16, 16);
+        
         s = bgav_track_add_video_stream(&ctx->tt->tracks[0], ctx->opt);
-        s->fourcc = BGAV_MK_FOURCC('m', 'p', 'g', 'v');
-        s->data.video.frametime_mode = BGAV_FRAMETIME_CODEC;
         s->index_mode = INDEX_MODE_MPEG;
-        s->flags |= STREAM_NEED_EXACT_COMPRESSION;
+
+        /* Try to distinguish between MPEG-1/2 and H.264 */
+        if((size >= 4) &&
+           (priv->ptr[0] == 0x00) && 
+           (priv->ptr[1] == 0x00) && 
+           (priv->ptr[2] == 0x01) &&
+           ((priv->ptr[3] == 0xb3) || // Sequence header
+            (priv->ptr[3] == 0x00) || // Picture
+            (priv->ptr[3] == 0xb8))) //  GOP 
+          {
+          s->fourcc = BGAV_MK_FOURCC('m', 'p', 'g', 'v');
+          s->flags |= STREAM_NEED_EXACT_COMPRESSION;
+          // fprintf(stderr, "Detected MPEG video\n");
+          }
+        else
+          {
+          s->fourcc = BGAV_MK_FOURCC('H', '2', '6', '4');
+          // fprintf(stderr, "Detected H.264 video\n");
+          }
+        
         }
       /* MPEG Audio */
       else if((pes_header.stream_id & 0xe0) == 0xc0)
@@ -946,7 +1054,7 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
       test_streams_append_packet(&ts, priv->ptr,
                                  priv->packet.payload_size,
                                  priv->packet.pid,
-                                 (bgav_pes_header_t*)0);
+                                 NULL);
       s = test_streams_detect(&ts, &ctx->tt->tracks[0], ctx->opt);
       }
     
@@ -956,7 +1064,7 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
       s->timescale = 90000;
       s->flags |= (STREAM_PARSE_FULL|STREAM_NEED_START_TIME);
       }
-    if(!next_packet_scan(ctx->input, priv, input_can_seek))
+    if(!next_packet_scan(ctx))
         break;
     }
   test_data_free(&ts);
@@ -964,6 +1072,9 @@ static int init_raw(bgav_demuxer_context_t * ctx, int input_can_seek)
   init_streams_priv(&priv->programs[0],
                     &ctx->tt->tracks[0]);
 
+  gavl_metadata_set(&ctx->tt->tracks[0].metadata, 
+                    GAVL_META_FORMAT, "MPEGTS");
+  
   return 1;
   }
 
@@ -1002,7 +1113,7 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
 
   
   priv->input_mem =
-    bgav_input_open_memory((uint8_t*)0, 0, ctx->opt);
+    bgav_input_open_memory(NULL, 0, ctx->opt);
   
   priv->ptr = priv->buffer;
   priv->packet_start = priv->buffer;
@@ -1016,9 +1127,7 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
     have_pat = 0;
     
     if(input_can_seek)
-      priv->buffer_size =
-        bgav_input_read_data(ctx->input, priv->buffer,
-                             priv->packet_size * SCAN_PACKETS);
+      priv->buffer_size = read_data(ctx, SCAN_PACKETS);
     else
       priv->buffer_size =
         bgav_input_get_data(ctx->input, priv->buffer,
@@ -1026,7 +1135,7 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
     
     while(1)
       {
-      parse_transport_packet(ctx->opt, priv);
+      parse_transport_packet(ctx);
     
       if(priv->packet.pid == 0x0000)
         {
@@ -1039,9 +1148,7 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
         {
         if(input_can_seek && (packets_scanned < SCAN_PACKETS_SEEK))
           {
-          priv->buffer_size =
-            bgav_input_read_data(ctx->input, priv->buffer,
-                                 priv->packet_size * SCAN_PACKETS);
+          priv->buffer_size = read_data(ctx, SCAN_PACKETS);
           if(!priv->buffer_size)
             break;
           priv->ptr = priv->buffer;
@@ -1066,9 +1173,7 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
         bgav_input_seek(ctx->input, priv->first_packet_pos,
                         SEEK_SET);
 
-        priv->buffer_size =
-          bgav_input_read_data(ctx->input, priv->buffer,
-                               priv->packet_size * SCAN_PACKETS);
+        priv->buffer_size = read_data(ctx, SCAN_PACKETS);
         priv->ptr = priv->buffer;
         priv->packet_start = priv->buffer;
         }
@@ -1093,6 +1198,8 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
       {
       init_streams_priv(&priv->programs[i],
                         &ctx->tt->tracks[i]);
+      gavl_metadata_set(&ctx->tt->tracks[i].metadata, 
+                        GAVL_META_FORMAT, "MPEGTS");
       }
     }
   
@@ -1110,8 +1217,11 @@ static int open_mpegts(bgav_demuxer_context_t * ctx)
                "Could not get program durations, seeking disabled");
 
       }
+
     }
-  ctx->stream_description = bgav_sprintf("MPEG-2 transport stream");
+
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "MPEGTS");
   
   ctx->index_mode = INDEX_MODE_MIXED;
   return 1;
@@ -1376,7 +1486,7 @@ static int process_packet(bgav_demuxer_context_t * ctx)
   int bytes_to_copy;
   bgav_pes_header_t pes_header;
   int64_t position;
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
 
   if(!priv->packet_size)
     return 0;
@@ -1389,9 +1499,7 @@ static int process_packet(bgav_demuxer_context_t * ctx)
      (position + priv->packet_size * num_packets > ctx->next_packet_pos))
     num_packets = (ctx->next_packet_pos - position) / priv->packet_size;
     
-  priv->buffer_size =
-    bgav_input_read_data(ctx->input,
-                         priv->buffer, priv->packet_size * num_packets);
+  priv->buffer_size = read_data(ctx, num_packets);
   
   if(priv->buffer_size < priv->packet_size)
     return 0;
@@ -1403,7 +1511,7 @@ static int process_packet(bgav_demuxer_context_t * ctx)
   
   for(i = 0; i < num_packets; i++)
     {
-    if(!parse_transport_packet(ctx->opt, priv))
+    if(!parse_transport_packet(ctx))
       {
       return 0;
       }
@@ -1671,7 +1779,7 @@ static int next_packet_mpegts(bgav_demuxer_context_t * ctx)
 static void resync_mpegts(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
   {
   stream_priv_t * priv;
-  priv = (stream_priv_t*)(s->priv);
+  priv = s->priv;
   priv->last_pts = BGAV_TIMESTAMP_UNDEFINED;
   priv->pts_offset = 0;
   }
@@ -1683,7 +1791,7 @@ static void seek_mpegts(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   int64_t position;
   
   mpegts_t * priv;
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
 
   reset_streams_priv(ctx->tt->cur);
   
@@ -1717,7 +1825,7 @@ static void close_mpegts(bgav_demuxer_context_t * ctx)
   {
   int i;
   mpegts_t * priv;
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
   
   if(!priv)
     return;
@@ -1745,7 +1853,7 @@ static int select_track_mpegts(bgav_demuxer_context_t * ctx,
                                 int track)
   {
   mpegts_t * priv;
-  priv = (mpegts_t*)(ctx->priv);
+  priv = ctx->priv;
   priv->current_program = track;
   priv->error_counter = 0;
   
