@@ -33,7 +33,6 @@
 #define CLIP_WRAPPED_CBR   1
 #define CLIP_WRAPPED_PARSE 2 /* Unsupported for now */
 
-// #define DUMP_MXF
 
 static void build_edl_mxf(bgav_demuxer_context_t * ctx);
 
@@ -80,12 +79,9 @@ static void set_pts(bgav_stream_t * s, stream_priv_t * sp,
   {
   if(s->type == BGAV_STREAM_VIDEO)
     {
-    if(s->data.video.frametime_mode != BGAV_FRAMETIME_CODEC)
-      {
-      p->pts = sp->pts_counter;
-      p->duration = s->data.video.format.frame_duration;
-      sp->pts_counter += p->duration;
-      }
+    p->pts = sp->pts_counter;
+    p->duration = s->data.video.format.frame_duration;
+    sp->pts_counter += p->duration;
     if(sp->frame_size)
       PACKET_SET_KEYFRAME(p);
     }
@@ -103,12 +99,12 @@ static int next_packet_clip_wrapped_const(bgav_demuxer_context_t * ctx, bgav_str
   {
   int bytes_to_read;
   mxf_klv_t klv;
-  bgav_stream_t * tmp_stream = (bgav_stream_t *)0;
+  bgav_stream_t * tmp_stream = NULL;
   stream_priv_t * sp;
   bgav_packet_t * p;
   mxf_t * priv;
-  priv = (mxf_t *)ctx->priv;
-  sp = (stream_priv_t*)(s->priv);
+  priv = ctx->priv;
+  sp = s->priv;
 
   /* Need the KLV packet for this stream */
   if(!sp->start)
@@ -169,7 +165,7 @@ static int process_packet_frame_wrapped(bgav_demuxer_context_t * ctx)
   stream_priv_t * sp;
   mxf_t * priv;
 
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
   position = ctx->input->position;
 
   if(position > ((partition_t*)(ctx->tt->cur->priv))->end_pos)
@@ -184,7 +180,7 @@ static int process_packet_frame_wrapped(bgav_demuxer_context_t * ctx)
     return 1;
     }
   
-  sp = (stream_priv_t*)(s->priv);
+  sp = s->priv;
 
   p = bgav_stream_get_packet_write(s);
   p->position = position;
@@ -287,7 +283,7 @@ static void init_stream_common(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   stream_priv_t * sp;
   mxf_t * priv;
   /* Common initialization */
-  priv = (mxf_t *)ctx->priv;
+  priv = ctx->priv;
   sp = calloc(1, sizeof(*priv));
   s->priv = sp;
   s->fourcc = fourcc;
@@ -355,7 +351,7 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   {
   stream_priv_t * priv;
   init_stream_common(ctx, s, st, sd, fourcc);
-  priv = (stream_priv_t *)s->priv;
+  priv = s->priv;
   if(sd->sample_rate_num % sd->sample_rate_den)
     bgav_log(ctx->opt, BGAV_LOG_WARNING, LOG_DOMAIN, "Rounding fractional audio samplerate");
   s->data.audio.format.samplerate = sd->sample_rate_num / sd->sample_rate_den;
@@ -401,7 +397,7 @@ static void init_video_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   {
   stream_priv_t * priv;
   init_stream_common(ctx, s, st, sd, fourcc);
-  priv = (stream_priv_t *)s->priv;
+  priv = s->priv;
   
   if((s->fourcc == BGAV_MK_FOURCC('m','p','g','v')) ||
      (s->fourcc == BGAV_MK_FOURCC('m','x','5','p')) ||
@@ -411,13 +407,11 @@ static void init_video_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
      (s->fourcc == BGAV_MK_FOURCC('m','x','4','n')) ||
      (s->fourcc == BGAV_MK_FOURCC('m','x','3','n')))
     {
-    s->data.video.frametime_mode = BGAV_FRAMETIME_CODEC;
     s->index_mode = INDEX_MODE_MPEG;
     s->flags |= STREAM_PARSE_FRAME;
     }
   else if(s->fourcc == BGAV_MK_FOURCC('m','p','4','v'))
     {
-    s->data.video.frametime_mode = BGAV_FRAMETIME_CONSTANT;
     s->index_mode = INDEX_MODE_MPEG;
     s->flags |= STREAM_PARSE_FULL; 
     }
@@ -432,11 +426,7 @@ static void init_video_stream(bgav_demuxer_context_t * ctx, bgav_stream_t * s,
   s->data.video.format.frame_height   = sd->height;
 
   if(sd->ext_size)
-    {
-    s->ext_size = sd->ext_size;
-    s->ext_data = malloc(s->ext_size);
-    memcpy(s->ext_data, sd->ext_data, s->ext_size);
-    }
+    bgav_stream_set_extradata(s, sd->ext_data, sd->ext_size);
   
   /* Todo: Aspect ratio */
   s->data.video.format.pixel_width    = 1;
@@ -459,9 +449,9 @@ handle_source_track_simple(bgav_demuxer_context_t * ctx,
   mxf_source_clip_t * sc;
   mxf_t * priv;
   uint32_t fourcc;
-  bgav_stream_t * s = (bgav_stream_t*)0;
+  bgav_stream_t * s = NULL;
   
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
   
   ss = (mxf_sequence_t*)(t->sequence);
   
@@ -557,7 +547,7 @@ static partition_t * get_body_partition(mxf_file_t * f, mxf_package_t * p)
   int i;
   uint32_t body_sid;
   if(!get_body_sid(f, p, &body_sid))
-    return (partition_t *)0;
+    return NULL;
   if(f->header.p.body_sid == body_sid)
     return &f->header;
 
@@ -566,7 +556,7 @@ static partition_t * get_body_partition(mxf_file_t * f, mxf_package_t * p)
     if(f->body_partitions[i].p.body_sid == body_sid)
       return &f->body_partitions[i];
     }
-  return (partition_t *)0;
+  return NULL;
   }
 
 /* Simple initialization */
@@ -574,10 +564,10 @@ static int init_simple(bgav_demuxer_context_t * ctx)
   {
   mxf_t * priv;
   int i, j, num_tracks = 0;
-  mxf_package_t * sp = (mxf_package_t *)0;
+  mxf_package_t * sp = NULL;
   int index = 0;
   
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
   /* We simply open the Source packages */
   
   for(i = 0; i < priv->mxf.header.num_metadata; i++)
@@ -628,9 +618,8 @@ static int open_mxf(bgav_demuxer_context_t * ctx)
              "Parsing MXF file failed, please report");
     return 0;
     }
-#ifdef DUMP_MXF
-  bgav_mxf_file_dump(&priv->mxf);
-#endif
+  if(ctx->opt->dump_headers)
+    bgav_mxf_file_dump(&priv->mxf);
   
   if(priv->mxf.header.max_source_sequence_components == 1)
     {
@@ -676,6 +665,10 @@ static int open_mxf(bgav_demuxer_context_t * ctx)
         ctx->index_mode = INDEX_MODE_NONE;
       }
     }
+
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "MXF");
+
   
   return 1;
   }
@@ -688,12 +681,12 @@ static bgav_stream_t * next_stream(bgav_stream_t * s, int num)
     {
     if(s[i].action != BGAV_STREAM_MUTE)
       {
-      sp = (stream_priv_t *)s[i].priv;
+      sp = s[i].priv;
       if(!sp->eof)
         return s + i;
       }
     }
-  return (bgav_stream_t*)0;
+  return NULL;
   }
 
 static int next_packet_mxf(bgav_demuxer_context_t * ctx)
@@ -703,9 +696,9 @@ static int next_packet_mxf(bgav_demuxer_context_t * ctx)
   
   s = ctx->request_stream;
   if(s)
-    sp = (stream_priv_t*)s->priv;
+    sp = s->priv;
   else
-    sp = (stream_priv_t*)0;
+    sp = NULL;
   
   while(!s || !sp->next_packet(ctx, s))
     {
@@ -717,7 +710,7 @@ static int next_packet_mxf(bgav_demuxer_context_t * ctx)
       s = next_stream(ctx->tt->cur->subtitle_streams,ctx->tt->cur->num_subtitle_streams);
     if(!s)
       return 0;
-    sp = (stream_priv_t*)s->priv;
+    sp = s->priv;
     }
   return 1;
   }
@@ -774,7 +767,7 @@ static int select_track_mxf(bgav_demuxer_context_t * ctx, int track)
 static void close_mxf(bgav_demuxer_context_t * ctx)
   {
   mxf_t * priv;
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
   bgav_mxf_file_free(&priv->mxf);
   free(priv);
   }
@@ -807,7 +800,7 @@ static int find_source_stream(bgav_stream_t * streams, int num, int track_id)
   
   for(i = 0; i < num; i++)
     {
-    p = (stream_priv_t *)streams->priv;
+    p = streams->priv;
     if(p->track_id == track_id)
       return i;
     }
@@ -878,11 +871,11 @@ static void handle_material_track(bgav_demuxer_context_t * ctx, mxf_package_t *
   mxf_sequence_t * ss;
   mxf_source_clip_t * sc;
   mxf_t * priv;
-  bgav_edl_stream_t * es = (bgav_edl_stream_t*)0;
+  bgav_edl_stream_t * es = NULL;
   int track_index, stream_index = 0;
   bgav_edl_segment_t * seg;
   int64_t duration = 0;
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
 
   if(!mt)
     return;
@@ -907,7 +900,7 @@ static void handle_material_track(bgav_demuxer_context_t * ctx, mxf_package_t *
        (ss->structural_components[0]->type != MXF_TYPE_SOURCE_CLIP))
       return;
 
-    es = (bgav_edl_stream_t*)0;
+    es = NULL;
     
     switch(ss->stream_type)
       {
@@ -960,9 +953,9 @@ static void build_edl_mxf(bgav_demuxer_context_t * ctx)
   {
   mxf_t * priv;
   int i, j;
-  mxf_package_t * sp = (mxf_package_t *)0;
+  mxf_package_t * sp = NULL;
   bgav_edl_track_t * t;
-  priv = (mxf_t*)ctx->priv;
+  priv = ctx->priv;
 
   if(!ctx->input->filename)
     return;
