@@ -32,8 +32,6 @@
 
 #define LOG_DOMAIN "nsv"
 
-// #define DUMP_HEADERS
-
 /*
  *  Straight forward implementation of the NSV spec from
  *  http://ultravox.aol.com/
@@ -80,11 +78,11 @@ static uint8_t * parse_metadata(uint8_t * buf,
   while(isspace(*buf) && (*buf != '\0'))
     buf++;
   if(*buf == '\0')
-    return (uint8_t*)0;
+    return NULL;
   start = (char*)buf;
   end = strchr(start, '=');
   if(!end)
-    return (uint8_t*)0;
+    return NULL;
   *name = bgav_strndup(start, end);
   start = end;
   start++; /* Start now points to the delimiter */
@@ -92,7 +90,7 @@ static uint8_t * parse_metadata(uint8_t * buf,
   start++;
   end = strchr(start, delim);
   if(!end)
-    return (uint8_t*)0;
+    return NULL;
   *value = bgav_strndup(start, end);
   end++;
   start = end;
@@ -130,8 +128,8 @@ static int nsv_file_header_read(bgav_input_context_t * ctx,
   pos = buf;
   while(pos && (pos - buf < ret->metadata_len))
     {
-    meta_name  = (char*)0;
-    meta_value = (char*)0;
+    meta_name  = NULL;
+    meta_value = NULL;
     pos = parse_metadata(pos, &meta_name, &meta_value);
 
     if(!pos)
@@ -200,7 +198,6 @@ static void nsv_file_header_free(nsv_file_header_t * h)
   if(h->toc.frames)         free(h->toc.frames);
   }
 
-#ifdef DUMP_HEADERS
 static void nsv_file_header_dump(nsv_file_header_t * h)
   {
   int i;
@@ -242,7 +239,6 @@ static void nsv_file_header_dump(nsv_file_header_t * h)
   else
     bgav_dprintf( "  No TOC\n");
   }
-#endif
 
 typedef struct
   {
@@ -269,7 +265,6 @@ static int nsv_sync_header_read(bgav_input_context_t * ctx,
   return 1;
   }
 
-#ifdef DUMP_HEADERS
 static void nsv_sync_header_dump(nsv_sync_header_t * h)
   {
   bgav_dprintf( "sync_header\n");
@@ -286,7 +281,6 @@ static void nsv_sync_header_dump(nsv_sync_header_t * h)
   bgav_dprintf( "  framerate_idx: %d\n", h->framerate);
   bgav_dprintf( "  syncoffs:      %d\n", h->syncoffs);
   }
-#endif
 
 typedef struct
   {
@@ -399,7 +393,8 @@ static int open_nsv(bgav_demuxer_context_t * ctx)
   nsv_sync_header_t sh;
   uint32_t fourcc;
   int done = 0;
-  bgav_input_context_t * input_save = (bgav_input_context_t*)0;
+  bgav_input_context_t * input_save = NULL;
+  const char * title;
   
   //  test_framerate();
   
@@ -426,18 +421,16 @@ static int open_nsv(bgav_demuxer_context_t * ctx)
       else
         {
         p->has_fh = 1;
-#ifdef DUMP_HEADERS
-        nsv_file_header_dump(&p->fh);
-#endif
+        if(ctx->opt->dump_headers)
+          nsv_file_header_dump(&p->fh);
         }
       }
     else if(fourcc == NSV_SYNC_HEADER)
       {
       if(!nsv_sync_header_read(ctx->input, &sh))
         return 0;
-#ifdef DUMP_HEADERS
-      nsv_sync_header_dump(&sh);
-#endif
+      if(ctx->opt->dump_headers)
+        nsv_sync_header_dump(&sh);
       done = 1;
       }
     }
@@ -448,7 +441,6 @@ static int open_nsv(bgav_demuxer_context_t * ctx)
   if(sh.vidfmt != BGAV_MK_FOURCC('N','O','N','E'))
     {
     s = bgav_track_add_video_stream(ctx->tt->cur, ctx->opt);
-    s->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
     s->flags |= STREAM_NO_DURATIONS;
     s->fourcc = sh.vidfmt;
     
@@ -498,16 +490,18 @@ static int open_nsv(bgav_demuxer_context_t * ctx)
 
     /* Metadata */
     if(p->fh.metadata.title)
-      ctx->tt->cur->metadata.title =
-        bgav_strdup(p->fh.metadata.title);
-
+      gavl_metadata_set(&ctx->tt->cur->metadata,
+                        GAVL_META_TITLE,
+                        p->fh.metadata.title);
     if(p->fh.metadata.url)
-      ctx->tt->cur->metadata.comment =
-        bgav_strdup(p->fh.metadata.url);
+      gavl_metadata_set(&ctx->tt->cur->metadata,
+                        GAVL_META_URL,
+                        p->fh.metadata.url);
     if(p->fh.metadata.creator)
-      ctx->tt->cur->metadata.author =
-        bgav_strdup(p->fh.metadata.creator);
-
+      gavl_metadata_set(&ctx->tt->cur->metadata,
+                        GAVL_META_AUTHOR,
+                        p->fh.metadata.creator);
+    
     /* Decide whether we can seek */
     if(ctx->input->input->seek_byte)
       {
@@ -527,12 +521,14 @@ static int open_nsv(bgav_demuxer_context_t * ctx)
   ctx->data_start = ctx->input->position;
   ctx->flags |= BGAV_DEMUXER_HAS_DATA_START;
   
-  if(!ctx->tt->tracks[0].name && ctx->input->metadata.title)
+  if(!ctx->tt->tracks[0].name &&
+     (title = gavl_metadata_get(&ctx->input->metadata, GAVL_META_TITLE)))
     {
-    ctx->tt->tracks[0].name = bgav_strdup(ctx->input->metadata.title);
+    ctx->tt->tracks[0].name = bgav_strdup(title);
     }
 
-  ctx->stream_description = bgav_sprintf("Nullsoft Video (NSV)");
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "NSV");
 
   if(p->need_pcm_format)
     {
@@ -609,7 +605,7 @@ static int next_packet_nsv(bgav_demuxer_context_t * ctx)
   uint32_t fourcc;
   int have_sync_header = 0;
   
-  priv = (nsv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   if(!priv->payload_follows)
     {
@@ -764,7 +760,7 @@ static void seek_nsv(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   nsv_sync_header_t sh;
   uint32_t fourcc;
 
-  priv = (nsv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   if(!priv->fh.toc.frames) /* TOC version 1 */
     {
@@ -825,7 +821,7 @@ static void seek_nsv(bgav_demuxer_context_t * ctx, int64_t time, int scale)
 static int select_track_nsv(bgav_demuxer_context_t * ctx, int track)
   {
   nsv_priv_t * priv;
-  priv = (nsv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   priv->payload_follows = 1;
   return 1;
   }
@@ -833,7 +829,7 @@ static int select_track_nsv(bgav_demuxer_context_t * ctx, int track)
 static void close_nsv(bgav_demuxer_context_t * ctx)
   {
   nsv_priv_t * priv;
-  priv = (nsv_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   nsv_file_header_free(&priv->fh);
   }
 
