@@ -46,6 +46,7 @@
 #include <avdec_private.h>
 #include <vorbis_comment.h>
 #include <dirac_header.h>
+#include <flac_header.h>
 
 #include <ogg/ogg.h>
 
@@ -107,7 +108,7 @@ typedef struct
 
   int keyframe_granule_shift;
 
-  bgav_metadata_t metadata;
+  gavl_metadata_t metadata;
 
   int64_t frame_counter;
 
@@ -268,32 +269,32 @@ static void dump_ogg(bgav_demuxer_context_t * ctx)
   for(i = 0; i < ctx->tt->num_tracks; i++)
     {
     track = &ctx->tt->tracks[i];
-    track_priv = (track_priv_t*)(track->priv);
+    track_priv = track->priv;
     bgav_dprintf( "Track %d, start_pos: %" PRId64 ", end_pos: %" PRId64 "\n",
             i+1, track_priv->start_pos, track_priv->end_pos);
     
     for(j = 0; j < track->num_audio_streams; j++)
       {
       s = &track->audio_streams[j];
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
       bgav_dprintf( "Audio stream %d\n", j+1);
       bgav_dprintf( "  Serialno: %d\n", s->stream_id);
       bgav_dprintf( "  Language: %s\n", s->language);
       bgav_dprintf( "  Last granulepos: %" PRId64 "\n",
               stream_priv->last_granulepos);
       bgav_dprintf( "  Metadata:\n");
-      bgav_metadata_dump(&stream_priv->metadata);
+      gavl_metadata_dump(&stream_priv->metadata);
       }
     for(j = 0; j < track->num_video_streams; j++)
       {
       s = &track->video_streams[j];
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
       bgav_dprintf( "Video stream %d\n", j+1);
       bgav_dprintf( "  Serialno: %d\n", s->stream_id);
       bgav_dprintf( "  Last granulepos: %" PRId64 "\n",
               stream_priv->last_granulepos);
       bgav_dprintf( "  Metadata:\n");
-      bgav_metadata_dump(&stream_priv->metadata);
+      gavl_metadata_dump(&stream_priv->metadata);
       }
     for(j = 0; j < track->num_subtitle_streams; j++)
       {
@@ -301,12 +302,12 @@ static void dump_ogg(bgav_demuxer_context_t * ctx)
         continue;
       
       s = &track->subtitle_streams[j];
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
       bgav_dprintf( "Subtitle stream %d\n", j+1);
       bgav_dprintf( "  Serialno: %d\n", s->stream_id);
       bgav_dprintf( "  Language: %s\n", s->language);
       bgav_dprintf( "  Metadata:\n");
-      bgav_metadata_dump(&stream_priv->metadata);
+      gavl_metadata_dump(&stream_priv->metadata);
       }
     
     }
@@ -328,14 +329,12 @@ static void parse_vorbis_comment(bgav_stream_t * s, uint8_t * data,
 
   memset(&vc, 0, sizeof(vc));
 
-  stream_priv = (stream_priv_t*)(s->priv);
+  stream_priv = s->priv;
   
   if(!bgav_vorbis_comment_read(&vc, input_mem))
     return;
 
-  bgav_metadata_free(&stream_priv->metadata);
-  if(s->language[0] != '\0')
-    s->language[0] = '\0';
+  gavl_metadata_free(&stream_priv->metadata);
   
   bgav_vorbis_comment_2_metadata(&vc, &stream_priv->metadata);
 
@@ -344,8 +343,12 @@ static void parse_vorbis_comment(bgav_stream_t * s, uint8_t * data,
     {
     language = bgav_lang_from_name(field);
     if(language)
-      strcpy(s->language, language);
+      {
+      gavl_metadata_set(&s->m, GAVL_META_LANGUAGE, language);
+      }
     }
+  
+  gavl_metadata_set(&s->m, GAVL_META_SOFTWARE, vc.vendor);
   bgav_vorbis_comment_free(&vc);
   bgav_input_destroy(input_mem);
   }
@@ -354,7 +357,7 @@ static void parse_vorbis_comment(bgav_stream_t * s, uint8_t * data,
 
 static void seek_byte(bgav_demuxer_context_t * ctx, int64_t pos)
   {
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
   ogg_sync_reset(&priv->oy);
   //  ogg_page_clear(&priv->os);
   bgav_input_seek(ctx->input, pos, SEEK_SET);
@@ -368,7 +371,7 @@ static int get_data(bgav_demuxer_context_t * ctx)
   int bytes_to_read;
   char * buf;
   int result;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
 
   
   bytes_to_read = BYTES_TO_READ;
@@ -401,7 +404,7 @@ static int get_page(bgav_demuxer_context_t * ctx)
   int page_size;
   char * buf;
   int bytes_skipped = 0;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
 
   if(priv->page_valid)
     return 1;
@@ -510,15 +513,30 @@ static uint32_t detect_stream(ogg_packet * op)
 static void cleanup_stream_ogg(bgav_stream_t * s)
   {
   stream_priv_t * stream_priv;
-  stream_priv = (stream_priv_t*)(s->priv);
+  stream_priv = s->priv;
   if(stream_priv)
     {
     ogg_stream_clear(&stream_priv->os);
-    bgav_metadata_free(&stream_priv->metadata);
+    gavl_metadata_free(&stream_priv->metadata);
     free(stream_priv);
     }
   }
 
+static void setup_flac(bgav_stream_t * s)
+  {
+  bgav_flac_streaminfo_t h;
+
+  if(s->ext_size - 8 < BGAV_FLAC_STREAMINFO_SIZE)
+    return;
+  
+  bgav_flac_streaminfo_read(s->ext_data + 8, &h);
+  bgav_flac_streaminfo_init_stream(&h, s);
+
+  s->flags |= STREAM_PARSE_FRAME;
+  s->index_mode = INDEX_MODE_SIMPLE;
+  s->timescale = s->data.audio.format.samplerate;
+  }
+
 /* Set up a track, which starts at start position */
 
 static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
@@ -535,7 +553,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
   int header_bytes = 0;
   bgav_dirac_sequence_header_t dirac_header;
   
-  priv = (ogg_t *)(ctx->priv);
+  priv = ctx->priv;
 
   ogg_track = calloc(1, sizeof(*ogg_track));
   ogg_track->start_pos = start_position;
@@ -617,7 +635,6 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         s->index_mode = INDEX_MODE_SIMPLE;
         s->priv   = ogg_stream;
         s->stream_id = serialno;
-        s->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
         ogg_stream->header_packets_needed = 3;
         append_extradata(s, &priv->op);
         ogg_stream->header_packets_read = 1;
@@ -671,7 +688,6 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         s->index_mode = INDEX_MODE_SIMPLE;
         s->priv   = ogg_stream;
         s->stream_id = serialno;
-        s->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
         ogg_stream->header_packets_needed = 1;
         ogg_stream->header_packets_read = 1;
 
@@ -704,19 +720,14 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         s->index_mode = INDEX_MODE_SIMPLE;
         s->flags |= STREAM_NO_DURATIONS;
         ogg_stream->fourcc_priv = FOURCC_FLAC_NEW;
-        s->ext_data = malloc(priv->op.bytes - 9);
-        memcpy(s->ext_data, priv->op.packet + 9, priv->op.bytes - 9);
-        s->ext_size = priv->op.bytes - 9;
+        bgav_stream_set_extradata(s, priv->op.packet + 9, priv->op.bytes - 9);
         s->stream_id = serialno;
         
         /* We tell the decoder, that this is the last metadata packet */
         s->ext_data[4] |= 0x80;
         
-        s->data.audio.format.samplerate = 
-          (s->ext_data[18] << 12) |
-          (s->ext_data[19] << 4) |
-          ((s->ext_data[20] >> 4)&0xf);
-
+        setup_flac(s);
+        
         ogg_stream->header_packets_needed = BGAV_PTR_2_16BE(priv->op.packet+7)+1;
         ogg_stream->header_packets_read = 1;
 
@@ -752,10 +763,8 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         ogg_stream->header_packets_read = 1;
 
         /* First packet is also the extradata */
-        s->ext_data = malloc(priv->op.bytes);
-        memcpy(s->ext_data, priv->op.packet, priv->op.bytes);
-        s->ext_size = priv->op.bytes;
-
+        bgav_stream_set_extradata(s, priv->op.packet, priv->op.bytes);
+        
         /* Samplerate */
         s->data.audio.format.samplerate = BGAV_PTR_2_32LE(priv->op.packet + 36);
 
@@ -776,8 +785,6 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         s->stream_id = serialno;
         s->index_mode = INDEX_MODE_SIMPLE;
         
-        s->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
-        
         input_mem = bgav_input_open_memory(priv->op.packet + 1, priv->op.bytes - 1, s->opt);
 
         if(!ogm_header_read(input_mem, &ogm_header))
@@ -823,7 +830,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         break;
       case FOURCC_OGM_TEXT:
         s = bgav_track_add_subtitle_stream(track, ctx->opt, 1,
-                                           (const char*)0);
+                                           NULL);
         s->cleanup = cleanup_stream_ogg;
         s->priv   = ogg_stream;
         s->stream_id = serialno;
@@ -850,7 +857,8 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         
         s->fourcc = FOURCC_OGM_TEXT;
         s->timescale = 10000000 / ogm_header.time_unit;
-        s->description = bgav_sprintf("OGM subtitles");
+
+        gavl_metadata_set(&s->m, GAVL_META_FORMAT, "OGM subtitles");
         ogg_stream->header_packets_needed = 2;
         ogg_stream->header_packets_read = 1;
         priv->is_ogm = 1;
@@ -883,7 +891,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
 
     if(s)
       {
-      ogg_stream = (stream_priv_t*)(s->priv);
+      ogg_stream = s->priv;
       ogg_stream_pagein(&ogg_stream->os, &priv->current_page);
       priv->page_valid = 0;
       header_bytes +=
@@ -959,11 +967,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
 
                 /* We tell the decoder, that this is the last metadata packet */
                 s->ext_data[4] |= 0x80;
-              
-                s->data.audio.format.samplerate = 
-                  (s->ext_data[18] << 12) |
-                  (s->ext_data[19] << 4) |
-                  ((s->ext_data[20] >> 4)&0xf);
+                setup_flac(s);
                 break;
               case 1:
                 parse_vorbis_comment(s, priv->op.packet+4, priv->op.bytes-4);
@@ -987,7 +991,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
     
     for(i = 0; i < track->num_audio_streams; i++)
       {
-      ogg_stream = (stream_priv_t*)(track->audio_streams[i].priv);
+      ogg_stream = track->audio_streams[i].priv;
       if(ogg_stream->header_packets_read < ogg_stream->header_packets_needed)
         done = 0;
       }
@@ -996,7 +1000,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
       {
       for(i = 0; i < track->num_video_streams; i++)
         {
-        ogg_stream = (stream_priv_t*)(track->video_streams[i].priv);
+        ogg_stream = track->video_streams[i].priv;
         if(ogg_stream->header_packets_read < ogg_stream->header_packets_needed)
           done = 0;
         }
@@ -1008,7 +1012,7 @@ static int setup_track(bgav_demuxer_context_t * ctx, bgav_track_t * track,
         {
         if(track->subtitle_streams[i].data.subtitle.subreader)
           continue;
-        ogg_stream = (stream_priv_t*)(track->subtitle_streams[i].priv);
+        ogg_stream = track->subtitle_streams[i].priv;
         if(ogg_stream->header_packets_read < ogg_stream->header_packets_needed)
           done = 0;
         }
@@ -1044,7 +1048,7 @@ find_first_page(bgav_demuxer_context_t * ctx, int64_t pos1, int64_t pos2,
   {
   int64_t ret;
   int result;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
 
   seek_byte(ctx, pos1);
   ret = pos1;
@@ -1152,9 +1156,9 @@ static int track_has_serialno(bgav_track_t * track,
     }
 
   if(s)
-    *s = (bgav_stream_t*)0;
+    *s = NULL;
   
-  track_priv = (track_priv_t*)(track->priv);
+  track_priv = track->priv;
 
   for(i = 0; i < track_priv->num_unsupported_streams; i++)
     {
@@ -1175,7 +1179,7 @@ static int64_t find_next_track(bgav_demuxer_context_t * ctx,
   bgav_stream_t * s;
   stream_priv_t * stream_priv;
   
-  priv = (ogg_t *)(ctx->priv);
+  priv = ctx->priv;
   
   /* Do bisection search */
   pos1 = start_pos;
@@ -1198,11 +1202,11 @@ static int64_t find_next_track(bgav_demuxer_context_t * ctx,
 
     page_pos_2 = find_first_page(ctx, test_pos,
                                  ctx->input->total_bytes, &serialno_2,
-                                 (int64_t*)0);
+                                 NULL);
     if(page_pos_2 < 0)
       return -1;
     
-    if(track_has_serialno(last_track, serialno_2, (bgav_stream_t**)0))
+    if(track_has_serialno(last_track, serialno_2, NULL))
       {
       //      pos1 = test_pos;
       pos1 = page_pos_2;
@@ -1228,7 +1232,7 @@ static int64_t find_next_track(bgav_demuxer_context_t * ctx,
 
     if(s)
       {
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
       stream_priv->last_granulepos = granulepos_1;
       }
     return page_pos_2;
@@ -1242,7 +1246,7 @@ static gavl_time_t granulepos_2_time(bgav_stream_t * s,
   int64_t frames;
 
   stream_priv_t * stream_priv;
-  stream_priv = (stream_priv_t *)(s->priv);
+  stream_priv = s->priv;
   switch(stream_priv->fourcc_priv)
     {
     case FOURCC_VORBIS:
@@ -1254,7 +1258,7 @@ static gavl_time_t granulepos_2_time(bgav_stream_t * s,
                                   pos);
       break;
     case FOURCC_THEORA:
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
 
       frames = pos >> stream_priv->keyframe_granule_shift;
       frames +=
@@ -1289,7 +1293,7 @@ static void get_last_granulepos(bgav_demuxer_context_t * ctx,
   int done = 0, i, serialno;
   bgav_stream_t * s;
   
-  track_priv = (track_priv_t*)(track->priv);
+  track_priv = track->priv;
 
   pos = track_priv->end_pos;
   
@@ -1299,7 +1303,7 @@ static void get_last_granulepos(bgav_demuxer_context_t * ctx,
     done = 1;
     for(i = 0; i < track->num_audio_streams; i++)
       {
-      stream_priv = (stream_priv_t*)(track->audio_streams[i].priv);
+      stream_priv = track->audio_streams[i].priv;
       if(stream_priv->last_granulepos < 0)
         {
         done = 0;
@@ -1310,7 +1314,7 @@ static void get_last_granulepos(bgav_demuxer_context_t * ctx,
       {
       for(i = 0; i < track->num_video_streams; i++)
         {
-        stream_priv = (stream_priv_t*)(track->video_streams[i].priv);
+        stream_priv = track->video_streams[i].priv;
         if(stream_priv->last_granulepos < 0)
           {
           done = 0;
@@ -1328,10 +1332,10 @@ static void get_last_granulepos(bgav_demuxer_context_t * ctx,
     if(pos < 0) /* Should never happen */
       break;
 
-    s = (bgav_stream_t*)0;
+    s = NULL;
     if(track_has_serialno(track, serialno, &s) && s)
       {
-      stream_priv = (stream_priv_t*)(s->priv);
+      stream_priv = s->priv;
       if(stream_priv->last_granulepos < 0)
         stream_priv->last_granulepos = granulepos;
       }
@@ -1348,18 +1352,19 @@ static void get_metadata(bgav_track_t * track)
   {
   stream_priv_t * stream_priv;
   int i;
-  bgav_metadata_free(&track->metadata);
+  gavl_metadata_free(&track->metadata);
 
   for(i = 0; i < track->num_audio_streams; i++)
     {
-    stream_priv = (stream_priv_t *)(track->audio_streams[i].priv);
-    bgav_metadata_merge2(&track->metadata, &stream_priv->metadata);
+    stream_priv = track->audio_streams[i].priv;
+    gavl_metadata_merge2(&track->metadata, &stream_priv->metadata);
     }
   for(i = 0; i < track->num_video_streams; i++)
     {
-    stream_priv = (stream_priv_t *)(track->video_streams[i].priv);
-    bgav_metadata_merge2(&track->metadata, &stream_priv->metadata);
+    stream_priv = track->video_streams[i].priv;
+    gavl_metadata_merge2(&track->metadata, &stream_priv->metadata);
     }
+  gavl_metadata_set(&track->metadata, GAVL_META_FORMAT, "Ogg");
   }
 
 static int open_ogg(bgav_demuxer_context_t * ctx)
@@ -1371,7 +1376,7 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
   bgav_stream_t * s;
   stream_priv_t * stream_priv;
   int64_t result, last_granulepos;
-  bgav_input_context_t * input_save = (bgav_input_context_t*)0;
+  bgav_input_context_t * input_save = NULL;
   
   ogg_t * priv;
   
@@ -1424,7 +1429,7 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
         {
         if(s && last_granulepos >= 0)
           {
-          stream_priv = (stream_priv_t*)(s->priv);
+          stream_priv = s->priv;
           stream_priv->last_granulepos = last_granulepos;
           }
         break;
@@ -1438,14 +1443,14 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
 
     /* Get the end positions of all tracks */
 
-    track_priv_1 = (track_priv_t*)(ctx->tt->tracks[0].priv);
+    track_priv_1 = ctx->tt->tracks[0].priv;
     for(i = 0; i < ctx->tt->num_tracks; i++)
       {
       if(i == ctx->tt->num_tracks-1)
         track_priv_1->end_pos = ctx->input->total_bytes;
       else
         {
-        track_priv_2 = (track_priv_t*)(ctx->tt->tracks[i+1].priv);
+        track_priv_2 = ctx->tt->tracks[i+1].priv;
         track_priv_1->end_pos = track_priv_2->start_pos;
         track_priv_1 = track_priv_2;
         }
@@ -1464,7 +1469,7 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
       for(j = 0; j < ctx->tt->tracks[i].num_audio_streams; j++)
         {
         stream_priv =
-          (stream_priv_t*)(ctx->tt->tracks[i].audio_streams[j].priv);
+          ctx->tt->tracks[i].audio_streams[j].priv;
 
         ogg_stream_reset(&stream_priv->os);
         
@@ -1479,7 +1484,7 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
       for(j = 0; j < ctx->tt->tracks[i].num_video_streams; j++)
         {
         stream_priv =
-          (stream_priv_t*)(ctx->tt->tracks[i].video_streams[j].priv);
+          ctx->tt->tracks[i].video_streams[j].priv;
 
         ogg_stream_reset(&stream_priv->os);
         
@@ -1496,7 +1501,7 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
           continue;
     
         stream_priv =
-          (stream_priv_t*)(ctx->tt->tracks[i].subtitle_streams[j].priv);
+          ctx->tt->tracks[i].subtitle_streams[j].priv;
 
         ogg_stream_reset(&stream_priv->os);
         }
@@ -1507,14 +1512,19 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
          metadata */
       if(ctx->tt->num_tracks > 1)
         {
-        if(ctx->tt->tracks[i].metadata.artist && ctx->tt->tracks[i].metadata.title)
-          ctx->tt->tracks[i].name = bgav_sprintf("%s - %s",
-                                                 ctx->tt->tracks[i].metadata.artist,
-                                                 ctx->tt->tracks[i].metadata.title);
-        else if(ctx->tt->tracks[i].metadata.title)
-          ctx->tt->tracks[i].name = bgav_sprintf("%s",
-                                                 ctx->tt->tracks[i].metadata.title);
+        const char * artist;
+        const char * title;
         
+        artist = gavl_metadata_get(&ctx->tt->tracks[i].metadata, GAVL_META_ARTIST);
+        title = gavl_metadata_get(&ctx->tt->tracks[i].metadata, GAVL_META_TITLE);
+        
+        
+        if(artist && title)
+          ctx->tt->tracks[i].name = bgav_sprintf("%s - %s",
+                                                 artist,
+                                                 title);
+        else if(title)
+          ctx->tt->tracks[i].name = bgav_sprintf("%s", title);
         else
           ctx->tt->tracks[i].name = bgav_sprintf("Track %d", i+1);
         }
@@ -1522,12 +1532,14 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
     }
   else /* Streaming case */
     {
-    if(ctx->input->metadata.title)
-      ctx->tt->tracks[0].name = bgav_strdup(ctx->input->metadata.title);
+    const char * title =
+      gavl_metadata_get(&ctx->tt->tracks[0].metadata, GAVL_META_TITLE);
+    if(title)
+      ctx->tt->tracks[0].name = bgav_strdup(title);
     get_metadata(&ctx->tt->tracks[0]);
 
     /* Set end position to -1 */
-    track_priv_1 = (track_priv_t*)(ctx->tt->tracks[0].priv);
+    track_priv_1 = ctx->tt->tracks[0].priv;
     track_priv_1->end_pos = -1;
 
     priv->is_live = 1;
@@ -1537,7 +1549,6 @@ static int open_ogg(bgav_demuxer_context_t * ctx)
   
   if(ctx->input->input->seek_byte)
     ctx->flags |= (BGAV_DEMUXER_CAN_SEEK|BGAV_DEMUXER_SEEK_ITERATIVE);
-  ctx->stream_description = bgav_strdup("Ogg bitstream");
   ctx->index_mode = INDEX_MODE_MIXED;
   return 1;
   }
@@ -1549,7 +1560,7 @@ static int new_streaming_track(bgav_demuxer_context_t * ctx)
   ogg_stream_state os;
   int serialno;
   int done, audio_done, video_done;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
     
   /*
    *  Ok, we try to get the new stuff, update the serial numbers from
@@ -1585,7 +1596,7 @@ static int new_streaming_track(bgav_demuxer_context_t * ctx)
     
     if(!audio_done)
       {
-      stream_priv = (stream_priv_t*)(ctx->tt->cur->audio_streams->priv);
+      stream_priv = ctx->tt->cur->audio_streams->priv;
       if(fourcc == stream_priv->fourcc_priv)
         {
         ogg_stream_init(&stream_priv->os, serialno);
@@ -1596,7 +1607,7 @@ static int new_streaming_track(bgav_demuxer_context_t * ctx)
       }
     if(!done && !video_done)
       {
-      stream_priv = (stream_priv_t*)(ctx->tt->cur->video_streams->priv);
+      stream_priv = ctx->tt->cur->video_streams->priv;
       if(fourcc == stream_priv->fourcc_priv)
         {
         ogg_stream_init(&stream_priv->os, serialno);
@@ -1611,24 +1622,30 @@ static int new_streaming_track(bgav_demuxer_context_t * ctx)
   return 1;
   }
 
-static char * get_name(bgav_metadata_t * m)
+static char * get_name(gavl_metadata_t * m)
   {
-  if(m->artist && m->title)
-    return bgav_sprintf("%s - %s", m->artist, m->title);
-  else if(m->title)
-    return bgav_sprintf("%s", m->title);
-  return (char*)0;
+  const char * artist;
+  const char * title;
+
+  artist = gavl_metadata_get(m, GAVL_META_ARTIST);
+  title = gavl_metadata_get(m, GAVL_META_TITLE);
+  
+  if(artist && title)
+    return bgav_sprintf("%s - %s", artist, title);
+  else if(title)
+    return bgav_sprintf("%s", title);
+  return NULL;
   }
 
 static void metadata_changed(bgav_demuxer_context_t * ctx)
   {
   char * name;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  ogg_t * priv = ctx->priv;
 
   if(ctx->opt->metadata_change_callback || ctx->opt->name_change_callback)
     {
     get_metadata(ctx->tt->cur);
-    bgav_metadata_merge2(&ctx->tt->cur->metadata, &ctx->input->metadata);
+    gavl_metadata_merge2(&ctx->tt->cur->metadata, &ctx->input->metadata);
     }
   
   if(ctx->opt->metadata_change_callback)
@@ -1668,7 +1685,7 @@ static void set_packet_pos(ogg_t * op, stream_priv_t * sp,
 
 static int check_header_packet(ogg_t * op, bgav_stream_t * s, ogg_packet * p)
   {
-  stream_priv_t * sp = (stream_priv_t *)s->priv;
+  stream_priv_t * sp = s->priv;
   switch(sp->fourcc_priv)
     {
     case FOURCC_THEORA:
@@ -1757,11 +1774,11 @@ static int next_packet_ogg(bgav_demuxer_context_t * ctx)
   int64_t iframes;
   int64_t pframes;
   int serialno;
-  bgav_packet_t * p = (bgav_packet_t *)0;
+  bgav_packet_t * p = NULL;
   bgav_stream_t * s;
   int64_t granulepos;
-  stream_priv_t * stream_priv = (stream_priv_t*)0;
-  ogg_t * priv = (ogg_t*)(ctx->priv);
+  stream_priv_t * stream_priv = NULL;
+  ogg_t * priv = ctx->priv;
   int subtitle_duration;
   int page_continued;
 
@@ -2077,12 +2094,15 @@ static int next_packet_ogg(bgav_demuxer_context_t * ctx)
         
           if(!check_header_packet(priv, s, &priv->op))
             break;
-        
+          
           p = bgav_stream_get_packet_write(s);
           bgav_packet_alloc(p, priv->op.bytes);
           memcpy(p->data, priv->op.packet, priv->op.bytes);
           p->data_size = priv->op.bytes;
 
+          //  fprintf(stderr, "Flac packet:\n");
+          //  bgav_hexdump(p->data, 16, 16);
+          
           if(stream_priv->prev_granulepos >= 0)
             {
             p->pts = stream_priv->prev_granulepos;
@@ -2157,7 +2177,7 @@ static void reset_track(bgav_track_t * track, int bos)
   for(i = 0; i < track->num_audio_streams; i++)
     {
     stream_priv =
-      (stream_priv_t*)(track->audio_streams[i].priv);
+      track->audio_streams[i].priv;
     stream_priv->prev_granulepos = position;
     stream_priv->frame_counter = position;
     stream_priv->do_sync = do_sync;
@@ -2170,7 +2190,7 @@ static void reset_track(bgav_track_t * track, int bos)
   for(i = 0; i < track->num_video_streams; i++)
     {
     stream_priv =
-      (stream_priv_t*)(track->video_streams[i].priv);
+      track->video_streams[i].priv;
 
     stream_priv->prev_granulepos = position;
     stream_priv->frame_counter = position;
@@ -2185,8 +2205,7 @@ static void reset_track(bgav_track_t * track, int bos)
     {
     if(track->subtitle_streams[i].data.subtitle.subreader)
       continue;
-    stream_priv =
-      (stream_priv_t*)(track->subtitle_streams[i].priv);
+    stream_priv = track->subtitle_streams[i].priv;
     ogg_stream_reset(&stream_priv->os);
     if(bos)
       STREAM_SET_SYNC(&track->subtitle_streams[i], 0);
@@ -2197,8 +2216,7 @@ static void resync_ogg(bgav_demuxer_context_t * ctx, bgav_stream_t * s)
   {
   stream_priv_t * stream_priv;
 
-  stream_priv =
-    (stream_priv_t*)(s->priv);
+  stream_priv = s->priv;
 
   switch(s->type)
     {
@@ -2243,17 +2261,17 @@ static void seek_ogg(bgav_demuxer_context_t * ctx, int64_t time, int scale)
     }
   
   /* Seek to the file position */
-  track_priv = (track_priv_t*)(ctx->tt->cur->priv);
+  track_priv = ctx->tt->cur->priv;
   filepos = track_priv->start_pos +
     (int64_t)((double)gavl_time_unscale(scale, time) /
               (double)(ctx->tt->cur->duration) *
               (track_priv->end_pos - track_priv->start_pos));
   if(filepos <= track_priv->start_pos)
     filepos = find_first_page(ctx, track_priv->start_pos, track_priv->end_pos,
-                              (int*)0, (int64_t*)0);
+                              NULL, NULL);
   else
     filepos = find_last_page(ctx, track_priv->start_pos, filepos,
-                             (int*)0, (int64_t*)0);
+                             NULL, NULL);
   seek_byte(ctx, filepos);
 
   /* Reset all the streams and set the stream times to -1 */
@@ -2267,7 +2285,7 @@ static void seek_ogg(bgav_demuxer_context_t * ctx, int64_t time, int scale)
     if(!next_packet_ogg(ctx)) 
       {
       filepos = find_last_page(ctx, track_priv->start_pos, filepos,
-                               (int*)0, (int64_t*)0);
+                               NULL, NULL);
       //      seek_byte(ctx, filepos);
       reset_track(ctx->tt->cur, 0);
       //      fprintf(stderr, "Reached EOF\n");
@@ -2277,8 +2295,7 @@ static void seek_ogg(bgav_demuxer_context_t * ctx, int64_t time, int scale)
 
     for(i = 0; i < ctx->tt->cur->num_audio_streams; i++)
       {
-      stream_priv =
-        (stream_priv_t*)(ctx->tt->cur->audio_streams[i].priv);
+      stream_priv = ctx->tt->cur->audio_streams[i].priv;
       
       if(stream_priv->do_sync &&
          (ctx->tt->cur->audio_streams[i].action != BGAV_STREAM_MUTE))
@@ -2288,8 +2305,7 @@ static void seek_ogg(bgav_demuxer_context_t * ctx, int64_t time, int scale)
       {
       for(i = 0; i < ctx->tt->cur->num_video_streams; i++)
         {
-        stream_priv =
-          (stream_priv_t*)(ctx->tt->cur->video_streams[i].priv);
+        stream_priv = ctx->tt->cur->video_streams[i].priv;
         if(stream_priv->do_sync &&
            (ctx->tt->cur->video_streams[i].action != BGAV_STREAM_MUTE))
           done = 0;
@@ -2310,13 +2326,13 @@ static void close_ogg(bgav_demuxer_context_t * ctx)
   
   for(i = 0; i < ctx->tt->num_tracks; i++)
     {
-    track_priv = (track_priv_t*)(ctx->tt->tracks[i].priv);
+    track_priv = ctx->tt->tracks[i].priv;
     if(track_priv->unsupported_streams)
       free(track_priv->unsupported_streams);
     free(track_priv);
     }
   
-  priv = (ogg_t *)ctx->priv;
+  priv = ctx->priv;
   ogg_sync_clear(&priv->oy);
 
   /* IMPORTANT: The paket will be freed by the last ogg_stream */
@@ -2331,8 +2347,7 @@ static void init_track(bgav_track_t * track)
   
   for(i = 0; i < track->num_audio_streams; i++)
     {
-    stream_priv =
-      (stream_priv_t*)(track->audio_streams[i].priv);
+    stream_priv = track->audio_streams[i].priv;
     stream_priv->prev_granulepos = 0;
     stream_priv->frame_counter = 0;
     stream_priv->do_sync = 0;
@@ -2341,8 +2356,7 @@ static void init_track(bgav_track_t * track)
 
   for(i = 0; i < track->num_video_streams; i++)
     {
-    stream_priv =
-      (stream_priv_t*)(track->video_streams[i].priv);
+    stream_priv = track->video_streams[i].priv;
     stream_priv->prev_granulepos = 0;
     stream_priv->frame_counter = 0;
     stream_priv->do_sync = 0;
@@ -2353,8 +2367,7 @@ static void init_track(bgav_track_t * track)
     {
     if(track->subtitle_streams[i].data.subtitle.subreader)
       continue;
-    stream_priv =
-      (stream_priv_t*)(track->subtitle_streams[i].priv);
+    stream_priv = track->subtitle_streams[i].priv;
     ogg_stream_reset(&stream_priv->os);
     }
   }
@@ -2366,9 +2379,9 @@ static int select_track_ogg(bgav_demuxer_context_t * ctx,
   ogg_t * priv;
   track_priv_t * track_priv;
   
-  priv = (ogg_t *)(ctx->priv);
+  priv = ctx->priv;
   
-  track_priv = (track_priv_t*)(ctx->tt->cur->priv);
+  track_priv = ctx->tt->cur->priv;
   
   if(ctx->input->input->seek_byte)
     {
