@@ -305,7 +305,7 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx,
   bgav_track_t * track = ctx->tt->cur;
 
   rm_private_t * priv;
-  priv = (rm_private_t*)(ctx->priv);
+  priv = ctx->priv;
 
     
   bg_as = bgav_track_add_audio_stream(track, ctx->opt);
@@ -407,14 +407,9 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx,
           data++;
 
         codecdata_length = BGAV_PTR_2_32BE(data);data+=4;
-      
-        bg_as->ext_size = codecdata_length;
-        if(bg_as->ext_size)
-          {
-          bg_as->ext_data = malloc(bg_as->ext_size);
-          memcpy(bg_as->ext_data, data, codecdata_length);
-          }
 
+        bgav_stream_set_extradata(bg_as, data, codecdata_length);
+        
         if((bg_as->fourcc == BGAV_MK_FOURCC('a', 't', 'r', 'c')) ||
            (bg_as->fourcc == BGAV_MK_FOURCC('c', 'o', 'o', 'k')))
           bg_as->data.audio.block_align = sub_packet_size;
@@ -433,16 +428,9 @@ static void init_audio_stream(bgav_demuxer_context_t * ctx,
           data++;
 
         codecdata_length = BGAV_PTR_2_32BE(data);data+=4;
+
         if(codecdata_length>=1)
-          {
-          bg_as->ext_size = codecdata_length-1;
-          data++;
-          if(bg_as->ext_size)
-            {
-            bg_as->ext_data = malloc(bg_as->ext_size);
-            memcpy(bg_as->ext_data, data, bg_as->ext_size);
-            }
-          }
+          bgav_stream_set_extradata(bg_as, data+1, codecdata_length-1);
         break;
       default:
       
@@ -485,7 +473,7 @@ static void init_audio_stream_mp3(bgav_demuxer_context_t * ctx, bgav_rmff_stream
   rm_audio_stream_t * rm_as;
   rm_as = calloc(1, sizeof(*rm_as));
 
-  priv = (rm_private_t*)(ctx->priv);
+  priv = ctx->priv;
   
   bg_as = bgav_track_add_audio_stream(track, ctx->opt);
   bg_as->cleanup = cleanup_stream_rm;
@@ -508,7 +496,7 @@ static void init_audio_stream_mp3(bgav_demuxer_context_t * ctx, bgav_rmff_stream
 
 static void init_video_stream(bgav_demuxer_context_t * ctx,
                               bgav_rmff_stream_t * stream,
-                              uint8_t * _data, int len)
+                              uint8_t * data_start, int len)
   {
   uint16_t fps, fps2;
   bgav_stream_t * bg_vs;
@@ -517,14 +505,13 @@ static void init_video_stream(bgav_demuxer_context_t * ctx,
   
   bgav_track_t * track = ctx->tt->cur;
 
-  uint8_t * data = _data;
+  uint8_t * data = data_start;
   
-  priv = (rm_private_t*)(ctx->priv);
+  priv = ctx->priv;
   
   bg_vs = bgav_track_add_video_stream(track, ctx->opt);
   bg_vs->cleanup = cleanup_stream_rm;
 
-  bg_vs->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
   bg_vs->flags |= STREAM_NO_DURATIONS;
   
   rm_vs = calloc(1, sizeof(*rm_vs));
@@ -532,8 +519,6 @@ static void init_video_stream(bgav_demuxer_context_t * ctx,
   /* Set container bitrate */
   bg_vs->container_bitrate = stream->mdpr.avg_bit_rate;
   
-  //  bg_vs->ext_size = 16;
-  //  bg_vs->ext_data = calloc(bg_vs->ext_size, 1);
   bg_vs->priv = rm_vs;
 
   data += 4; // VIDO
@@ -568,11 +553,8 @@ static void init_video_stream(bgav_demuxer_context_t * ctx,
   
   /* Read extradata */
 
-  bg_vs->ext_size = len - (data - _data);
-  bg_vs->ext_data = malloc(bg_vs->ext_size);
-
-  memcpy(bg_vs->ext_data, data, bg_vs->ext_size);
-
+  bgav_stream_set_extradata(bg_vs, data, len - (data - data_start));
+  
   switch(bg_vs->fourcc)
     {
     case BGAV_MK_FOURCC('R','V','1','0'):
@@ -717,7 +699,7 @@ static int get_multirate_offsets(bgav_demuxer_context_t * ctx,
 
   old_pos = ctx->input->position;
     
-  priv = (rm_private_t*)(ctx->priv);
+  priv = ctx->priv;
   
   /* Look in each logical stream description for the stream number */
   for(i = 0; i < priv->header->num_streams; i++)
@@ -799,7 +781,8 @@ int bgav_demux_rm_open_with_header(bgav_demuxer_context_t * ctx,
   bgav_rmff_mdpr_t * mdpr;
   uint32_t header = 0;
   bgav_track_t * track;
-
+  const char * title;
+  
   bgav_charset_converter_t * cnv;
 
   //  bgav_rmff_header_dump(h);
@@ -813,8 +796,8 @@ int bgav_demux_rm_open_with_header(bgav_demuxer_context_t * ctx,
   ctx->tt = bgav_track_table_create(1);
   track = ctx->tt->cur;
 
-  if(ctx->input->metadata.title)
-    track->name = bgav_strdup(ctx->input->metadata.title);
+  if((title = gavl_metadata_get(&ctx->input->metadata, GAVL_META_TITLE)))
+    track->name = bgav_strdup(title);
     
   /* Create streams */
 
@@ -893,33 +876,43 @@ int bgav_demux_rm_open_with_header(bgav_demuxer_context_t * ctx,
       }
     }
 
-  ctx->stream_description = bgav_sprintf("Real Media file format");
-
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "RM");
+  
   /* Handle metadata */
 
   cnv = bgav_charset_converter_create(ctx->opt, "ISO-8859-1", "UTF-8");
 
   if(priv->header->cont.title_len)
-    track->metadata.title = bgav_convert_string(cnv,
+    gavl_metadata_set_nocpy(&track->metadata,
+                            GAVL_META_TITLE,
+                            bgav_convert_string(cnv,
                                                 priv->header->cont.title,
                                                 priv->header->cont.title_len,
-                                                NULL);
+                                                NULL));
   if(priv->header->cont.author_len)
-    track->metadata.author = bgav_convert_string(cnv,
+    gavl_metadata_set_nocpy(&track->metadata,
+                            GAVL_META_AUTHOR,
+                            bgav_convert_string(cnv,
                                                 priv->header->cont.author,
                                                 priv->header->cont.author_len,
-                                                NULL);
+                                                NULL));
+  
   if(priv->header->cont.copyright_len)
-    track->metadata.copyright = bgav_convert_string(cnv,
+    gavl_metadata_set_nocpy(&track->metadata,
+                            GAVL_META_COPYRIGHT,
+                            bgav_convert_string(cnv,
                                                 priv->header->cont.copyright,
                                                 priv->header->cont.copyright_len,
-                                                NULL);
-
+                                                NULL));
+  
   if(priv->header->cont.comment_len)
-    track->metadata.comment = bgav_convert_string(cnv,
+    gavl_metadata_set_nocpy(&track->metadata,
+                            GAVL_META_COMMENT,
+                            bgav_convert_string(cnv,
                                                 priv->header->cont.comment,
                                                 priv->header->cont.comment_len,
-                                                NULL);
+                                                NULL));
   bgav_charset_converter_destroy(cnv);
 
   ctx->data_start = ctx->input->position;
@@ -977,7 +970,7 @@ fix_timestamp(bgav_stream_t * stream, uint8_t * s, uint32_t timestamp, bgav_pack
   int kf=timestamp;
   int pict_type;
   int orig_kf;
-  rm_video_stream_t * priv = (rm_video_stream_t*)stream->priv;
+  rm_video_stream_t * priv = stream->priv;
 #if 1
   if(stream->fourcc==BGAV_MK_FOURCC('R','V','3','0') ||
      stream->fourcc==BGAV_MK_FOURCC('R','V','4','0'))
@@ -1283,7 +1276,7 @@ static int process_audio_chunk(bgav_demuxer_context_t * ctx,
   uint16_t aac_packet_lengths[16];
   uint8_t num_aac_packets;
   
-  as = (rm_audio_stream_t*)stream->priv;
+  as = stream->priv;
   
   packet_size = PAYLOAD_LENGTH(h);
 
@@ -1423,15 +1416,15 @@ static int process_audio_chunk(bgav_demuxer_context_t * ctx,
 static int next_packet_rmff(bgav_demuxer_context_t * ctx)
   {
   //  bgav_packet_t * p;
-  bgav_stream_t * stream = (bgav_stream_t*)0;
+  bgav_stream_t * stream = NULL;
   rm_private_t * rm;
   bgav_rmff_packet_header_t h;
-  rm_stream_t * rs = (rm_stream_t*)0;
+  rm_stream_t * rs = NULL;
   int result = 0;
   uint32_t stream_pos = 0;
   int i;
   
-  rm = (rm_private_t*)(ctx->priv);
+  rm = ctx->priv;
 
   if(!rm->is_multirate)
     {
@@ -1469,7 +1462,7 @@ static int next_packet_rmff(bgav_demuxer_context_t * ctx)
       {
       stream = ctx->request_stream;
 
-      rs = (rm_stream_t*)(stream->priv);
+      rs = stream->priv;
 
       if(rs->data_pos >= rs->data_end)
         return 0;
@@ -1482,13 +1475,13 @@ static int next_packet_rmff(bgav_demuxer_context_t * ctx)
       for(i = 0; i < ctx->tt->cur->num_audio_streams; i++)
         {
         stream = &ctx->tt->cur->audio_streams[i];
-        rs = (rm_stream_t*)(stream->priv);
+        rs = stream->priv;
         
         if((stream->action == BGAV_STREAM_MUTE) ||
            !bgav_packet_buffer_is_empty(stream->packet_buffer) ||
            rs->data_pos >= rs->data_end)
           {
-          stream = (bgav_stream_t*)0;
+          stream = NULL;
           }
         else
           {
@@ -1501,13 +1494,13 @@ static int next_packet_rmff(bgav_demuxer_context_t * ctx)
         for(i = 0; i < ctx->tt->cur->num_video_streams; i++)
           {
           stream = &ctx->tt->cur->video_streams[i];
-          rs = (rm_stream_t*)(stream->priv);
+          rs = stream->priv;
           
           if((stream->action == BGAV_STREAM_MUTE) ||
              !bgav_packet_buffer_is_empty(stream->packet_buffer) ||
              rs->data_pos >= rs->data_end)
             {
-            stream = (bgav_stream_t*)0;
+            stream = NULL;
             }
           else
             {
@@ -1551,7 +1544,7 @@ static int next_packet_rmff(bgav_demuxer_context_t * ctx)
   
   if(stream->type == BGAV_STREAM_VIDEO)
     {
-    rs = (rm_stream_t*)(stream->priv);
+    rs = stream->priv;
     if(rm->do_seek)
       {
       if(rs->stream->indx.records[rs->index_record].packet_count_for_this_packet > rm->next_packet)
@@ -1568,7 +1561,7 @@ static int next_packet_rmff(bgav_demuxer_context_t * ctx)
     }
   else if(stream->type == BGAV_STREAM_AUDIO)
     {
-    rs = (rm_stream_t*)(stream->priv);
+    rs = stream->priv;
     if(rm->do_seek)
       {
       if(rs->stream->indx.records[rs->index_record].packet_count_for_this_packet > rm->next_packet)
@@ -1602,7 +1595,7 @@ static void seek_rmff(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   uint32_t end_packet =    0x00;
   
   
-  rm = (rm_private_t*)(ctx->priv);
+  rm = ctx->priv;
   track = ctx->tt->cur;
   
   real_time = gavl_time_rescale(scale, 1000, time);
@@ -1615,7 +1608,7 @@ static void seek_rmff(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   for(i = 0; i < track->num_video_streams; i++)
     {
     stream = &track->video_streams[i];
-    vs = (rm_video_stream_t*)(stream->priv);
+    vs = stream->priv;
     vs->com.index_record = seek_indx(&vs->com.stream->indx, real_time,
                                  &position, &start_packet, &end_packet);
     STREAM_SET_SYNC(stream, vs->com.stream->indx.records[vs->com.index_record].timestamp);
@@ -1626,7 +1619,7 @@ static void seek_rmff(bgav_demuxer_context_t * ctx, int64_t time, int scale)
   for(i = 0; i < track->num_audio_streams; i++)
     {
     stream = &track->audio_streams[i];
-    rs = (rm_stream_t*)(stream->priv);
+    rs = stream->priv;
     rs->index_record = seek_indx(&rs->stream->indx, real_time,
                                  &position, &start_packet, &end_packet);
     STREAM_SET_SYNC(stream, rs->stream->indx.records[rs->index_record].timestamp);
@@ -1653,7 +1646,7 @@ static void seek_rmff(bgav_demuxer_context_t * ctx, int64_t time, int scale)
 static void close_rmff(bgav_demuxer_context_t * ctx)
   {
   rm_private_t * priv;
-  priv = (rm_private_t *)ctx->priv;
+  priv = ctx->priv;
   
   if(priv)
     {
@@ -1670,7 +1663,7 @@ static int select_track_rmff(bgav_demuxer_context_t * ctx, int t)
   rm_private_t * priv;
   bgav_track_t * track;
   int i;
-  priv = (rm_private_t *)ctx->priv;
+  priv = ctx->priv;
   
   priv->next_packet = 0;
   
@@ -1680,7 +1673,7 @@ static int select_track_rmff(bgav_demuxer_context_t * ctx, int t)
   
   for(i = 0; i < track->num_audio_streams; i++)
     {
-    as = (rm_audio_stream_t*)(track->audio_streams[i].priv);
+    as = track->audio_streams[i].priv;
     if(as)
       {
       as->sub_packet_cnt = 0;
@@ -1689,7 +1682,7 @@ static int select_track_rmff(bgav_demuxer_context_t * ctx, int t)
     }
   for(i = 0; i < track->num_video_streams; i++)
     {
-    vs = (rm_video_stream_t*)(track->video_streams[i].priv);
+    vs = track->video_streams[i].priv;
     if(vs)
       {
       //      vs->kf_pts = 0;
