@@ -152,7 +152,6 @@ static int check_key(char * buffer, const char * key, char ** pos)
   return 0;
   }
 
-#if DUMP_HEADERS
 static void vivo_header_dump(vivo_header_t * h)
   {
   bgav_dprintf( "Main VIVO header\n");
@@ -193,15 +192,13 @@ static void vivo_header_dump(vivo_header_t * h)
   
   }
 
-#endif
-
 static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
   {
   uint8_t c;
   int len;
   int64_t header_start;
-  char * buffer = (char*)0;
-  char * pos = (char*)0;
+  char * buffer = NULL;
+  char * pos = NULL;
   int result = 0;
   int record_type;
     
@@ -223,7 +220,7 @@ static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
   
   while(input->position < header_start + len)
     {
-    if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, (int*)0))
+    if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, NULL))
       goto fail;
 
     if(check_key(buffer, "Version", &pos))
@@ -291,7 +288,7 @@ static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
     record_type = -1;
     while(input->position < header_start + len)
       {
-      if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, (int*)0))
+      if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, NULL))
         goto fail;
 
       /* Skip empty lines */
@@ -316,7 +313,7 @@ static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
       case 2:
         while(input->position < header_start + len)
           {
-          if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, (int*)0))
+          if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, NULL))
             goto fail;
 
           if(check_key(buffer, "TimestampType", &pos))
@@ -341,7 +338,7 @@ static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
       case 4:
         while(input->position < header_start + len)
           {
-          if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, (int*)0))
+          if(!bgav_input_read_line(input, &buffer, &buffer_alloc, 0, NULL))
             goto fail;
 
           if(check_key(buffer, "Length", &pos))
@@ -367,9 +364,8 @@ static int vivo_header_read(vivo_header_t * ret, bgav_input_context_t * input)
     }
 
   result = 1;
-#if DUMP_HEADERS
-  vivo_header_dump(ret);
-#endif
+  if(input->opt->dump_headers)
+    vivo_header_dump(ret);
   fail:
   if(buffer)
     free(buffer);
@@ -416,8 +412,8 @@ static int open_vivo(bgav_demuxer_context_t * ctx)
   {
   
   vivo_priv_t * priv;
-  bgav_stream_t * audio_stream = (bgav_stream_t*)0;
-  bgav_stream_t * video_stream = (bgav_stream_t*)0;
+  bgav_stream_t * audio_stream = NULL;
+  bgav_stream_t * video_stream = NULL;
   
   priv = calloc(1, sizeof(*priv));
   ctx->priv = priv;
@@ -459,7 +455,6 @@ static int open_vivo(bgav_demuxer_context_t * ctx)
   /* Set up video stream */
   
   video_stream = bgav_track_add_video_stream(ctx->tt->cur, ctx->opt);
-  video_stream->data.video.frametime_mode = BGAV_FRAMETIME_PTS;
   video_stream->flags |= STREAM_NO_DURATIONS;
 
   if(priv->header.version == 1)
@@ -492,13 +487,17 @@ static int open_vivo(bgav_demuxer_context_t * ctx)
     video_stream->data.video.format.image_height * 3;
   /* Set up metadata */
 
-  ctx->tt->cur->metadata.title     = bgav_strdup(priv->header.title);
-  ctx->tt->cur->metadata.author    = bgav_strdup(priv->header.author);
-  ctx->tt->cur->metadata.copyright = bgav_strdup(priv->header.copyright);
-  ctx->tt->cur->metadata.comment   = bgav_sprintf("Made with %s",
-                                                            priv->header.producer);
+  gavl_metadata_set(&ctx->tt->cur->metadata,
+                    GAVL_META_TITLE, priv->header.title);
+  gavl_metadata_set(&ctx->tt->cur->metadata,
+                    GAVL_META_AUTHOR, priv->header.author);
+  gavl_metadata_set(&ctx->tt->cur->metadata,
+                    GAVL_META_COPYRIGHT, priv->header.copyright);
+  gavl_metadata_set(&ctx->tt->cur->metadata,
+                    GAVL_META_SOFTWARE, priv->header.producer);
 
-  ctx->stream_description = bgav_sprintf("Vivo Version %d.x", priv->header.version);
+  gavl_metadata_set(&ctx->tt->cur->metadata, 
+                    GAVL_META_FORMAT, "Vivo");
 
   ctx->tt->cur->duration = (GAVL_TIME_SCALE * (int64_t)(priv->header.duration)) / 1000;
 
@@ -517,12 +516,12 @@ static int next_packet_vivo(bgav_demuxer_context_t * ctx)
   int prefix = 0;
   int len;
   int seq;
-  bgav_stream_t * stream = (bgav_stream_t*)0;
+  bgav_stream_t * stream = NULL;
   vivo_priv_t * priv;
   int do_audio = 0;
   int do_video = 0;
   
-  priv = (vivo_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   if(!bgav_input_read_data(ctx->input, &c, 1))
     return 0;
@@ -604,7 +603,7 @@ static int next_packet_vivo(bgav_demuxer_context_t * ctx)
         ctx->tt->cur->audio_streams[0].container_bitrate;
       }
 
-    stream->packet = (bgav_packet_t*)0;
+    stream->packet = NULL;
     }
 
   /* Get new packet */
@@ -634,7 +633,7 @@ static int next_packet_vivo(bgav_demuxer_context_t * ctx)
 static int select_track_vivo(bgav_demuxer_context_t * ctx, int t)
   {
   vivo_priv_t * priv;
-  priv = (vivo_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   priv->audio_pos = 0;
   return 1;
   }
@@ -642,7 +641,7 @@ static int select_track_vivo(bgav_demuxer_context_t * ctx, int t)
 static void close_vivo(bgav_demuxer_context_t * ctx)
   {
   vivo_priv_t * priv;
-  priv = (vivo_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   vivo_header_free(&priv->header);
   free(priv);
