@@ -37,6 +37,7 @@ extern const bgav_demuxer_t bgav_demuxer_vivo;
 extern const bgav_demuxer_t bgav_demuxer_fli;
 extern const bgav_demuxer_t bgav_demuxer_flv;
 
+extern const bgav_demuxer_t bgav_demuxer_ape;
 extern const bgav_demuxer_t bgav_demuxer_wavpack;
 extern const bgav_demuxer_t bgav_demuxer_tta;
 extern const bgav_demuxer_t bgav_demuxer_voc;
@@ -119,6 +120,7 @@ static const demuxer_t demuxers[] =
     { &bgav_demuxer_rmff,      "Real Media" },
     { &bgav_demuxer_ra,        "Real Audio" },
     { &bgav_demuxer_quicktime, "Quicktime/mp4/m4a" },
+    { &bgav_demuxer_ape,       "APE"               },
     { &bgav_demuxer_wav,       "WAV" },
     { &bgav_demuxer_au,        "Sun AU" },
     { &bgav_demuxer_aiff,      "AIFF(C)" },
@@ -240,7 +242,7 @@ const bgav_demuxer_t * bgav_demuxer_probe(bgav_input_context_t * input,
       {
       if(demuxers[i].demuxer->probe(input))
         {
-        bgav_log(input->opt, BGAV_LOG_INFO, LOG_DOMAIN,
+        bgav_log(input->opt, BGAV_LOG_DEBUG, LOG_DOMAIN,
                  "Detected %s format", demuxers[i].format_name);
         return demuxers[i].demuxer;
         }
@@ -250,7 +252,7 @@ const bgav_demuxer_t * bgav_demuxer_probe(bgav_input_context_t * input,
       {
       if(sync_demuxers[i].demuxer->probe(input))
         {
-        bgav_log(input->opt, BGAV_LOG_INFO, LOG_DOMAIN,
+        bgav_log(input->opt, BGAV_LOG_DEBUG, LOG_DOMAIN,
                  "Detected %s format",
                  sync_demuxers[i].format_name);
         return sync_demuxers[i].demuxer;
@@ -263,7 +265,7 @@ const bgav_demuxer_t * bgav_demuxer_probe(bgav_input_context_t * input,
       {
       if(yml_demuxers[i].demuxer->probe_yml(yml))
         {
-        bgav_log(input->opt, BGAV_LOG_INFO, LOG_DOMAIN,
+        bgav_log(input->opt, BGAV_LOG_DEBUG, LOG_DOMAIN,
                  "Detected %s format",
                  yml_demuxers[i].format_name);
         return yml_demuxers[i].demuxer;
@@ -279,7 +281,7 @@ const bgav_demuxer_t * bgav_demuxer_probe(bgav_input_context_t * input,
     {
     bytes_skipped++;
     if(!bgav_input_read_data(input, &skip, 1))
-      return (bgav_demuxer_t *)0;
+      return NULL;
 
     for(i = 0; i < num_sync_demuxers; i++)
       {
@@ -303,7 +305,7 @@ const bgav_demuxer_t * bgav_demuxer_probe(bgav_input_context_t * input,
     }
 #endif
   
-  return (bgav_demuxer_t *)0;
+  return NULL;
   }
 
 bgav_demuxer_context_t *
@@ -336,9 +338,7 @@ void bgav_demuxer_destroy(bgav_demuxer_context_t * ctx)
 
   if(ctx->redirector)
     bgav_redirector_destroy(ctx->redirector);
-  
-  
-  FREE(ctx->stream_description);
+    
   free(ctx);
   }
 
@@ -489,9 +489,8 @@ int bgav_demuxer_start(bgav_demuxer_context_t * ctx,
 void bgav_demuxer_stop(bgav_demuxer_context_t * ctx)
   {
   ctx->demuxer->close(ctx);
-  ctx->priv = (void*)0;
-  FREE(ctx->stream_description);
-  
+  ctx->priv = NULL;
+    
   /* Reset global variables */
   ctx->flags &= ~(BGAV_DEMUXER_SI_SEEKING |
                   BGAV_DEMUXER_HAS_TIMESTAMP_OFFSET);
@@ -500,7 +499,7 @@ void bgav_demuxer_stop(bgav_demuxer_context_t * ctx)
   if(ctx->si)
     {
     bgav_superindex_destroy(ctx->si);
-    ctx->si = (bgav_superindex_t*)0;
+    ctx->si = NULL;
     }
   
   }
@@ -574,7 +573,7 @@ int bgav_demuxer_next_packet_interleaved(bgav_demuxer_context_t * ctx)
 static int next_packet_noninterleaved(bgav_demuxer_context_t * ctx)
   {
   bgav_packet_t * p;
-  bgav_stream_t * s = (bgav_stream_t*)0;
+  bgav_stream_t * s = NULL;
 
   s = ctx->request_stream;
   
@@ -656,7 +655,7 @@ int bgav_demuxer_next_packet(bgav_demuxer_context_t * demuxer)
             {
             bgav_stream_done_packet_write(&demuxer->tt->cur->audio_streams[i],
                                           demuxer->tt->cur->audio_streams[i].packet);
-            demuxer->tt->cur->audio_streams[i].packet = (bgav_packet_t*)0;
+            demuxer->tt->cur->audio_streams[i].packet = NULL;
             ret = 1;
             }
           }
@@ -666,7 +665,7 @@ int bgav_demuxer_next_packet(bgav_demuxer_context_t * demuxer)
             {
             bgav_stream_done_packet_write(&demuxer->tt->cur->video_streams[i],
                                           demuxer->tt->cur->video_streams[i].packet);
-            demuxer->tt->cur->video_streams[i].packet = (bgav_packet_t*)0;
+            demuxer->tt->cur->video_streams[i].packet = NULL;
             ret = 1;
             }
           }
@@ -678,96 +677,6 @@ int bgav_demuxer_next_packet(bgav_demuxer_context_t * demuxer)
   return ret;
   }
 
-#if 0
-bgav_packet_t *
-bgav_demuxer_peek_packet_read_generic(bgav_demuxer_context_t * demuxer,
-                                      bgav_stream_t * s, int force)
-  {
-  bgav_packet_t * ret;
-  int get_duration = 0;
-  
-  if((s->type == BGAV_STREAM_VIDEO) &&
-     (s->data.video.frametime_mode == BGAV_FRAMETIME_PTS))
-    get_duration = 1;
-  
-  //  fprintf(stderr, "bgav_stream_peek_packet_read: %d %d\n",
-  //          force, get_duration);
-  
-  if((demuxer->flags & BGAV_DEMUXER_PEEK_FORCES_READ) || force)
-    {
-    demuxer->request_stream = s; 
-    while(!(ret = bgav_packet_buffer_peek_packet_read(s->packet_buffer,
-                                                      get_duration)))
-      {
-      if(!bgav_demuxer_next_packet(demuxer))
-        {
-        if(get_duration)
-          {
-          ret = bgav_packet_buffer_peek_packet_read(s->packet_buffer, 0);
-          if(!ret)
-            return (bgav_packet_t*)0;
-
-          if(s->duration)
-            ret->duration = s->duration - ret->pts;
-          else
-            ret->duration = 0;
-          break;
-          }
-        else
-          return (bgav_packet_t*)0;
-        }
-      }
-    demuxer->request_stream = (bgav_stream_t*)0;
-    return ret;
-    }
-  else
-    return bgav_packet_buffer_peek_packet_read(s->packet_buffer, get_duration);
-  }
-
-bgav_packet_t *
-bgav_demuxer_get_packet_read_generic(bgav_demuxer_context_t * demuxer,
-                                     bgav_stream_t * s)
-  {
-  int get_duration = 0;
-  bgav_packet_t * ret;
-  
-  if((s->type == BGAV_STREAM_VIDEO) &&
-     (s->data.video.frametime_mode == BGAV_FRAMETIME_PTS))
-    get_duration = 1;
-  
-  while(!(ret = bgav_packet_buffer_get_packet_read(s->packet_buffer,
-                                                   get_duration)))
-    {
-    if(!bgav_demuxer_next_packet(demuxer))
-      {
-      if(get_duration)
-        {
-        ret = bgav_packet_buffer_get_packet_read(s->packet_buffer, 0);
-        if(!ret)
-          return (bgav_packet_t*)0;
-
-        if(s->duration)
-          ret->duration = s->duration - ret->pts;
-        else
-          ret->duration = 0;
-        break;
-        }
-      else
-        return (bgav_packet_t*)0;
-      }
-    }
-  /* Hack: If we got the duration from pts only and the stream
-   * has B-frames, the durations are invalid and set as such
-   */
-#if 1
-  if(get_duration && (ret->dts == BGAV_TIMESTAMP_UNDEFINED) &&
-     (s->flags & STREAM_B_FRAMES))
-    ret->duration = -1;
-#endif
-  return ret;
-  }
-#endif
-
 bgav_packet_t *
 bgav_demuxer_get_packet_read(void * stream1)
   {
