@@ -173,28 +173,20 @@ void bgav_id3v2_dump(bgav_id3v2_tag_t * t)
   /* Dump header */
 
   bgav_dprintf( "Header:\n");
-  bgav_dprintf( "  Major .version = %d\n", t->header.major_version);
-  bgav_dprintf( "  Minor .version = %d\n", t->header.minor_version);
+  bgav_dprintf( "  Major version: %d\n", t->header.major_version);
+  bgav_dprintf( "  Minor version: %d\n", t->header.minor_version);
   bgav_dprintf( "  Flags:         ");
   if(t->header.flags & ID3V2_TAG_UNSYNCHRONIZED)
-    {
     bgav_dprintf( "UNSYNCHRONIZED ");
-    }
   if(t->header.flags & ID3V2_TAG_EXTENDED_HEADER)
-    {
     bgav_dprintf( " EXTENDED_HEADER");
-    }
   if(t->header.flags & ID3V2_TAG_EXPERIMENTAL)
-    {
     bgav_dprintf( " EXPERIMENTAL");
-    }
   if(t->header.flags & ID3V2_TAG_EXPERIMENTAL)
-    {
     bgav_dprintf( " FOOTER_PRESENT");
-    }
   bgav_dprintf( "\n");
   bgav_dprintf( "  Size: %d\n", t->header.size);
-
+  
   for(i = 0; i < t->num_frames; i++)
     {
     bgav_dprintf( "========== Frame %d ==========\n", i+1);
@@ -253,11 +245,11 @@ static char ** read_string_list(const bgav_options_t * opt,
   int num_strings;
   char ** ret;
   bgav_charset_converter_t * cnv;
-  cnv = (bgav_charset_converter_t*)0;
+  cnv = NULL;
   encoding = *data;
 
   if(data_size == 1)
-    return (char**)0;
+    return NULL;
   
   switch(encoding)
     {
@@ -285,7 +277,7 @@ static char ** read_string_list(const bgav_options_t * opt,
       pos = ((char*)data) + 1;
       break;
     default:
-      return (char **)0;
+      return NULL;
     }
   
   end_pos = pos;
@@ -416,7 +408,7 @@ bgav_id3v2_tag_t * bgav_id3v2_read(bgav_input_context_t * input)
   uint8_t * data;
   int data_size;
       
-  bgav_id3v2_tag_t * ret = (bgav_id3v2_tag_t *)0;
+  bgav_id3v2_tag_t * ret = NULL;
   
   if(bgav_input_read_data(input, probe_data, 3) < 3)
     goto fail;
@@ -519,7 +511,7 @@ bgav_id3v2_tag_t * bgav_id3v2_read(bgav_input_context_t * input)
   fail:
   if(ret)
     bgav_id3v2_destroy(ret);
-  return (bgav_id3v2_tag_t *)0;
+  return NULL;
   }
 
 static bgav_id3v2_frame_t * bgav_id3v2_find_frame(bgav_id3v2_tag_t*t,
@@ -541,7 +533,7 @@ static bgav_id3v2_frame_t * bgav_id3v2_find_frame(bgav_id3v2_tag_t*t,
       j++;
       }
     }
-  return (bgav_id3v2_frame_t*)0;
+  return NULL;
   }
 
 static const uint32_t title_tags[] =
@@ -573,7 +565,14 @@ static const uint32_t artist_tags[] =
     0x00,
   };
 
-static const uint32_t date_tags[] =
+static const uint32_t albumartist_tags[] =
+  {
+    BGAV_MK_FOURCC('T','P','E','2'),
+    BGAV_MK_FOURCC('T','P','2',0x00),
+    0x00,
+  };
+
+static const uint32_t year_tags[] =
   {
     BGAV_MK_FOURCC('T','Y','E',0x00),
     BGAV_MK_FOURCC('T','Y','E','R'),
@@ -618,11 +617,10 @@ static char * get_comment(const bgav_options_t * opt,
   uint8_t encoding;
   uint8_t * pos;
   int bytes_per_char;
-  bgav_charset_converter_t * cnv =
-    (bgav_charset_converter_t*)0;
+  bgav_charset_converter_t * cnv = NULL;
   
   if(!frame->data)
-    return (char*)0;
+    return NULL;
   
   encoding = *frame->data;
     
@@ -652,7 +650,7 @@ static char * get_comment(const bgav_options_t * opt,
       pos = frame->data + 4;
       break;
     default:
-      return (char *)0;
+      return NULL;
     }
   
   pos = frame->data + 4; /* Skip encoding and language */
@@ -676,7 +674,7 @@ static char * get_comment(const bgav_options_t * opt,
   return ret;
   }
 
-void bgav_id3v2_2_metadata(bgav_id3v2_tag_t * t, bgav_metadata_t*m)
+void bgav_id3v2_2_metadata(bgav_id3v2_tag_t * t, gavl_metadata_t*m)
   {
   int i_tmp;
   bgav_id3v2_frame_t * frame;
@@ -685,44 +683,50 @@ void bgav_id3v2_2_metadata(bgav_id3v2_tag_t * t, bgav_metadata_t*m)
 
   frame = bgav_id3v2_find_frame(t, title_tags);
   if(frame && frame->strings)
-    m->title = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_TITLE, frame->strings[0]);
 
   /* Album */
     
   frame = bgav_id3v2_find_frame(t, album_tags);
   if(frame && frame->strings)
-    m->album = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_ALBUM, frame->strings[0]);
 
   /* Copyright */
     
   frame = bgav_id3v2_find_frame(t, copyright_tags);
   if(frame && frame->strings)
-    m->copyright = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_COPYRIGHT, frame->strings[0]);
 
   /* Artist */
 
   frame = bgav_id3v2_find_frame(t, artist_tags);
   if(frame && frame->strings)
-    m->artist = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_ARTIST, frame->strings[0]);
 
+  /* Albumartist */
+
+  frame = bgav_id3v2_find_frame(t, albumartist_tags);
+  if(frame && frame->strings)
+    gavl_metadata_set(m, GAVL_META_ALBUMARTIST, frame->strings[0]);
+  
   /* Author */
 
   frame = bgav_id3v2_find_frame(t, author_tags);
   if(frame && frame->strings)
-    m->author = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_AUTHOR, frame->strings[0]);
   
-  /* Date */
+  /* Year */
   
-  frame = bgav_id3v2_find_frame(t, date_tags);
+  frame = bgav_id3v2_find_frame(t, year_tags);
   if(frame && frame->strings)
-    m->date = bgav_strdup(frame->strings[0]);
+    gavl_metadata_set(m, GAVL_META_YEAR, frame->strings[0]);
 
   /* Track */
 
   frame = bgav_id3v2_find_frame(t, track_tags);
   if(frame && frame->strings)
-    m->track = atoi(frame->strings[0]);
-
+    gavl_metadata_set_int(m, GAVL_META_TRACKNUMBER, atoi(frame->strings[0]));
+  
   /* Genre */
   
   frame = bgav_id3v2_find_frame(t, genre_tags);
@@ -731,11 +735,11 @@ void bgav_id3v2_2_metadata(bgav_id3v2_tag_t * t, bgav_metadata_t*m)
     if((frame->strings[0][0] == '(') && isdigit(frame->strings[0][1]))
       {
       i_tmp = atoi(&frame->strings[0][1]);
-      m->genre = bgav_strdup(bgav_id3v1_get_genre(i_tmp));
+      gavl_metadata_set(m, GAVL_META_GENRE, bgav_id3v1_get_genre(i_tmp));
       }
     else
       {
-      m->genre = bgav_strdup(frame->strings[0]);
+      gavl_metadata_set(m, GAVL_META_GENRE, frame->strings[0]);
       }
     }
 
@@ -744,7 +748,7 @@ void bgav_id3v2_2_metadata(bgav_id3v2_tag_t * t, bgav_metadata_t*m)
   frame = bgav_id3v2_find_frame(t, comment_tags);
 
   if(frame)
-    m->comment = get_comment(t->opt, frame);
+    gavl_metadata_set_nocpy(m, GAVL_META_COMMENT, get_comment(t->opt, frame));
   
   }
 
