@@ -200,7 +200,7 @@ static int set_diseqc(bgav_input_context_t * ctx, bgav_dvb_channel_info_t * c)
   struct dvb_diseqc_master_cmd cmd =
     {{0xe0, 0x10, 0x38, 0xf0, 0x00, 0x00}, 4};
 
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   cmd.msg[3] = 0xf0 | ((c->sat_no * 4) & 0x0f) |
     (c->tone ? 1 : 0) | (c->pol ? 0 : 2);
@@ -235,7 +235,7 @@ static int tune_in(bgav_input_context_t * ctx,
   struct dvb_frontend_event event;
   struct pollfd pfd[1];
   
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   /* set_diseqc for satellite tuners */
 
@@ -330,7 +330,7 @@ static int get_streams(bgav_input_context_t * ctx,
   pmt_section_t pmts;
   struct dmx_sct_filter_params params;
   dvb_priv_t * priv;
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
     
   if(channel->initialized)
     return 1;
@@ -449,8 +449,8 @@ static int load_channel_cache(bgav_input_context_t * ctx)
   {
   bgav_input_context_t * input;
   char * filename;
-  bgav_yml_node_t * yml = (bgav_yml_node_t*)0;
-  bgav_yml_node_t * yml_root = (bgav_yml_node_t*)0;
+  bgav_yml_node_t * yml = NULL;
+  bgav_yml_node_t * yml_root = NULL;
   bgav_yml_node_t * channel_node;
   bgav_yml_node_t * channel_child;
   bgav_yml_node_t * stream_node;
@@ -465,7 +465,7 @@ static int load_channel_cache(bgav_input_context_t * ctx)
   const char * attr;
   long file_time;
   
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   input = bgav_input_create(ctx->opt);
   
@@ -516,7 +516,7 @@ static int load_channel_cache(bgav_input_context_t * ctx)
       {
       if(stat(priv->channels_conf_file, &st))
         goto fail;
-      file_time = strtol(channel_node->children->str, (char**)0, 10);
+      file_time = strtol(channel_node->children->str, NULL, 10);
       if(file_time != st.st_mtime)
         {
         bgav_log(ctx->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
@@ -531,14 +531,15 @@ static int load_channel_cache(bgav_input_context_t * ctx)
       attr = bgav_yml_get_attribute(channel_node, "pcr_pid");
       if(!attr)
         goto fail;
-      priv->channels[channel_index].pcr_pid = strtol(attr, (char**)0, 16);
+      priv->channels[channel_index].pcr_pid = strtol(attr, NULL, 16);
 
       attr = bgav_yml_get_attribute(channel_node, "extra_pcr_pid");
       if(!attr)
         goto fail;
       priv->channels[channel_index].extra_pcr_pid = atoi(attr);
 
-      ctx->tt->tracks[channel_index].name = bgav_strdup(priv->channels[channel_index].name);
+      ctx->tt->tracks[channel_index].name =
+        bgav_strdup(priv->channels[channel_index].name);
       
       while(channel_child)
         {
@@ -563,7 +564,8 @@ static int load_channel_cache(bgav_input_context_t * ctx)
             if(!strcmp(stream_node->name, "astream"))
               {
               s =
-                bgav_track_add_audio_stream(&ctx->tt->tracks[channel_index], ctx->opt);
+                bgav_track_add_audio_stream(&ctx->tt->tracks[channel_index],
+                                            ctx->opt);
               s->timescale = 90000;
               s->flags |= (STREAM_PARSE_FULL|STREAM_NEED_START_TIME);
               
@@ -578,13 +580,16 @@ static int load_channel_cache(bgav_input_context_t * ctx)
                   }
 
                 if(!strcmp(stream_child->name, "pid"))
-                  s->stream_id = strtol(stream_child->children->str, (char**)0, 16);
+                  s->stream_id = strtol(stream_child->children->str, NULL, 16);
                 else if(!strcmp(stream_child->name, "fourcc"))
-                  s->fourcc = strtol(stream_child->children->str, (char**)0, 16);
+                  s->fourcc = strtol(stream_child->children->str, NULL, 16);
 
                 else if(!strcmp(stream_child->name, "language"))
-                  sscanf(stream_child->children->str, "%3s", s->language);
-                
+                  {
+                  char language[4];
+                  sscanf(stream_child->children->str, "%3s", language);
+                  gavl_metadata_set(&s->m, GAVL_META_LANGUAGE, language);
+                  }
                 stream_child = stream_child->next;
                 }
               }
@@ -612,7 +617,6 @@ static int load_channel_cache(bgav_input_context_t * ctx)
 
               s->timescale = 90000;
               s->flags |= (STREAM_PARSE_FULL|STREAM_NEED_START_TIME);
-              s->data.video.frametime_mode = BGAV_FRAMETIME_CODEC;
               stream_child = stream_node->children;
 
               while(stream_child)
@@ -624,9 +628,9 @@ static int load_channel_cache(bgav_input_context_t * ctx)
                   }
 
                 if(!strcmp(stream_child->name, "pid"))
-                  s->stream_id = strtol(stream_child->children->str, (char**)0, 16);
+                  s->stream_id = strtol(stream_child->children->str, NULL, 16);
                 else if(!strcmp(stream_child->name, "fourcc"))
-                  s->fourcc = strtol(stream_child->children->str, (char**)0, 16);
+                  s->fourcc = strtol(stream_child->children->str, NULL, 16);
                 
                 stream_child = stream_child->next;
                 }
@@ -659,13 +663,14 @@ static int load_channel_cache(bgav_input_context_t * ctx)
 
 static void save_channel_cache(bgav_input_context_t * ctx)
   {
-  FILE * output = (FILE*)0;
+  FILE * output = NULL;
   char * filename;
-  char * path = (char*)0;
+  char * path = NULL;
+  const char * language;
   dvb_priv_t * priv;
   int i, j;
   struct stat st;
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   filename = strrchr(priv->device_directory, '/');
   filename++;
@@ -698,11 +703,11 @@ static void save_channel_cache(bgav_input_context_t * ctx)
       fprintf(output, "        <fourcc>%08x</fourcc>\n",
               ctx->tt->tracks[i].audio_streams[j].fourcc);
 
-      if(ctx->tt->tracks[i].audio_streams[j].language[0])
-        {
+      language = gavl_metadata_get(&ctx->tt->tracks[i].audio_streams[j].m,
+                                   GAVL_META_LANGUAGE);
+      if(language)
         fprintf(output, "        <language>%3s</language>\n",
-                ctx->tt->tracks[i].audio_streams[j].language);
-        }
+                language);
       fprintf(output, "      </astream>\n");
       }
     fprintf(output, "    </astreams>\n");
@@ -826,7 +831,7 @@ static int open_dvb(bgav_input_context_t * ctx, const char * url, char ** redire
 static void close_dvb(bgav_input_context_t * ctx)
   {
   dvb_priv_t * priv;
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   set_num_filters(priv, 0);
   if(priv->dvr_fd >= 0)
@@ -881,10 +886,10 @@ static char * decode_eit_string(const bgav_options_t * opt,
   {
   char * ret;
   int i;
-  const char * charset = (const char*)0;
+  const char * charset = NULL;
   bgav_charset_converter_t * cnv;
   if(!(*pos))
-    return (char*)0;
+    return NULL;
   
   /* Detect charset */
   if(*pos >= 0x20)
@@ -909,20 +914,23 @@ static char * decode_eit_string(const bgav_options_t * opt,
     }
   if(!charset)
     {
-    return (char*)0;
+    return NULL;
     }
 
   cnv = bgav_charset_converter_create(opt, charset, "UTF-8");
-  ret = bgav_convert_string(cnv, (char*)pos, len, (int *)0);
+  ret = bgav_convert_string(cnv, (char*)pos, len, NULL);
   bgav_charset_converter_destroy(cnv);
   return ret;
   }
 
 #define bcdtoint(i) ((((i & 0xf0) >> 4) * 10) + (i & 0x0f))
 
-/* Extract UTC time and date encoded in modified julian date format and return it as a time_t.
+/*
+ *  Extract UTC time and date encoded in modified julian date format and
+ *  return it as a time_t.
  */
-static void dvb_mjdtime (uint8_t *buf, struct tm *tma)
+
+static void dvb_mjdtime(uint8_t *buf, struct tm *tma)
   {
   int i;
   unsigned int year, month, day, hour, min, sec;
@@ -969,9 +977,8 @@ static void check_eit(bgav_input_context_t* ctx)
   fd_set rset;
   int version;
   struct timeval timeout;
-  char * pos_c;
   //  char * tmp_string;
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   FD_ZERO(&rset);
   FD_SET (priv->filter_fds[priv->eit_filter], &rset);
@@ -1025,11 +1032,9 @@ static void check_eit(bgav_input_context_t* ctx)
   while(pos - start + 4 < len)
     {
     tmp = BGAV_PTR_2_16BE(pos); pos+=2;
-
     
     dvb_mjdtime(pos, &start_time);
     pos += 5;
-
     
     tmp = BGAV_PTR_2_24BE(pos); pos+=3;
 
@@ -1037,24 +1042,29 @@ static void check_eit(bgav_input_context_t* ctx)
     
     descriptors_end = pos + (tmp & 0x0fff);
     
-    
     if(tmp >> 13 == 0x04)
       {
       int desc_len, desc_tag;
       uint8_t * end_pos;
-      bgav_metadata_t * m;
-
-      char time_string[32]; /* 26 actually */
-      asctime_r(&start_time, time_string);
-      pos_c = strchr(time_string, '\n');
-      if(pos_c)
-        *pos_c = '\0';
+      gavl_metadata_t * m;
+      char time_string[GAVL_METADATA_DATE_TIME_STRING_LEN];
+      const char * tmp_string;
+      
+      gavl_metadata_date_time_to_string(start_time.tm_year + 1900,
+                                        start_time.tm_mon+1,
+                                        start_time.tm_mday,
+                                        start_time.tm_hour,
+                                        start_time.tm_min,
+                                        start_time.tm_sec,
+                                        time_string);
       
       /* Parse descriptors */
 
       m = &ctx->tt->cur->metadata;
 
-      if(m->date && !strcmp(m->date, time_string))
+      tmp_string = gavl_metadata_get(m, GAVL_META_DATE);
+      
+      if(tmp_string && !strcmp(tmp_string, time_string))
         {
         /* Already read this */
         pos = descriptors_end;
@@ -1064,7 +1074,7 @@ static void check_eit(bgav_input_context_t* ctx)
       priv->name_changed = 1;
       priv->metadata_changed = 1;
       
-      bgav_metadata_free(m);
+      gavl_metadata_free(m);
       
       while(1)
         {
@@ -1083,8 +1093,8 @@ static void check_eit(bgav_input_context_t* ctx)
             if(ctx->opt->name_change_callback ||
                ctx->opt->metadata_change_callback)
               {
-              m->title = decode_eit_string(ctx->opt, pos, tmp);
-
+              gavl_metadata_set_nocpy(m, GAVL_META_TITLE, 
+                                      decode_eit_string(ctx->opt, pos, tmp));
               }
             pos += tmp;
             
@@ -1092,8 +1102,9 @@ static void check_eit(bgav_input_context_t* ctx)
             
             if(ctx->opt->metadata_change_callback)
               {
-              m->comment = decode_eit_string(ctx->opt,
-                                             pos, tmp);
+              gavl_metadata_set_nocpy(m, GAVL_META_COMMENT, 
+                                      decode_eit_string(ctx->opt,
+                                                        pos, tmp));
               }
             pos += tmp;
             break;
@@ -1103,11 +1114,9 @@ static void check_eit(bgav_input_context_t* ctx)
         
         if(ctx->opt->metadata_change_callback)
           {
-          m->date = bgav_strdup(time_string);
-          
+          gavl_metadata_set(m, GAVL_META_DATE, time_string);
           ctx->opt->metadata_change_callback(ctx->opt->metadata_change_callback_data,
                                              m);
-          
           }
         pos = end_pos;
         if(pos >= descriptors_end)
@@ -1124,7 +1133,8 @@ static void check_eit(bgav_input_context_t* ctx)
   if(ctx->opt->name_change_callback && priv->name_changed)
     {
     ctx->opt->name_change_callback(ctx->opt->name_change_callback_data,
-                                   ctx->tt->cur->metadata.title);
+                                   gavl_metadata_get(&ctx->tt->cur->metadata,
+                                                     GAVL_META_TITLE));
     priv->name_changed = 0;
     }
   if(ctx->opt->metadata_change_callback && priv->metadata_changed)
@@ -1144,7 +1154,7 @@ static int read_dvb(bgav_input_context_t* ctx,
   int result;
   struct pollfd pfd[1];
   
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   /* Flush events */
   while (ioctl(priv->fe_fd, FE_GET_EVENT, &event) != -1);
@@ -1206,7 +1216,7 @@ static int setup_pes_filter(const bgav_options_t * opt,
                             int fd, int pes_type, int stream_id)
   {
   struct dmx_pes_filter_params params;
-  
+  memset(&params, 0, sizeof(params));
   params.pid      = stream_id;
   params.input    = DMX_IN_FRONTEND;
   params.output   = DMX_OUT_TS_TAP;
@@ -1251,7 +1261,7 @@ static int setup_filters(bgav_input_context_t * ctx,
   int num_audio_streams;
   int audio_index;
   
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
 
   set_num_filters(priv, track->num_audio_streams +
                   track->num_video_streams + channel->extra_pcr_pid + 1);
@@ -1405,13 +1415,13 @@ static int setup_filters(bgav_input_context_t * ctx,
   
   }
 
-static void select_track_dvb(bgav_input_context_t * ctx, int track)
+static int select_track_dvb(bgav_input_context_t * ctx, int track)
   {
   dvb_priv_t * priv;
   fd_set rset;
   struct timeval timeout;
   
-  priv = (dvb_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   
   if(priv->dvr_fd >= 0)
     {
@@ -1419,10 +1429,10 @@ static void select_track_dvb(bgav_input_context_t * ctx, int track)
     priv->dvr_fd = -1;
     }
   if(!tune_in(ctx, &priv->channels[track]))
-    return;
+    return 0;
   
   if(!setup_filters(ctx, &priv->channels[track], ctx->tt->cur))
-    return;
+    return 0;
   
   priv->dvr_fd = open(priv->dvr_filename, O_RDONLY | O_NONBLOCK);
   ctx->sync_id = priv->channels[track].pcr_pid;
@@ -1438,7 +1448,8 @@ static void select_track_dvb(bgav_input_context_t * ctx, int track)
   select(priv->dvr_fd+1, &rset, NULL, NULL, &timeout);
 
   /* Clear metadata so they are reread */
-  bgav_metadata_free(&ctx->tt->cur->metadata);
+  gavl_metadata_free(&ctx->tt->cur->metadata);
+  return 1;
   }
 
 const bgav_input_t bgav_input_dvb =
@@ -1454,7 +1465,7 @@ static
 bgav_input_context_t * bgav_input_open_dvb(const char * device,
                                            bgav_options_t * opt)
   {
-  bgav_input_context_t * ret = (bgav_input_context_t *)0;
+  bgav_input_context_t * ret = NULL;
   ret = bgav_input_create(opt);
   ret->input = &bgav_input_dvb;
 
@@ -1466,7 +1477,7 @@ bgav_input_context_t * bgav_input_open_dvb(const char * device,
   fail:
   if(ret)
     free(ret);
-  return (bgav_input_context_t *)0;
+  return NULL;
   }
 
 int bgav_open_dvb(bgav_t * b, const char * device)
@@ -1513,11 +1524,11 @@ bgav_device_info_t * bgav_find_devices_dvb()
   char * device_name;
   char * directory;
   
-  bgav_device_info_t * ret = (bgav_device_info_t *)0;
+  bgav_device_info_t * ret = NULL;
   
   for(i = 0; i < 8; i++)
     {
-    device_name = (char*)0;
+    device_name = NULL;
     
     directory = bgav_sprintf("/dev/dvb/adapter%d", i);
     
@@ -1546,7 +1557,7 @@ int bgav_check_device_dvb(const char * device, char ** name)
 
 bgav_device_info_t * bgav_find_devices_dvb()
   {
-  return (bgav_device_info_t*)0;
+  return NULL;
   }
 
 int bgav_open_dvb(bgav_t * b, const char * device)
