@@ -242,7 +242,7 @@ static int setup_track(bgav_input_context_t * ctx,
                         int title, int chapter, int angle)
   {
   int video_width = 0, video_height;
-  const char * audio_codec = (const char *)0;
+  const char * audio_codec = NULL;
   audio_attr_t * audio_attr;
   video_attr_t * video_attr;
   subp_attr_t *  subp_attr;
@@ -251,14 +251,14 @@ static int setup_track(bgav_input_context_t * ctx,
   bgav_stream_t * s;
   bgav_track_t * new_track;
   tt_srpt_t *ttsrpt;
-  pgc_t * pgc = (pgc_t*)0;
+  pgc_t * pgc = NULL;
   vts_ptt_srpt_t *vts_ptt_srpt;
   int ttn, pgn;
   int pgc_id;
   track_priv_t * track_priv;
   const char * language_3cc;
   char language_2cc[3];
-  dvd_t * dvd = (dvd_t*)(ctx->priv);
+  dvd_t * dvd = ctx->priv;
   ttsrpt = dvd->vmg_ifo->tt_srpt;
 
   /* Open VTS */
@@ -488,7 +488,7 @@ static int setup_track(bgav_input_context_t * ctx,
       language_2cc[2] = '\0';
       language_3cc = bgav_lang_from_twocc(language_2cc);
       if(language_3cc)
-        strcpy(s->language, language_3cc);
+        gavl_metadata_set(&s->m, GAVL_META_LANGUAGE, language_3cc);
       }
 
     /* Set description */
@@ -496,24 +496,29 @@ static int setup_track(bgav_input_context_t * ctx,
     switch(audio_attr->code_extension)
       {
       case 0:
-        s->info = bgav_sprintf("Unspecified (%s, %dch)",
-                               audio_codec, audio_attr->channels+1);
+        gavl_metadata_set_nocpy(&s->m, GAVL_META_LABEL,
+                                bgav_sprintf("Unspecified (%s, %dch)",
+                                             audio_codec, audio_attr->channels+1));
         break;
       case 1:
-        s->info = bgav_sprintf("Audio stream (%s, %dch)",
-                               audio_codec, audio_attr->channels+1);
+        gavl_metadata_set_nocpy(&s->m, GAVL_META_LABEL,
+                                bgav_sprintf("Audio stream (%s, %dch)",
+                                             audio_codec, audio_attr->channels+1));
         break;
       case 2:
-        s->info = bgav_sprintf("Audio for visually impaired (%s, %dch)",
-                               audio_codec, audio_attr->channels+1);
+        gavl_metadata_set_nocpy(&s->m, GAVL_META_LABEL,
+                                bgav_sprintf("Audio for visually impaired (%s, %dch)",
+                                             audio_codec, audio_attr->channels+1));
         break;
       case 3:
-        s->info = bgav_sprintf("Director's comments 1 (%s, %dch)",
-                               audio_codec, audio_attr->channels+1);
+        gavl_metadata_set_nocpy(&s->m, GAVL_META_LABEL,
+                                bgav_sprintf("Director's comments 1 (%s, %dch)",
+                                             audio_codec, audio_attr->channels+1));
         break;
       case 4:
-        s->info = bgav_sprintf("Director's comments 2 (%s, %dch)",
-                               audio_codec, audio_attr->channels+1);
+        gavl_metadata_set_nocpy(&s->m, GAVL_META_LABEL,
+                                bgav_sprintf("Director's comments 2 (%s, %dch)",
+                                             audio_codec, audio_attr->channels+1));
         break;
       }
     }
@@ -524,7 +529,7 @@ static int setup_track(bgav_input_context_t * ctx,
     if(!(pgc->subp_control[i] & 0x80000000))
       continue;
 
-    s = bgav_track_add_subtitle_stream(new_track, ctx->opt, 0, (char*)0);
+    s = bgav_track_add_subtitle_stream(new_track, ctx->opt, 0, NULL);
     s->fourcc = BGAV_MK_FOURCC('D', 'V', 'D', 'S');
 
     /*  there are several streams for one spu */
@@ -561,7 +566,8 @@ static int setup_track(bgav_input_context_t * ctx,
       language_2cc[2] = '\0';
       language_3cc = bgav_lang_from_twocc(language_2cc);
       if(language_3cc)
-        strcpy(s->language, language_3cc);
+        gavl_metadata_set(&s->m, GAVL_META_LANGUAGE,
+                          language_3cc);
       }
 
     switch(subp_attr->code_extension)
@@ -570,38 +576,42 @@ static int setup_track(bgav_input_context_t * ctx,
         //        printf("Not specified ");
         break;
       case 1:
-        s->info = bgav_sprintf("Caption");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Caption");
         //        printf("Caption with normal size character ");
         break;
       case 2:
-        s->info = bgav_sprintf("Caption, big");
-        // printf("Caption with bigger size character ");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Caption, big");
         break;
       case 3:
-        s->info = bgav_sprintf("Caption for children");
-        //        printf("Caption for children ");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Caption for children");
         break;
       case 4:
         //        printf("reserved ");
         break;
       case 5:
-        s->info = bgav_sprintf("Closed caption");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Closed caption");
         //        printf("Closed Caption with normal size character ");
         break;
       case 6:
-        s->info = bgav_sprintf("Closed caption, big");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Closed caption, big");
         //        printf("Closed Caption with bigger size character ");
         break;
       case 7:
-        s->info = bgav_sprintf("Closed caption for children");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Closed caption for children");
         //        printf("Closed Caption for children ");
         break;
       case 8:
         //        printf("reserved ");
         break;
       case 9:
-        s->info = bgav_sprintf("Forced caption");
-        //        printf("Forced Caption");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Forced caption");
         break;
       case 10:
         //        printf("reserved ");
@@ -613,16 +623,16 @@ static int setup_track(bgav_input_context_t * ctx,
         //        printf("reserved ");
         break;
       case 13:
-        //        printf("Director's comments with normal size character ");
-        s->info = bgav_sprintf("Directors comments");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Directors comments");
         break;
       case 14:
-        //        printf("Director's comments with bigger size character ");
-        s->info = bgav_sprintf("Directors comments, big");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Directors comments, big");
         break;
       case 15:
-        s->info = bgav_sprintf("Directors comments for children");
-        //        printf("Director's comments for children ");
+        gavl_metadata_set(&s->m, GAVL_META_LABEL,
+                          "Directors comments for children");
         break;
       default:
         //        printf("(please send a bug report) ");
@@ -800,7 +810,7 @@ read_nav(bgav_input_context_t * ctx, int sector, int *next)
   int blocks;
   dvd_t * d;
 
-  d = (dvd_t*)ctx->priv;
+  d = ctx->priv;
   
   if(DVDReadBlocks(d->dvd_file, sector, 1, buf) != 1)
     {
@@ -852,7 +862,7 @@ read_nav(bgav_input_context_t * ctx, int sector, int *next)
 static int check_next_cell(bgav_input_context_t * ctx)
   {
   dvd_t * d;
-  d = (dvd_t*)(ctx->priv);
+  d = ctx->priv;
 
   if(d->next_cell < 0)
     return 0;
@@ -868,7 +878,7 @@ static int read_sector_dvd(bgav_input_context_t * ctx, uint8_t * data)
   {
   dvd_t * d;
   int l;
-  d = (dvd_t*)(ctx->priv);
+  d = ctx->priv;
   switch(d->state)
     {
     case CELL_START:
@@ -924,25 +934,29 @@ static void    close_dvd(bgav_input_context_t * ctx)
   int i;
   track_priv_t * track_priv;
   dvd_t * dvd;
-  dvd = (dvd_t*)(ctx->priv);
-
-  if(dvd->dvd_reader)
-    DVDClose(dvd->dvd_reader);
-  
-  if(dvd->dvd_file)
-    DVDCloseFile(dvd->dvd_file);
+  dvd = ctx->priv;
   
+  // ifo_handle_t
   if(dvd->vmg_ifo)
     ifoClose(dvd->vmg_ifo);
-  
+
+  // ifo_handle_t
   if(dvd->vts_ifo)
     ifoClose(dvd->vts_ifo);
 
+  // dvd_file_t
+  if(dvd->dvd_file)
+    DVDCloseFile(dvd->dvd_file);
+
+  // dvd_reader_t
+  if(dvd->dvd_reader)
+    DVDClose(dvd->dvd_reader);
+  
   if(ctx->tt)
     {
     for(i = 0; i < ctx->tt->num_tracks; i++)
       {
-      track_priv = (track_priv_t*)(ctx->tt->tracks[i].priv);
+      track_priv = ctx->tt->tracks[i].priv;
       if(track_priv)
         {
         if(track_priv->chapters)
@@ -956,7 +970,7 @@ static void    close_dvd(bgav_input_context_t * ctx)
   return;
   }
 
-static void select_track_dvd(bgav_input_context_t * ctx, int track)
+static int select_track_dvd(bgav_input_context_t * ctx, int track)
   {
   dvd_t * dvd;
   int ttn, pgn;
@@ -965,12 +979,12 @@ static void select_track_dvd(bgav_input_context_t * ctx, int track)
   track_priv_t * track_priv;
   vts_ptt_srpt_t *vts_ptt_srpt;
   
-  dvd = (dvd_t*)(ctx->priv);
+  dvd = ctx->priv;
   dvd->last_vobu_end_pts = BGAV_TIMESTAMP_UNDEFINED;
   ctx->demuxer->flags &= ~BGAV_DEMUXER_HAS_TIMESTAMP_OFFSET;
   
   ttsrpt = dvd->vmg_ifo->tt_srpt;
-  track_priv = (track_priv_t*)(ctx->tt->cur->priv);
+  track_priv = ctx->tt->cur->priv;
 
   dvd->current_track_priv = track_priv;
   
@@ -999,9 +1013,11 @@ static void select_track_dvd(bgav_input_context_t * ctx, int track)
   /* Set the subtitle palettes */
   for(i = 0; i < ctx->tt->cur->num_subtitle_streams; i++)
     {
-    ctx->tt->cur->subtitle_streams[i].ext_data = (uint8_t*)(dvd->pgc->palette);
-    ctx->tt->cur->subtitle_streams[i].ext_size = sizeof(dvd->pgc->palette);
+    bgav_stream_set_extradata(&ctx->tt->cur->subtitle_streams[i],
+                              (uint8_t*)dvd->pgc->palette,
+                              sizeof(dvd->pgc->palette));
     }
+  return 1;
   }
 
 static void seek_time_dvd(bgav_input_context_t * ctx, int64_t t1, int scale)
@@ -1016,7 +1032,7 @@ static void seek_time_dvd(bgav_input_context_t * ctx, int64_t t1, int scale)
   gavl_time_t t;
   t = gavl_time_unscale(scale, t1);
   
-  dvd = (dvd_t*)(ctx->priv);
+  dvd = ctx->priv;
 
   time = 0;
   cell_start_time = 0;
@@ -1207,7 +1223,7 @@ bgav_device_info_t * bgav_find_devices_dvd()
   int i;
   char * device_name;
   char ** devices;
-  bgav_device_info_t * ret = (bgav_device_info_t *)0;
+  bgav_device_info_t * ret = NULL;
 
   devices = cdio_get_devices(DRIVER_DEVICE);
 
@@ -1217,7 +1233,7 @@ bgav_device_info_t * bgav_find_devices_dvd()
   i = 0;
   while(devices[i])
     {
-    device_name = (char*)0;
+    device_name = NULL;
     if(bgav_check_device_dvd(devices[i], &device_name))
       {
       ret = bgav_device_info_append(ret,
@@ -1237,7 +1253,7 @@ static
 bgav_input_context_t * bgav_input_open_dvd(const char * device,
                                            bgav_options_t * opt)
   {
-  bgav_input_context_t * ret = (bgav_input_context_t *)0;
+  bgav_input_context_t * ret = NULL;
   ret = bgav_input_create(opt);
   ret->input = &bgav_input_dvd;
   if(!ret->input->open(ret, device, NULL))
@@ -1249,7 +1265,7 @@ bgav_input_context_t * bgav_input_open_dvd(const char * device,
   fail:
   if(ret)
     free(ret);
-  return (bgav_input_context_t *)0;
+  return NULL;
   }
 
 int bgav_open_dvd(bgav_t * b, const char * device)
@@ -1310,7 +1326,7 @@ int bgav_check_device_dvd(const char * device, char ** name)
 
 bgav_device_info_t * bgav_find_devices_dvd()
   {
-  return (bgav_device_info_t*)0;
+  return NULL;
   
   }
 
