@@ -119,7 +119,7 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
   int fd;
   uint8_t header[8];
   int seq;
-  rtsp_priv_t * priv = (rtsp_priv_t *)(ctx->priv);
+  rtsp_priv_t * priv = ctx->priv;
 
   if(priv->eof)
     return 0;
@@ -130,17 +130,17 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
     {
     if(block)
       {
-      if(bgav_read_data_fd(fd, header, 8, ctx->opt->read_timeout) < 8)
+      if(bgav_read_data_fd(ctx->opt, fd, header, 8, ctx->opt->read_timeout) < 8)
         return 0;
       }
     else
       {
-      bytes_read = bgav_read_data_fd(fd, header, 8, 0);
+      bytes_read = bgav_read_data_fd(ctx->opt, fd, header, 8, 0);
       if(!bytes_read)
         return 0;
       else if(bytes_read < 8)
         {
-        if(bgav_read_data_fd(fd, &header[bytes_read], 8 - bytes_read,
+        if(bgav_read_data_fd(ctx->opt, fd, &header[bytes_read], 8 - bytes_read,
                              ctx->opt->read_timeout) < 8 - bytes_read)
           return 0;
         
@@ -158,7 +158,7 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
       while(1)
         {
         char * ptr = (char*)priv->packet;
-        if(!bgav_read_line_fd(fd, &ptr,
+        if(!bgav_read_line_fd(ctx->opt, fd, &ptr,
                               &priv->packet_alloc,
                               ctx->opt->read_timeout))
           return 0;
@@ -197,10 +197,10 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
         header[1]=header[6];
         header[2]=header[7];
 
-        if(bgav_read_data_fd(fd, &header[3], 5, ctx->opt->read_timeout) < 5)
+        if(bgav_read_data_fd(ctx->opt, fd, &header[3], 5, ctx->opt->read_timeout) < 5)
           return 0;
 
-        if(bgav_read_data_fd(fd, &header[4], 4, ctx->opt->read_timeout) < 4)
+        if(bgav_read_data_fd(ctx->opt, fd, &header[4], 4, ctx->opt->read_timeout) < 4)
           return 0;
         flags1 = header[4];
         size-=9;
@@ -208,7 +208,7 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
 
       flags2=header[7];
       unknown1 = (header[5]<<16)+(header[6]<<8)+(header[7]);
-      if(bgav_read_data_fd(fd, header, 6, ctx->opt->read_timeout) < 6)
+      if(bgav_read_data_fd(ctx->opt, fd, header, 6, ctx->opt->read_timeout) < 6)
         return 0;
 
       timestamp = BGAV_PTR_2_32BE(header);
@@ -236,7 +236,7 @@ static int next_packet_rdt(bgav_input_context_t * ctx, int block)
     
       size -= 12;
       bgav_rmff_packet_header_to_pointer(&ph, priv->packet);
-      if(bgav_read_data_fd(fd, priv->packet + 12, size,
+      if(bgav_read_data_fd(ctx->opt, fd, priv->packet + 12, size,
                            ctx->opt->read_timeout) < size)
         return 0;
 
@@ -281,7 +281,7 @@ static const char * get_answer_var(const char * str, const char * name,
   
   pos1 = strstr(str, name);
   if(!pos1)
-    return (char*)0;
+    return NULL;
   pos2 = strchr(pos1, ';');
   if(!pos2)
     pos2 = pos1 + strlen(pos1);
@@ -294,7 +294,7 @@ static int open_and_describe(bgav_input_context_t * ctx,
                              const char * url, int * got_redirected)
   {
   const char * var;
-  rtsp_priv_t * priv = (rtsp_priv_t *)(ctx->priv);
+  rtsp_priv_t * priv = ctx->priv;
   
   /* Open URL */
 
@@ -412,7 +412,7 @@ static int is_real_smil(bgav_sdp_t * s)
 static int init_real(bgav_input_context_t * ctx, bgav_sdp_t * sdp, char * session_id)
   {
   rtsp_priv_t * priv;
-  char * stream_rules = (char*)0;
+  char * stream_rules = NULL;
   char challenge2[64];
   char checksum[34];
   char * field;
@@ -527,7 +527,7 @@ static int handle_stream_transport(bgav_stream_t * s,
   int server_ports[2] = { 0, 0 };
   int client_ports[2] = { 0, 0 };
   //  int i;
-  rtp_stream_priv_t * sp = (rtp_stream_priv_t *)s->priv;
+  rtp_stream_priv_t * sp = s->priv;
   
   if(!tcp)
     {
@@ -588,7 +588,7 @@ static int handle_stream_transport(bgav_stream_t * s,
   
 
   if((var = get_answer_var(transport, "ssrc=", &var_len)))
-    sp->server_ssrc = strtoul(var, (char**)0, 16);
+    sp->server_ssrc = strtoul(var, NULL, 16);
 
   bgav_log(s->opt, BGAV_LOG_INFO, LOG_DOMAIN, "ssrc: %08x\n",
            sp->server_ssrc);
@@ -600,8 +600,8 @@ static int init_stream_generic(bgav_input_context_t * ctx,
                                bgav_stream_t * s, int * port,
                                char ** session_id, int tcp)
   {
-  rtsp_priv_t * priv = (rtsp_priv_t*)ctx->priv;
-  rtp_stream_priv_t * sp = (rtp_stream_priv_t *)s->priv;
+  rtsp_priv_t * priv = ctx->priv;
+  rtp_stream_priv_t * sp = s->priv;
   char * field;
   const char * var;
 
@@ -692,7 +692,7 @@ static int handle_rtpinfo(bgav_input_context_t * ctx,
     else
       pos1 = var;
     
-    s = (bgav_stream_t*)0;
+    s = NULL;
     /* Search for the bgav stream */
     for(j = 0; j < ctx->demuxer->tt->cur->num_video_streams; j++)
       {
@@ -734,14 +734,14 @@ static int handle_rtpinfo(bgav_input_context_t * ctx,
         {
         return 0;
         }
-      sp->first_rtptime = strtoul(var, (char**)0, 10);
+      sp->first_rtptime = strtoul(var, NULL, 10);
 
       
       var = get_answer_var(streams[i], "seq=", &var_len);
       if(!var)
         return 0;
       
-      sp->first_seq = strtoul(var, (char**)0, 10);
+      sp->first_seq = strtoul(var, NULL, 10);
       }
     
     i++;
@@ -757,7 +757,7 @@ static int init_generic(bgav_input_context_t * ctx, bgav_sdp_t * sdp, int tcp)
   bgav_stream_t * s;
   int i;
   int port;
-  char * session_id = (char *)0;
+  char * session_id = NULL;
   char * field;
   const char * var;
   priv = ctx->priv;
@@ -836,7 +836,7 @@ static int open_rtsp(bgav_input_context_t * ctx, const char * url, char ** r)
   rtsp_priv_t * priv;
   bgav_sdp_t * sdp;
   const char * var;
-  char * session_id = (char*)0;
+  char * session_id = NULL;
   int num_redirections = 0;
   int got_redirected = 0;
   priv = calloc(1, sizeof(*priv));
@@ -888,10 +888,9 @@ static int open_rtsp(bgav_input_context_t * ctx, const char * url, char ** r)
   bgav_sdp_dump(sdp);
   
   /* Set up input metadata from sdp */
-  ctx->metadata.title = bgav_strdup(sdp->session_name);
-  ctx->metadata.comment = bgav_strdup(sdp->session_description);
-
-  
+  gavl_metadata_set(&ctx->metadata, GAVL_META_TITLE, sdp->session_name);
+  gavl_metadata_set(&ctx->metadata, GAVL_META_COMMENT, sdp->session_description);
+    
   switch(priv->type)
     {
     case SERVER_TYPE_REAL:
@@ -920,19 +919,19 @@ static int open_rtsp(bgav_input_context_t * ctx, const char * url, char ** r)
   if(priv->challenge1)
     {
     free(priv->challenge1);
-    priv->challenge1 = (char*)0;
+    priv->challenge1 = NULL;
     }
   if(session_id)
     free(session_id);
   free(priv);
-  ctx->priv = (void*)0;
+  ctx->priv = NULL;
   return 0;
   }
 
 static void close_rtsp(bgav_input_context_t * ctx)
   {
   rtsp_priv_t * priv;
-  priv = (rtsp_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   if(!priv)
     return;
   
@@ -963,7 +962,7 @@ static int do_read(bgav_input_context_t* ctx,
   rtsp_priv_t * priv;
   int bytes_to_copy;
   int fd;
-  priv = (rtsp_priv_t*)(ctx->priv);
+  priv = ctx->priv;
   bytes_read = 0;
   if(priv->next_packet)
     {
@@ -989,9 +988,9 @@ static int do_read(bgav_input_context_t* ctx,
   fd = bgav_rtsp_get_fd(priv->r);
 
   if(block)
-    return bgav_read_data_fd(fd, buffer, len, ctx->opt->read_timeout);
+    return bgav_read_data_fd(ctx->opt, fd, buffer, len, ctx->opt->read_timeout);
   else
-    return bgav_read_data_fd(fd, buffer, len, 0);
+    return bgav_read_data_fd(ctx->opt, fd, buffer, len, 0);
   }
 
 static int read_rtsp(bgav_input_context_t* ctx,
