@@ -45,10 +45,11 @@ static void add_char_16(char ** buffer, int * buffer_alloc,
   *ptr = c;
   }
 
-static int read_line_utf16(bgav_input_context_t * ctx,
-                           int (*read_char)(bgav_input_context_t*,uint16_t*),
-                           char ** buffer, int * buffer_alloc, int buffer_offset,
-                           int * ret_len)
+static int
+read_line_utf16(bgav_input_context_t * ctx,
+                int (*read_char)(bgav_input_context_t*,uint16_t*),
+                char ** buffer, int * buffer_alloc, int buffer_offset,
+                int * ret_len)
   {
   uint16_t c;
   int pos = buffer_offset;
@@ -94,7 +95,8 @@ static void add_char(char ** buffer, int * buffer_alloc,
   }
 
 int bgav_input_read_line(bgav_input_context_t* input,
-                         char ** buffer, int * buffer_alloc, int buffer_offset, int * len)
+                         char ** buffer, int * buffer_alloc,
+                         int buffer_offset, int * len)
   {
   char c;
   int pos = buffer_offset;
@@ -115,7 +117,7 @@ int bgav_input_read_line(bgav_input_context_t* input,
     if(!bgav_input_read_data(input, (uint8_t*)(&c), 1))
       {
       //      return 0;
-      add_char(buffer, buffer_alloc, pos, 0);
+      add_char(buffer, buffer_alloc, pos, '\0');
       return pos - buffer_offset;
       break;
       }
@@ -127,7 +129,7 @@ int bgav_input_read_line(bgav_input_context_t* input,
       pos++;
       }
     }
-  add_char(buffer, buffer_alloc, pos, 0);
+  add_char(buffer, buffer_alloc, pos, '\0');
   if(len)
     *len = pos - buffer_offset;
   return input->position - old_pos;
@@ -138,7 +140,7 @@ int bgav_input_read_convert_line(bgav_input_context_t * input,
                                  int * len)
   {
   int line_alloc = 0;
-  char * line = (char*)0;
+  char * line = NULL;
   int in_len, out_len;
   int64_t old_pos = input->position;
   
@@ -617,12 +619,12 @@ extern const bgav_input_t bgav_input_mmsh;
 
 #ifdef HAVE_CDIO
 extern const bgav_input_t bgav_input_vcd;
+#endif // HAVE_CDIO
 
 #ifdef HAVE_DVDREAD
 extern const bgav_input_t bgav_input_dvd;
 #endif
 
-#endif // HAVE_CDIO
 
 #ifdef HAVE_LINUXDVB
 extern const bgav_input_t bgav_input_dvb;
@@ -688,18 +690,18 @@ static int input_open(bgav_input_context_t * ctx,
   {
   int ret = 0;
   const char * pos;
-  char * protocol = (char*)0;
+  char * protocol = NULL;
   char * tmp_url;
   
   tmp_url = bgav_strdup(url);
   
   if(bgav_url_split(tmp_url,
                     &protocol,
-                    (char**)0, /* User */
-                    (char**)0, /* Pass */
-                    (char**)0,
-                    (int*)0,
-                    (char**)0))
+                    NULL, /* User */
+                    NULL, /* Pass */
+                    NULL,
+                    NULL,
+                    NULL))
     {
     if(!strcasecmp(protocol, "rtsp"))
       ctx->input = &bgav_input_rtsp;
@@ -789,7 +791,7 @@ static int do_open(bgav_input_context_t * ctx,
                     const char *url)
   {
   int ret = 0;
-  char * r = (char*)0;
+  char * r = NULL;
   int i;
   char * tmp_url;
   tmp_url = bgav_strdup(url);
@@ -805,7 +807,7 @@ static int do_open(bgav_input_context_t * ctx,
     free(tmp_url);
     tmp_url = r;
     bgav_log(ctx->opt, BGAV_LOG_INFO, LOG_DOMAIN, "Got redirected to %s", r);
-    r = (char*)0;
+    r = NULL;
     }
   
   free(tmp_url);
@@ -818,23 +820,28 @@ int bgav_input_open(bgav_input_context_t * ctx,
   {
   int ret;
   ret = do_open(ctx, url);
-  if(!ret && ctx->input == &bgav_input_mms)
+  if(!ret)
     {
-    char * new_url;
-    char * pos;
-    if(ctx->priv)
+    if(ctx->input == &bgav_input_mms)
       {
-      ctx->input->close(ctx);
-      ctx->priv = NULL;
+      char * new_url;
+      char * pos;
+      if(ctx->priv)
+        {
+        ctx->input->close(ctx);
+        ctx->priv = NULL;
+        }
+      pos = strstr(url, "://");
+      if(!pos)
+        return 0;
+      new_url = bgav_sprintf("http%s", pos);
+      bgav_log(ctx->opt, BGAV_LOG_INFO, LOG_DOMAIN,
+               "mms connection failed, trying http");
+      ret = do_open(ctx, new_url);
+      free(new_url);
       }
-    pos = strstr(url, "://");
-    if(!pos)
-      return 0;
-    new_url = bgav_sprintf("http%s", pos);
-    bgav_log(ctx->opt, BGAV_LOG_INFO, LOG_DOMAIN, "mms connection failed, trying http");
-    ret = do_open(ctx, new_url);
-    free(new_url);
     }
+  
   return ret;
   }
 
@@ -862,11 +869,13 @@ void bgav_input_close(bgav_input_context_t * ctx)
     free(ctx->charset);
   if(ctx->cnv)
     bgav_charset_converter_destroy(ctx->cnv);
+  if(ctx->disc_name)
+    free(ctx->disc_name);
 
   if(ctx->tt)
     bgav_track_table_unref(ctx->tt);
     
-  bgav_metadata_free(&ctx->metadata);
+  gavl_metadata_free(&ctx->metadata);
   //  free(ctx);
   
   opt = ctx->opt;
@@ -1055,7 +1064,7 @@ int bgav_input_reopen(bgav_input_context_t * ctx)
   {
   gavl_time_t delay_time = GAVL_TIME_SCALE / 2;
   const bgav_input_t * input;
-  char * url = (char*)0;
+  char * url = NULL;
   int ret = 0;
   char * redir;
   const bgav_options_t * opt;
@@ -1065,7 +1074,7 @@ int bgav_input_reopen(bgav_input_context_t * ctx)
     input = ctx->input;
     opt = ctx->opt;
     
-    ctx->url = (char*)0;
+    ctx->url = NULL;
     
     bgav_input_close(ctx);
     
