@@ -388,8 +388,8 @@ static int handle_nal(bgav_video_parser_t * parser)
         {
         /* has_picture_start is also set if the sps was found, so we must check for
            coding_type as well */
-        if(!priv->has_picture_start || !parser->cache[parser->cache_size-1].coding_type)
-          {
+        //        if(!priv->has_picture_start || !parser->cache[parser->cache_size-1].coding_type)
+        //          {
           get_rbsp(parser, parser->buf.buffer + parser->pos + header_len,
                    priv->nal_len - header_len);
                 
@@ -402,31 +402,32 @@ static int handle_nal(bgav_video_parser_t * parser)
           if(!priv->has_picture_start &&
              !bgav_video_parser_set_picture_start(parser))
             return PARSER_ERROR;
-          
-          switch(sh.slice_type)
-            {
-            case 2:
-            case 7:
-              bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_I);
-              break;
-            case 0:
-            case 5:
-              bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_P);
-              break;
-            case 1:
-            case 6:
-              bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_B);
-              break;
-            default: /* Assume the worst */
-              fprintf(stderr, "Unknown slice type %d\n", sh.slice_type);
-              break;
-                    
-            }
 
+          if(!parser->cache[parser->cache_size-1].coding_type)
+            {
+            switch(sh.slice_type)
+              {
+              case 2:
+              case 7:
+                bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_I);
+                break;
+              case 0:
+              case 5:
+                bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_P);
+                break;
+              case 1:
+              case 6:
+                bgav_video_parser_set_coding_type(parser, BGAV_CODING_TYPE_B);
+                break;
+              default: /* Assume the worst */
+                fprintf(stderr, "Unknown slice type %d\n", sh.slice_type);
+                break;
+              }
+            }
           if(sh.field_pic_flag)
             parser->cache[parser->cache_size-1].field_pic = 1;
           
-          }
+          //          }
         priv->has_picture_start = 0;
         }
       else
@@ -467,7 +468,7 @@ static int handle_nal(bgav_video_parser_t * parser)
         get_rbsp(parser,
                  parser->buf.buffer + parser->pos + header_len,
                  priv->nal_len - header_len);
-        bgav_h264_sps_parse(parser->opt,
+        bgav_h264_sps_parse(parser->s->opt,
                             &priv->sps,
                             priv->rbsp, priv->rbsp_len);
         // bgav_h264_sps_dump(&priv->sps);
@@ -476,10 +477,11 @@ static int handle_nal(bgav_video_parser_t * parser)
         priv->sps_buffer = malloc(priv->sps_len);
         memcpy(priv->sps_buffer,
                parser->buf.buffer + parser->pos, priv->sps_len);
-        
-        bgav_video_parser_set_framerate(parser, priv->sps.vui.time_scale,
-                                        priv->sps.vui.num_units_in_tick * 2);
 
+        parser->format->timescale = priv->sps.vui.time_scale;
+        parser->format->frame_duration = priv->sps.vui.num_units_in_tick * 2;
+        bgav_video_parser_set_framerate(parser);
+        
         bgav_h264_sps_get_image_size(&priv->sps,
                                      parser->format);
         parser->s->data.video.max_ref_frames = priv->sps.num_ref_frames;
@@ -544,6 +546,9 @@ static int handle_nal(bgav_video_parser_t * parser)
       break;
     case H264_NAL_FILLER_DATA:
       break;
+    default:
+      fprintf(stderr, "Unknown nh.unit_type %d", nh.unit_type);
+      break;
     }
   
   //        bgav_video_parser_flush(parser, priv->nal_len);
