@@ -61,6 +61,9 @@ typedef struct
   int i_count;
 
   int d10; // Special handling for D10 */
+  
+  int framerate_from_container;
+  
   } mpeg12_priv_t;
 
 static void reset_mpeg12(bgav_video_parser_t * parser)
@@ -137,7 +140,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
           if(priv->have_sh && !priv->sh.has_dpy_ext)
             {
             priv->state = MPEG_HAS_SEQUENCE_DISPLAY_EXT_CODE;
-            fprintf(stderr, "Got sequence display extenstion\n");
+            // fprintf(stderr, "Got sequence display extenstion\n");
             }
           else
             {
@@ -236,14 +239,14 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         parser->s->data.video.format.framerate_mode = GAVL_FRAMERATE_CONSTANT;
       
       /* Try to get the picture header */
-      len = bgav_mpv_picture_header_parse(parser->opt,
+      len = bgav_mpv_picture_header_parse(parser->s->opt,
                                           &ph,
                                           parser->buf.buffer + parser->pos,
                                           parser->buf.size - parser->pos);
       
       if(len < 0)
         {
-        bgav_log(parser->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
+        bgav_log(parser->s->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
                  "Bogus picture header or broken frame");
         return PARSER_ERROR;
         }
@@ -259,7 +262,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
            (!priv->frames_since_sh))
           {
           parser->flags |= PARSER_NO_I_FRAMES;
-          bgav_log(parser->opt, BGAV_LOG_DEBUG, LOG_DOMAIN,
+          bgav_log(parser->s->opt, BGAV_LOG_DEBUG, LOG_DOMAIN,
                    "Detected Intra slice refresh");
           }
         priv->frames_since_sh++;
@@ -278,7 +281,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
       break;
     case MPEG_HAS_PICTURE_EXT_CODE:
       /* Try to get the picture extension */
-      len = bgav_mpv_picture_extension_parse(parser->opt,
+      len = bgav_mpv_picture_extension_parse(parser->s->opt,
                                              &pe,
                                              parser->buf.buffer + parser->pos,
                                              parser->buf.size - parser->pos);
@@ -328,7 +331,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
       priv->state = MPEG_NEED_STARTCODE;
       break;
     case MPEG_HAS_GOP_CODE:
-      len = bgav_mpv_gop_header_parse(parser->opt,
+      len = bgav_mpv_gop_header_parse(parser->s->opt,
                                       &gh,
                                       parser->buf.buffer + parser->pos,
                                       parser->buf.size - parser->pos);
@@ -342,7 +345,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
 
       if(!priv->have_sh)
         {
-        len = bgav_mpv_sequence_header_parse(parser->opt,
+        len = bgav_mpv_sequence_header_parse(parser->s->opt,
                                              &priv->sh,
                                              parser->buf.buffer + parser->pos,
                                              parser->buf.size - parser->pos);
@@ -351,9 +354,10 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         parser->pos += len;
 
         bgav_mpv_get_framerate(priv->sh.frame_rate_index, &timescale, &frame_duration);
-        
-        bgav_video_parser_set_framerate(parser,
-                                        timescale, frame_duration);
+
+        parser->format->timescale = timescale;
+        parser->format->frame_duration = frame_duration;
+        bgav_video_parser_set_framerate(parser);
         
         parser->format->image_width  = priv->sh.horizontal_size_value;
         parser->format->image_height = priv->sh.vertical_size_value;
@@ -361,8 +365,8 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
           (parser->format->image_width + 15) & ~15;
         parser->format->frame_height  =
           (parser->format->image_height + 15) & ~15;
-
         
+        parser->s->codec_bitrate = priv->sh.bitrate * 400;
         
         priv->have_sh = 1;
         }
@@ -383,7 +387,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         
         /* Try to get the sequence extension */
         len =
-          bgav_mpv_sequence_extension_parse(parser->opt,
+          bgav_mpv_sequence_extension_parse(parser->s->opt,
                                             &priv->sh.ext,
                                             parser->buf.buffer + parser->pos,
                                             parser->buf.size - parser->pos);
@@ -392,9 +396,10 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         
         priv->sh.mpeg2 = 1;
 
-        bgav_video_parser_set_framerate(parser,
-                                        parser->format->timescale * (priv->sh.ext.timescale_ext+1) * 2,
-                                        parser->format->frame_duration * (priv->sh.ext.frame_duration_ext+1) * 2);
+        parser->format->timescale *=  (priv->sh.ext.timescale_ext+1) * 2;
+        parser->format->frame_duration *= (priv->sh.ext.frame_duration_ext+1) * 2;
+        
+        bgav_video_parser_set_framerate(parser);
         
         parser->format->image_width  += priv->sh.ext.horizontal_size_ext;
         parser->format->image_height += priv->sh.ext.vertical_size_ext;
@@ -406,6 +411,8 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         
         if(priv->sh.ext.low_delay)
           parser->s->flags &= ~STREAM_B_FRAMES;
+
+        parser->s->codec_bitrate += (priv->sh.ext.bitrate_ext << 18) * 400;
         
         parser->pos += len;
         
@@ -420,7 +427,7 @@ static int parse_mpeg12(bgav_video_parser_t * parser)
         {
         /* Try to get the sequence extension */
         len =
-          bgav_mpv_sequence_display_extension_parse(parser->opt,
+          bgav_mpv_sequence_display_extension_parse(parser->s->opt,
                                                     &priv->sh.dpy_ext,
                                                     parser->buf.buffer + parser->pos,
                                                     parser->buf.size - parser->pos);
@@ -474,18 +481,25 @@ static int parse_frame_mpeg12(bgav_video_parser_t * parser, bgav_packet_t * p)
       case MPEG_CODE_SEQUENCE:
         if(!priv->have_sh)
           {
-          len = bgav_mpv_sequence_header_parse(parser->opt,
+          len = bgav_mpv_sequence_header_parse(parser->s->opt,
                                                &priv->sh,
                                                start, end - start);
           if(!len)
             return PARSER_ERROR;
           priv->have_sh = 1;
+
+          if(!parser->format->timescale)
+            {
+            bgav_mpv_get_framerate(priv->sh.frame_rate_index,
+                                   &timescale, &frame_duration);
           
-          bgav_mpv_get_framerate(priv->sh.frame_rate_index,
-                                 &timescale, &frame_duration);
+            parser->format->timescale = timescale;
+            parser->format->frame_duration = frame_duration;
+            bgav_video_parser_set_framerate(parser);
+            }
+          else
+            priv->framerate_from_container = 1;
           
-          bgav_video_parser_set_framerate(parser,
-                                          timescale, frame_duration);
           start += len;
           }
         else
@@ -495,15 +509,17 @@ static int parse_frame_mpeg12(bgav_video_parser_t * parser, bgav_packet_t * p)
         if(priv->have_sh && !priv->sh.mpeg2)
           {
           len =
-            bgav_mpv_sequence_extension_parse(parser->opt, &priv->sh.ext, start, end - start);
+            bgav_mpv_sequence_extension_parse(parser->s->opt, &priv->sh.ext, start, end - start);
           if(!len)
             return PARSER_ERROR;
           priv->sh.mpeg2 = 1;
 
-          bgav_video_parser_set_framerate(parser,
-                                          parser->format->timescale * (priv->sh.ext.timescale_ext+1) * 2,
-                                          parser->format->frame_duration * (priv->sh.ext.frame_duration_ext+1) * 2);
-          
+          if(!priv->framerate_from_container)
+            {
+            parser->format->timescale *= (priv->sh.ext.timescale_ext+1) * 2;
+            parser->format->frame_duration *= (priv->sh.ext.frame_duration_ext+1) * 2;
+            bgav_video_parser_set_framerate(parser);
+            }
           start += len;
           }
         else
@@ -515,7 +531,7 @@ static int parse_frame_mpeg12(bgav_video_parser_t * parser, bgav_packet_t * p)
           bgav_video_parser_extract_header(parser);
           ret = PARSER_CONTINUE;
           }
-        len = bgav_mpv_picture_header_parse(parser->opt,
+        len = bgav_mpv_picture_header_parse(parser->s->opt,
                                             &ph, start, end - start);
         p->duration = parser->format->frame_duration;
         if(!len)
@@ -547,7 +563,7 @@ static int parse_frame_mpeg12(bgav_video_parser_t * parser, bgav_packet_t * p)
           }
         break;
       case MPEG_CODE_PICTURE_EXT:
-        len = bgav_mpv_picture_extension_parse(parser->opt,
+        len = bgav_mpv_picture_extension_parse(parser->s->opt,
                                                &pe, start, end - start);
         if(!len)
           return PARSER_ERROR;
