@@ -26,7 +26,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
-
+#include <glob.h>
 
 #include <avdec_private.h>
 
@@ -196,7 +196,7 @@ static int read_mpsub(bgav_stream_t * s)
   char * ptr;
   
   ctx = s->data.subtitle.subreader;
-  priv = (mpsub_priv_t*)(ctx->priv);
+  priv = ctx->priv;
     
   while(1)
     {
@@ -286,7 +286,7 @@ static void close_mpsub(bgav_stream_t * s)
   mpsub_priv_t * priv;
   
   ctx = s->data.subtitle.subreader;
-  priv = (mpsub_priv_t *)(ctx->priv);
+  priv = ctx->priv;
   free(priv);
   }
 
@@ -321,7 +321,7 @@ static int init_current_spumux(bgav_stream_t * s)
   bgav_subtitle_reader_context_t * ctx;
   spumux_t * priv;
   ctx = s->data.subtitle.subreader;
-  priv = (spumux_t*)(ctx->priv);
+  priv = ctx->priv;
   
   priv->cur = priv->yml->children;
   while(priv->cur && (!priv->cur->name || strcasecmp(priv->cur->name, "stream")))
@@ -345,7 +345,7 @@ static int advance_current_spumux(bgav_stream_t * s)
   bgav_subtitle_reader_context_t * ctx;
   spumux_t * priv;
   ctx = s->data.subtitle.subreader;
-  priv = (spumux_t*)(ctx->priv);
+  priv = ctx->priv;
 
   priv->cur = priv->cur->next;
   while(priv->cur && (!priv->cur->name || strcasecmp(priv->cur->name, "spu")))
@@ -383,12 +383,12 @@ static int read_spumux(bgav_stream_t * s)
   const char * filename;
   const char * start_time;
   const char * tmp;
-  char * error_msg = (char*)0;
+  char * error_msg = NULL;
   
   bgav_subtitle_reader_context_t * ctx;
   spumux_t * priv;
   ctx = s->data.subtitle.subreader;
-  priv = (spumux_t*)(ctx->priv);
+  priv = ctx->priv;
 
   if(!priv->cur)
     return 0;
@@ -560,7 +560,7 @@ static void seek_spumux(bgav_stream_t * s, int64_t time1, int scale)
   gavl_time_t time = gavl_time_unscale(scale, time1);
   
   ctx = s->data.subtitle.subreader;
-  priv = (spumux_t*)(ctx->priv);
+  priv = ctx->priv;
   init_current_spumux(s);
   
   while(1)
@@ -610,7 +610,7 @@ static void close_spumux(bgav_stream_t * s)
   bgav_subtitle_reader_context_t * ctx;
   spumux_t * priv;
   ctx = s->data.subtitle.subreader;
-  priv = (spumux_t*)(ctx->priv);
+  priv = ctx->priv;
 
   if(priv->yml)
     bgav_yml_free(priv->yml);
@@ -674,7 +674,7 @@ static char const * const extensions[] =
     "srt",
     "sub",
     "xml",
-    (char*)0
+    NULL
   };
 
 static const bgav_subtitle_reader_t * find_subtitle_reader(const char * filename,
@@ -683,15 +683,15 @@ static const bgav_subtitle_reader_t * find_subtitle_reader(const char * filename
   int i;
   bgav_input_context_t * input;
   const char * extension;
-  const bgav_subtitle_reader_t* ret = (const bgav_subtitle_reader_t*)0;
+  const bgav_subtitle_reader_t* ret = NULL;
   
-  char * line = (char*)0;
+  char * line = NULL;
   int line_alloc = 0;
   int line_len;
   /* 1. Check if we have a supported extension */
   extension = strrchr(filename, '.');
   if(!extension)
-    return (bgav_subtitle_reader_t*)0;
+    return NULL;
 
   extension++;
   i = 0;
@@ -702,14 +702,14 @@ static const bgav_subtitle_reader_t * find_subtitle_reader(const char * filename
     i++;
     }
   if(!extensions[i])
-    return (bgav_subtitle_reader_t*)0;
+    return NULL;
 
   /* 2. Open the file and do autodetection */
   input = bgav_input_create(opt);
   if(!bgav_input_open(input, filename))
     {
     bgav_input_destroy(input);
-    return (bgav_subtitle_reader_t*)0;
+    return NULL;
     }
 
   bgav_input_detect_charset(input);
@@ -741,71 +741,56 @@ extern bgav_input_t bgav_input_file;
 bgav_subtitle_reader_context_t *
 bgav_subtitle_reader_open(bgav_input_context_t * input_ctx)
   {
-  char * file, *directory, *pos, *name;
-  int file_len;
-  DIR * dir;
-  struct dirent * res;
+  char *pattern, *pos, *name;
   const bgav_subtitle_reader_t * r;
-  char * subtitle_filename;
-  bgav_subtitle_reader_context_t * ret = (bgav_subtitle_reader_context_t *)0;
-  bgav_subtitle_reader_context_t * end = (bgav_subtitle_reader_context_t *)0;
+  bgav_subtitle_reader_context_t * ret = NULL;
+  bgav_subtitle_reader_context_t * end = NULL;
   bgav_subtitle_reader_context_t *new;
+  glob_t glob_buf;
+  int i;
+  int base_len;
   
   /* Check if input is a regular file */
   if((input_ctx->input != &bgav_input_file) || !input_ctx->filename)
     {
-    return (bgav_subtitle_reader_context_t*)0;
+    return NULL;
     }
 
-  /* Get directory name and open directory */
-  directory = bgav_strdup(input_ctx->filename);
-  pos = strrchr(directory, '/');
+  pattern = bgav_strdup(input_ctx->filename);
+  pos = strrchr(pattern, '.');
   if(!pos)
     {
-    free(directory);
-    return (bgav_subtitle_reader_context_t*)0;
-    }
-  *pos = '\0';
-
-  file = pos + 1;
-  /* Cut off extension */
-  pos = strrchr(file, '.');
-  if(pos)
-    file_len = pos - file;
-  else
-    file_len = strlen(file);
-  dir = opendir(directory);
-  if(!dir)
-    {
-    return (bgav_subtitle_reader_context_t*)0;
+    free(pattern);
+    return NULL;
     }
 
-  while( (res=readdir(dir)) )
+  base_len = pos - pattern;
+  
+  pos[0] = '*';
+  pos[1] = '\0';
+
+  if(glob(pattern, 0, NULL, &glob_buf))
+    return NULL;
+
+  free(pattern);
+  
+  for(i = 0; i < glob_buf.gl_pathc; i++)
     {
-    if(!res)
-      break;
-    
-    /* Check, if the filenames match */
-    if(strncasecmp(res->d_name, file, file_len) ||
-       !strcmp(res->d_name, file))
+    if(!strcmp(glob_buf.gl_pathv[i], input_ctx->filename))
       continue;
-    
-    subtitle_filename = bgav_sprintf("%s/%s", directory, res->d_name);
-    
-    r = find_subtitle_reader(subtitle_filename, input_ctx->opt);
+    //    fprintf(stderr, "Found %s\n", glob_buf.gl_pathv[i]);
+
+    r = find_subtitle_reader(glob_buf.gl_pathv[i], input_ctx->opt);
     if(!r)
-      {
-      free(subtitle_filename);
       continue;
-      }
+    
     new = calloc(1, sizeof(*new));
-    new->filename = subtitle_filename;
+    new->filename = bgav_strdup(glob_buf.gl_pathv[i]);
     new->input    = bgav_input_create(input_ctx->opt);
     new->reader   = r;
     new->p = bgav_packet_create();
-    
-    name = res->d_name + file_len;
 
+    name = glob_buf.gl_pathv[i] + base_len;
     
     while(!isalnum(*name) && (*name != '\0'))
       name++;
@@ -815,7 +800,9 @@ bgav_subtitle_reader_open(bgav_input_context_t * input_ctx)
       pos = strrchr(name, '.');
       new->info = bgav_strndup(name, pos);
       }
-
+    
+    /* Apped to list */
+    
     if(!ret)
       {
       ret = new;
@@ -827,8 +814,7 @@ bgav_subtitle_reader_open(bgav_input_context_t * input_ctx)
       end = end->next;
       }
     }
-  closedir(dir);
-  free(directory);
+  globfree(&glob_buf);
   return ret;
   }
 
@@ -843,7 +829,7 @@ void bgav_subtitle_reader_stop(bgav_stream_t * s)
   if(ctx->ovl.frame)
     {
     gavl_video_frame_destroy(ctx->ovl.frame);
-    ctx->ovl.frame = (gavl_video_frame_t*)0;
+    ctx->ovl.frame = NULL;
     }
   
   if(ctx->input)
@@ -892,7 +878,7 @@ bgav_packet_t * bgav_subtitle_reader_read_text(bgav_stream_t * s)
     return s->data.subtitle.subreader->p;
     }
   else
-    return (bgav_packet_t *)0;
+    return NULL;
   }
 
 int bgav_subtitle_reader_start(bgav_stream_t * s)
