@@ -40,7 +40,7 @@ bgav_track_add_audio_stream(bgav_track_t * t, const bgav_options_t * opt)
   
   bgav_stream_init(ret, opt);
   bgav_stream_create_packet_buffer(ret);
-  ret->data.audio.bits_per_sample = 16;
+  // ret->data.audio.bits_per_sample = 16;
   ret->type = BGAV_STREAM_AUDIO;
   ret->track = t;
   
@@ -110,7 +110,7 @@ bgav_track_add_subtitle_stream(bgav_track_t * t, const bgav_options_t * opt,
                              opt,
                              text,
                              encoding,
-                             (bgav_subtitle_reader_context_t*)0);
+                             NULL);
   }
 
 bgav_stream_t *
@@ -121,9 +121,9 @@ bgav_track_attach_subtitle_reader(bgav_track_t * t,
   bgav_stream_t * ret;
   ret = add_subtitle_stream(t, opt,
                             r->reader->read_subtitle_text ? 1 : 0,
-                            (char*)0, r);
+                            NULL, r);
   if(r->info)
-    ret->info = bgav_strdup(r->info);
+    gavl_metadata_set(&ret->m, GAVL_META_LABEL, r->info);
   return ret;
   }
 
@@ -147,7 +147,7 @@ bgav_track_find_stream_all(bgav_track_t * t, int stream_id)
        (!t->subtitle_streams[i].data.subtitle.subreader))
       return &t->subtitle_streams[i];
     }
-  return (bgav_stream_t *)0;
+  return NULL;
   }
 
 bgav_stream_t * bgav_track_find_stream(bgav_demuxer_context_t * ctx,
@@ -250,10 +250,12 @@ int bgav_track_start(bgav_track_t * t, bgav_demuxer_context_t * demuxer)
     if(t->subtitle_streams[i].action == BGAV_STREAM_MUTE)
       continue;
     num_active_subtitle_streams++;
-    if(!t->subtitle_streams[i].data.subtitle.video_stream)
+    if(!t->subtitle_streams[i].data.subtitle.video_stream &&
+       t->num_video_streams)
+      {
       t->subtitle_streams[i].data.subtitle.video_stream =
         t->video_streams;
-
+      }
     if(!t->subtitle_streams[i].data.subtitle.video_stream)
       {
       bgav_log(demuxer->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
@@ -278,7 +280,7 @@ int bgav_track_start(bgav_track_t * t, bgav_demuxer_context_t * demuxer)
       }
     
     if((!video_format->image_width || !video_format->image_height ||
-        !video_format->timescale || !video_format->frame_duration) &&
+        !video_format->timescale) &&
        (video_stream->action != BGAV_STREAM_PARSE))
       {
       bgav_log(demuxer->opt, BGAV_LOG_ERROR, LOG_DOMAIN,
@@ -339,8 +341,8 @@ void bgav_track_dump(bgav_t * b, bgav_track_t * t)
   else
     bgav_dprintf( "Not specified (maybe live)\n");
 
-  
-  bgav_metadata_dump(&t->metadata);
+  bgav_diprintf(2, "Metadata\n");
+  gavl_metadata_dump(&t->metadata, 4);
 
   if(t->chapter_list)
     bgav_chapter_list_dump(t->chapter_list);
@@ -367,7 +369,7 @@ void bgav_track_free(bgav_track_t * t)
   {
   int i;
   
-  bgav_metadata_free(&t->metadata);
+  gavl_metadata_free(&t->metadata);
   if(t->chapter_list)
     bgav_chapter_list_destroy(t->chapter_list);
   
@@ -413,6 +415,15 @@ void bgav_track_remove_audio_stream(bgav_track_t * track, int stream)
 
 void bgav_track_remove_video_stream(bgav_track_t * track, int stream)
   {
+  /* Remove this stream from the subtitle streams as well */
+  int i;
+  for(i = 0; i < track->num_subtitle_streams; i++)
+    {
+    if(track->subtitle_streams[i].data.subtitle.video_stream ==
+       &track->video_streams[stream])
+      track->subtitle_streams[i].data.subtitle.video_stream = NULL;
+    }
+  
   remove_stream(track->video_streams, stream, track->num_video_streams);
   track->num_video_streams--;
   }
@@ -461,8 +472,6 @@ void bgav_track_remove_unsupported(bgav_track_t * track)
     s = &track->video_streams[i];
     if(!bgav_find_video_decoder(s))
       {
-      bgav_track_remove_video_stream(track, i);
-      
       bgav_log(s->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
                "No video decoder found for fourcc %c%c%c%c (0x%08x)",
                (s->fourcc & 0xFF000000) >> 24,
@@ -470,11 +479,11 @@ void bgav_track_remove_unsupported(bgav_track_t * track)
                (s->fourcc & 0x0000FF00) >> 8,
                (s->fourcc & 0x000000FF),
                s->fourcc);
+      bgav_track_remove_video_stream(track, i);
       }
     else if((s->flags & (STREAM_PARSE_FULL|STREAM_PARSE_FRAME)) &&
        !bgav_video_parser_supported(s->fourcc))
       {
-      bgav_track_remove_video_stream(track, i);
       bgav_log(s->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
                "No parser found for fourcc %c%c%c%c (0x%08x)",
                (s->fourcc & 0xFF000000) >> 24,
@@ -482,6 +491,7 @@ void bgav_track_remove_unsupported(bgav_track_t * track)
                (s->fourcc & 0x0000FF00) >> 8,
                (s->fourcc & 0x000000FF),
                s->fourcc);
+      bgav_track_remove_video_stream(track, i);
       }
     else
       i++;
@@ -806,7 +816,7 @@ void bgav_track_get_compression(bgav_track_t * t)
     if(vdec && !vdec->get_format)
       vdec = NULL;
     
-    if((s->flags & (STREAM_PARSE_FULL|STREAM_PARSE_FRAME)) || vdec)
+    if((s->flags & (STREAM_PARSE_FULL|STREAM_PARSE_FRAME|STREAM_NEED_FRAMETYPES)) || vdec)
       {
       bgav_stream_start(s);
       p = bgav_stream_peek_packet_read(s, 1);
