@@ -42,6 +42,13 @@ const gavl_video_format_t * bgav_get_video_format(bgav_t * bgav, int stream)
   return &bgav->tt->cur->video_streams[stream].data.video.format;
   }
 
+const bgav_metadata_t *
+bgav_get_video_metadata(bgav_t * b, int s)
+  {
+  return &b->tt->cur->video_streams[s].m;
+  }
+
+
 int bgav_set_video_stream(bgav_t * b, int stream, bgav_stream_action_t action)
   {
   if((stream >= b->tt->cur->num_video_streams) ||
@@ -86,6 +93,18 @@ int bgav_video_start(bgav_stream_t * s)
       }
     s->index_mode = INDEX_MODE_SIMPLE;
     }
+  /* Frametype detector */
+  if((s->flags & STREAM_NEED_FRAMETYPES) && !s->fd)
+    {
+    s->fd = bgav_frametype_detector_create(s);
+    if(!bgav_stream_peek_packet_read(s, 1))
+      {
+      bgav_log(s->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+               "EOF while initializing frametype detector");
+      return 0;
+      }
+    }
+  
   /* Packet timer */
   if((s->flags & (STREAM_NO_DURATIONS|STREAM_WRONG_B_TIMESTAMPS)) &&
      !s->pt)
@@ -159,7 +178,8 @@ int bgav_video_start(bgav_stream_t * s)
 
 const char * bgav_get_video_description(bgav_t * b, int s)
   {
-  return b->tt->cur->video_streams[s].description;
+  return gavl_metadata_get(&b->tt->cur->video_streams[s].m,
+                           GAVL_META_FORMAT);
   }
 
 
@@ -227,6 +247,11 @@ void bgav_video_stop(bgav_stream_t * s)
     bgav_packet_timer_destroy(s->pt);
     s->pt = NULL;
     }
+  if(s->fd)
+    {
+    bgav_frametype_detector_destroy(s->fd);
+    s->fd = NULL;
+    }
   if(s->data.video.ft)
     {
     bgav_video_format_tracker_destroy(s->data.video.ft);
@@ -267,10 +292,10 @@ void bgav_video_resync(bgav_stream_t * s)
     }
 
   if(s->pt)
-    {
     bgav_packet_timer_reset(s->pt);
-    }
-
+  if(s->fd)
+    bgav_frametype_detector_reset(s->fd);
+  
   /* If the stream has keyframes, skip until the next one */
 
   if(!(s->flags & (STREAM_INTRA_ONLY|STREAM_STILL_MODE)))
@@ -429,7 +454,7 @@ int bgav_video_skipto(bgav_stream_t * s, int64_t * time, int scale,
         s->out_time = p->pts;
         return 1;
         }
-      result = bgav_video_decode(s, (gavl_video_frame_t*)0);
+      result = bgav_video_decode(s, NULL);
       if(!result)
         {
         s->out_time = BGAV_TIMESTAMP_UNDEFINED;
@@ -792,18 +817,16 @@ int bgav_get_video_compression_info(bgav_t * bgav, int stream,
                                     &info->global_header_len);
     }
 
-  if(need_bitrate)
+  if(s->codec_bitrate)
+    info->bitrate = s->codec_bitrate;
+  else if(s->container_bitrate)
+    info->bitrate = s->container_bitrate;
+  
+  if(need_bitrate && !info->bitrate)
     {
-    if(s->codec_bitrate)
-      info->bitrate = s->codec_bitrate;
-    else if(s->container_bitrate)
-      info->bitrate = s->container_bitrate;
-    else
-      {
-      bgav_log(&bgav->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
-               "Video compression format needs bitrate for compressed output");
-      return 0;
-      }
+    bgav_log(&bgav->opt, BGAV_LOG_WARNING, LOG_DOMAIN,
+             "Video compression format needs bitrate for compressed output");
+    return 0;
     }
   
   if(!(s->flags & STREAM_INTRA_ONLY))
