@@ -35,17 +35,14 @@
 
 #define LOG_DOMAIN "video_libmpeg2"
 
-// #define DUMP_TIMESTAMS
-
-// #define DUMP_SEQUENCE_HEADER
-
-// #define DUMP_PACKETS
+//#define DUMP_SEQUENCE_HEADER
+//#define DUMP_PACKETS
 
 static const char picture_types[] = { "?IPB????" };
 
 /* Debug function */
 #ifdef DUMP_SEQUENCE_HEADER
-void dump_sequence_header(const mpeg2_sequence_t * s)
+static void dump_sequence_header(const mpeg2_sequence_t * s)
   {
   bgav_dprintf("Sequence header:\n");
   bgav_dprintf("size:         %d x %d\n", s->width, s->height);
@@ -100,7 +97,7 @@ static int get_data(bgav_stream_t*s)
   {
   mpeg2_priv_t * priv;
   int cache_index;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   if(priv->p)
     {
@@ -127,9 +124,6 @@ static int get_data(bgav_stream_t*s)
     else
       return 0;
     }
-#ifdef DUMP_TIMESTAMS
-  bgav_dprintf("Packet timestamp: %"PRId64"\n", priv->p->pts);
-#endif
 
   if(priv->flags & FLAG_NEED_SEQUENCE)
     {
@@ -146,7 +140,7 @@ static int get_data(bgav_stream_t*s)
     }
   
   priv->p = bgav_stream_get_packet_read(s);
-
+  
 #ifdef DUMP_PACKETS
   fprintf(stderr, "Got packet ");
   bgav_packet_dump(priv->p);
@@ -162,7 +156,7 @@ static int get_data(bgav_stream_t*s)
                       priv->p->pts, priv->p->duration,
                       priv->p->tc,
                       &cache_index,
-                      (bgav_pts_cache_entry_t**)0);
+                      NULL);
   
   mpeg2_tag_picture(priv->dec, cache_index, 0);
   
@@ -175,9 +169,9 @@ static void get_format(bgav_stream_t*s,
   {
   mpeg2_priv_t * priv;
   int container_time;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
-  container_time = (ret->timescale > 0) && (ret->frame_duration > 0);
+  container_time = (ret->timescale > 0);
 
   if(!container_time)
     {
@@ -279,7 +273,6 @@ static void get_format(bgav_stream_t*s,
       {
       ret->timescale /= 2;
       ret->frame_duration /= 2;
-      s->data.video.frametime_mode = BGAV_FRAMETIME_CONSTANT;
       }
     }
 #ifdef DUMP_SEQUENCE_HEADER
@@ -301,7 +294,7 @@ static void get_format(bgav_stream_t*s,
 static int parse(bgav_stream_t*s, mpeg2_state_t * state)
   {
   mpeg2_priv_t * priv;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   while(1)
     {
@@ -316,7 +309,8 @@ static int parse(bgav_stream_t*s, mpeg2_state_t * state)
       continue;
       }
     
-    if((*state == STATE_GOP) && !(s->flags & FLAG_EXTERN_TIMECODES))
+    if((*state == STATE_GOP) && !(s->flags & FLAG_EXTERN_TIMECODES) &&
+       s->data.video.format.frame_duration)
       {
       if(!s->data.video.format.timecode_format.int_framerate)
         {
@@ -349,7 +343,7 @@ static int decode_picture(bgav_stream_t*s)
   int cache_index;
   mpeg2_priv_t * priv;
   mpeg2_state_t state;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
   done = 0;
   while(1)
     {
@@ -464,7 +458,7 @@ static int decode_mpeg2(bgav_stream_t*s, gavl_video_frame_t*f)
   {
   mpeg2_priv_t * priv;
 
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
   
   /* Decode frame */
   
@@ -553,11 +547,19 @@ static int init_mpeg2(bgav_stream_t*s)
   priv->frame->strides[0] = priv->info->sequence->width;
   priv->frame->strides[1] = priv->info->sequence->chroma_width;
   priv->frame->strides[2] = priv->info->sequence->chroma_width;
-  
+
+
+  gavl_metadata_set_nocpy(&s->m, GAVL_META_FORMAT,
+                          bgav_sprintf("MPEG-%d",
+                                       (priv->info->sequence->flags & SEQ_FLAG_MPEG2) ? 2 : 1));
+
+#if 0
   s->description =
     bgav_sprintf("MPEG-%d, %d kb/sec", 
                  (priv->info->sequence->flags & SEQ_FLAG_MPEG2) ? 2 : 1,
                  (priv->info->sequence->byte_rate * 8) / 1000);
+#endif
+  
   s->codec_bitrate = priv->info->sequence->byte_rate * 8;
   priv->flags |= FLAG_NEED_SEQUENCE;
   
@@ -575,7 +577,7 @@ static int init_mpeg2(bgav_stream_t*s)
   /* Decode first frame to check for still mode */
   
   priv->flags |= FLAG_INIT;
-  decode_mpeg2(s, (gavl_video_frame_t*)0);
+  decode_mpeg2(s, NULL);
   priv->flags &= ~FLAG_INIT;
   s->out_time = priv->picture_timestamp;
   return 1;
@@ -585,7 +587,7 @@ static void resync_mpeg2(bgav_stream_t*s)
   {
   mpeg2_priv_t * priv;
   bgav_packet_t * p;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   if(priv->p)
     {
@@ -630,7 +632,7 @@ static void resync_mpeg2(bgav_stream_t*s)
 static void close_mpeg2(bgav_stream_t*s)
   {
   mpeg2_priv_t * priv;
-  priv = (mpeg2_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   if(priv->p)
     bgav_packet_pool_put(s->pp, priv->p);
