@@ -56,7 +56,7 @@ static void decode_yuv2(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   int i, j;
   uint8_t * src, *dst_y, *dst_u, *dst_v;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   
@@ -86,9 +86,11 @@ static int init_yuv2(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("Full scale YUV 4:2:2 packed (yuv2)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "Full scale YUV 4:2:2 packed (yuv2)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width * 2, 4);
   priv->decode_func = decode_yuv2;
@@ -140,7 +142,7 @@ static void decode_v408(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   int i, j;
   uint8_t * src, *dst;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   
@@ -166,9 +168,11 @@ static int init_v408(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUVA 4:4:4:4 (v408)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUVA 4:4:4:4 (v408)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = s->data.video.format.image_width * 4;
   priv->decode_func = decode_v408;
@@ -186,7 +190,7 @@ static int init_v408(bgav_stream_t * s)
 static void decode_2vuy(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   gavl_video_frame_copy(&s->data.video.format,
@@ -198,9 +202,11 @@ static int init_2vuy(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:2 packed (2vuy)");
-
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:2 packed (2vuy)");
+  
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width * 2, 4);
   priv->decode_func = decode_2vuy;
@@ -216,7 +222,7 @@ static int init_2vuy(bgav_stream_t * s)
 static void decode_VYUY(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   gavl_video_frame_copy(&s->data.video.format,
@@ -228,9 +234,11 @@ static int init_VYUY(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:2 packed (VYUY)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:2 packed (VYUY)");
+  
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width * 2, 4);
   priv->decode_func = decode_VYUY;
@@ -243,7 +251,7 @@ static int init_VYUY(bgav_stream_t * s)
 static void decode_yv12(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   priv->frame->planes[1] = priv->frame->planes[0] + s->data.video.format.image_height * priv->frame->strides[0];
@@ -256,9 +264,11 @@ static int init_yv12(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:0 planar (yv12)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:0 planar (yv12)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width, 2);
   priv->frame->strides[1] = priv->frame->strides[0]/2;
@@ -272,7 +282,7 @@ static int init_yv12(bgav_stream_t * s)
 static void decode_YV12(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   priv->frame->planes[2] = priv->frame->planes[0] + s->data.video.format.image_height * priv->frame->strides[0];
@@ -287,9 +297,11 @@ static int init_YV12(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:0 planar (YV12)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:0 planar (YV12)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width, 2);
   priv->frame->strides[1] = priv->frame->strides[0]/2;
@@ -307,7 +319,7 @@ static int init_YV12(bgav_stream_t * s)
 static void decode_YVU9(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   priv->frame->planes[2] = priv->frame->planes[0] + s->data.video.format.image_height * priv->frame->strides[0];
@@ -323,9 +335,11 @@ static int init_YVU9(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YVU9");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YVU9");
+  
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width, 8);
   priv->frame->strides[1] = priv->frame->strides[0]/4;
@@ -344,7 +358,7 @@ static void decode_v308(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   int i, j;
   uint8_t * src, *dst_y, *dst_u, *dst_v;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   
@@ -376,9 +390,11 @@ static int init_v308(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:4:4 packed (v308)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:4:4 packed (v308)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = s->data.video.format.image_width * 3;
   priv->decode_func = decode_v308;
@@ -398,7 +414,7 @@ static void decode_v410(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   uint16_t *dst_y, *dst_u, *dst_v;
   uint32_t src_i;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   
@@ -428,9 +444,11 @@ static int init_v410(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:4:4 packed (v410)");
+  
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:4:4 packed (v410)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = s->data.video.format.image_width * 4;
   priv->decode_func = decode_v410;
@@ -450,7 +468,7 @@ static void decode_v210(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   uint16_t *dst_y, *dst_u, *dst_v;
   uint32_t i1, i2, i3, i4;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
   
@@ -516,9 +534,11 @@ static int init_v210(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:2 packed (v210)");
 
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:2 packed (v210)");
+
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = (PADD(s->data.video.format.image_width, 48) * 8) / 3;
   priv->decode_func = decode_v210;
@@ -538,7 +558,7 @@ static void decode_yuv4(bgav_stream_t * s, bgav_packet_t * p, gavl_video_frame_t
   int i, j;
   uint8_t * src, *dst_y, *dst_u, *dst_v;
   yuv_priv_t * priv;
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->planes[0] = p->data;
 
@@ -575,10 +595,12 @@ static int init_yuv4(bgav_stream_t * s)
   yuv_priv_t * priv;
   
   init_common(s);
-  s->description = bgav_sprintf("YUV 4:2:0 packed (yuv4)");
+
+  gavl_metadata_set(&s->m, GAVL_META_FORMAT,
+                    "YUV 4:2:0 packed (yuv4)");
   s->flags |= STREAM_INTRA_ONLY;
   
-  priv = (yuv_priv_t *)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   priv->frame->strides[0] = PADD(s->data.video.format.image_width, 2) * 3;
   priv->decode_func = decode_yuv4;
@@ -593,7 +615,7 @@ static int decode(bgav_stream_t * s, gavl_video_frame_t * f)
   {
   yuv_priv_t * priv;
   bgav_packet_t * p;
-  priv = (yuv_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
   
   /* We assume one frame per packet */
   
@@ -628,7 +650,7 @@ static int decode(bgav_stream_t * s, gavl_video_frame_t * f)
 static void close(bgav_stream_t * s)
   {
   yuv_priv_t * priv;
-  priv = (yuv_priv_t*)(s->data.video.decoder->priv);
+  priv = s->data.video.decoder->priv;
 
   gavl_video_frame_null(priv->frame);
   gavl_video_frame_destroy(priv->frame);
