@@ -87,6 +87,7 @@ parsers[] =
     { BGAV_MK_FOURCC('C', 'A', 'V', 'S'), bgav_video_parser_init_cavs },
     { BGAV_MK_FOURCC('V', 'C', '-', '1'), bgav_video_parser_init_vc1 },
     { BGAV_MK_FOURCC('d', 'r', 'a', 'c'), bgav_video_parser_init_dirac },
+    { BGAV_MK_FOURCC('m', 'j', 'p', 'a'), bgav_video_parser_init_mjpa },
   };
 
 int bgav_video_parser_supported(uint32_t fourcc)
@@ -143,7 +144,6 @@ bgav_video_parser_create(bgav_stream_t * s)
     s->src.peek_func = bgav_video_parser_peek_packet_parse_frame;
     }
   s->src.data = ret;
-
   
   func(ret);
   return ret;
@@ -584,28 +584,16 @@ void bgav_video_parser_get_packet(bgav_video_parser_t * parser,
   
   }
 
-void bgav_video_parser_set_framerate(bgav_video_parser_t * parser,
-                                     int timescale, int frame_duration)
+
+
+void bgav_video_parser_set_framerate(bgav_video_parser_t * parser)
   {
   int i;
-
-  if(!parser->format->timescale || !parser->format->frame_duration)
-    {
-    parser->format->timescale = timescale;
-    parser->format->frame_duration = frame_duration;
-    
-    /*
-     *  Frame duration is set by bgav_video_parser_set_picture_start(),
-     *  which will be before the global header for most formats (i.e.
-     *  when frame duration isn't known yet).
-     */
-    
-    for(i = 0; i < parser->cache_size ; i++)
-      parser->cache[i].duration = parser->format->frame_duration;
-    }
-
+  for(i = 0; i < parser->cache_size ; i++)
+    parser->cache[i].duration = parser->format->frame_duration;
+  
   if(!parser->s->timescale)
-    parser->s->timescale = timescale;
+    parser->s->timescale = parser->format->timescale;
   
   }
 
