@@ -29,6 +29,7 @@
 #include <avdec.h>
 #include "avdec_common.h"
 
+#if 0
 static void convert_metadata(bg_metadata_t * dst,
                              const bgav_metadata_t * m)
   {
@@ -43,11 +44,11 @@ static void convert_metadata(bg_metadata_t * dst,
   if(str)
     dst->artist =
       bg_strdup(dst->artist, str);
-
-  str = bgav_metadata_get_artist(m);
+  
+  str = bgav_metadata_get_albumartist(m);
   if(str)
-    dst->artist =
-      bg_strdup(dst->artist, str);
+    dst->albumartist =
+      bg_strdup(dst->albumartist, str);
 
   str = bgav_metadata_get_album(m);
   if(str)
@@ -80,6 +81,7 @@ static void convert_metadata(bg_metadata_t * dst,
     bgav_metadata_get_track(m);
   
   }
+#endif
 
 static void log_callback(void*data, bgav_log_level_t level,
                          const char * log_domain,
@@ -113,7 +115,7 @@ void * bg_avdec_create()
   {
   avdec_priv * ret = calloc(1, sizeof(*ret));
   ret->opt = bgav_options_create();
-  bgav_options_set_log_callback(ret->opt, log_callback, (void*)0);
+  bgav_options_set_log_callback(ret->opt, log_callback, NULL);
   return ret;
   }
 
@@ -125,14 +127,14 @@ void bg_avdec_close(void * priv)
   if(avdec->dec)
     {
     bgav_close(avdec->dec);
-    avdec->dec = (bgav_t*)0;
+    avdec->dec = NULL;
     }
   if(avdec->track_info)
     {
     for(i = 0; i < avdec->num_tracks; i++)
       bg_track_info_free(&(avdec->track_info[i]));
     free(avdec->track_info);
-    avdec->track_info = (bg_track_info_t*)0;
+    avdec->track_info = NULL;
     }
   }
 
@@ -187,7 +189,7 @@ bg_track_info_t * bg_avdec_get_track_info(void * priv, int track)
   avdec_priv * avdec;
   avdec = (avdec_priv*)(priv);
   if((track < 0) || (track >= avdec->num_tracks))
-    return (bg_track_info_t*)0;
+    return NULL;
   return &(avdec->track_info[track]);
   }
 
@@ -316,7 +318,6 @@ int bg_avdec_set_subtitle_stream(void * priv,
 int bg_avdec_start(void * priv)
   {
   int i;
-  const char * str;
   avdec_priv * avdec;
   const gavl_video_format_t * format;
   avdec = (avdec_priv*)(priv);
@@ -330,9 +331,9 @@ int bg_avdec_start(void * priv)
     gavl_video_format_copy(&(avdec->current_track->video_streams[i].format),
                            bgav_get_video_format(avdec->dec, i));
 
-    str = bgav_get_video_description(avdec->dec, i);
-    if(str)
-      avdec->current_track->video_streams[i].description = bg_strdup(NULL, str);
+    gavl_metadata_copy(&avdec->current_track->video_streams[i].m,
+                       bgav_get_video_metadata(avdec->dec, i));
+    
     avdec->current_track->video_streams[i].duration =
       bgav_video_duration(avdec->dec, i);
     
@@ -341,31 +342,18 @@ int bg_avdec_start(void * priv)
     {
     gavl_audio_format_copy(&(avdec->current_track->audio_streams[i].format),
                            bgav_get_audio_format(avdec->dec, i));
+
+    gavl_metadata_copy(&avdec->current_track->audio_streams[i].m,
+                       bgav_get_audio_metadata(avdec->dec, i));
     
-    str = bgav_get_audio_description(avdec->dec, i);
-    if(str)
-      avdec->current_track->audio_streams[i].description = bg_strdup(NULL, str);
-    str = bgav_get_audio_info(avdec->dec, i);
-    if(str)
-      avdec->current_track->audio_streams[i].info = bg_strdup(NULL, str);
-
-    str = bgav_get_audio_language(avdec->dec, i);
-    if(str && *str)
-      memcpy(avdec->current_track->audio_streams[i].language, str, 4);
-
     avdec->current_track->audio_streams[i].duration =
       bgav_audio_duration(avdec->dec, i);
     }
   for(i = 0; i < avdec->current_track->num_subtitle_streams; i++)
     {
-    str = bgav_get_subtitle_language(avdec->dec, i);
-    if(str && *str)
-      memcpy(avdec->current_track->subtitle_streams[i].language, str, 4);
+    gavl_metadata_copy(&avdec->current_track->subtitle_streams[i].m,
+                       bgav_get_subtitle_metadata(avdec->dec, i));
     
-    str = bgav_get_subtitle_info(avdec->dec, i);
-    if(str)
-      avdec->current_track->subtitle_streams[i].info = bg_strdup(NULL, str);
-
     if(bgav_subtitle_is_text(avdec->dec, i))
       {
       avdec->current_track->subtitle_streams[i].is_text = 1;
@@ -442,7 +430,8 @@ int bg_avdec_init(avdec_priv * avdec)
     /* Get metadata */
     
     m = bgav_get_metadata(avdec->dec, i);
-    convert_metadata(&(avdec->track_info[i].metadata), m);
+    
+    gavl_metadata_copy(&avdec->track_info[i].metadata, m);
 
     /* Get chapters */
 
@@ -595,6 +584,10 @@ bg_avdec_set_parameter(void * p, const char * name,
     {
     bgav_options_set_vdpau(avdec->opt, val->val_i);
     }
+  else if(!strcmp(name, "threads"))
+    {
+    bgav_options_set_threads(avdec->opt, val->val_i);
+    }
   }
 
 int bg_avdec_get_num_tracks(void * p)
@@ -609,7 +602,6 @@ int bg_avdec_get_num_tracks(void * p)
 int bg_avdec_set_track(void * priv, int track)
   {
   int i;
-  const char * str;
   avdec_priv * avdec;
   avdec = (avdec_priv*)(priv);
   
@@ -617,10 +609,6 @@ int bg_avdec_set_track(void * priv, int track)
     return 0;
   avdec->current_track = &(avdec->track_info[track]);
   
-  str = bgav_get_description(avdec->dec);
-  if(str)
-    avdec->track_info[track].description =
-      bg_strdup(avdec->track_info[track].description, str);
 
   /* Get formats (need them for compressed output */
   for(i = 0; i < avdec->current_track->num_audio_streams; i++)
@@ -644,21 +632,15 @@ gavl_frame_table_t * bg_avdec_get_frame_table(void * priv, int stream)
 
 
 static void metadata_change_callback(void * priv,
-                                     const bgav_metadata_t * metadata)
+                                     const gavl_metadata_t * metadata)
   {
-  bg_metadata_t m;
   avdec_priv * avdec;
-  avdec = (avdec_priv*)(priv);
-
+  avdec = priv;
   
   if(avdec->bg_callbacks && avdec->bg_callbacks->metadata_changed)
     {
-    memset(&m, 0, sizeof(m));
-    convert_metadata(&m, metadata);
-
     avdec->bg_callbacks->metadata_changed(avdec->bg_callbacks->data,
-                                          &m);
-    bg_metadata_free(&m);
+                                          metadata);
     }
   }
 
@@ -707,7 +689,7 @@ void bg_avdec_set_callbacks(void * priv,
 bg_device_info_t * bg_avdec_get_devices(bgav_device_info_t * info)
   {
   int i = 0;
-  bg_device_info_t * ret = (bg_device_info_t *)0;
+  bg_device_info_t * ret = NULL;
 
   if(!info)
     return ret;
@@ -728,7 +710,7 @@ const char * bg_avdec_get_disc_name(void * priv)
   avdec = (avdec_priv*)(priv);
   if(avdec->dec)
     return bgav_get_disc_name(avdec->dec);
-  return (const char *)0;
+  return NULL;
   }
 
 static bg_edl_segment_t * copy_segments(const bgav_edl_segment_t * src, int len)
@@ -778,6 +760,12 @@ static bg_edl_track_t * copy_tracks(const bgav_edl_track_t * src, int len)
   
   for(i = 0; i < len; i++)
     {
+    /* Metadata */
+    if(src[i].metadata)
+      gavl_metadata_copy(&ret[i].metadata, src[i].metadata);
+    
+    ret[i].name = bg_strdup(ret[i].name, src[i].name);
+    
     /* Copy pointers */
     ret[i].audio_streams = copy_streams(src[i].audio_streams,
                                         src[i].num_audio_streams);
